      subroutine arrange_dm(dm1,temp,nstate,n_active,ndm_size,nft)


      implicit real*8 (a-h,o-z)

c
c This subroutine arranges the density matrices (DMs) 
c (1) Read data of DMs (one size=ndm_size**2) from the scrathc file
c     (FT21), and rearrange the exact size (n_active**2)
c (2) Make state averaged DM 
c (3) return all re-sized DMs to dm1
c


      dimension dm1(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension temp(ndm_size,ndm_size) 


      L1=nstate*(nstate+1)/2
      L2=ndm_size*ndm_size


c
c Read data of DM1 and arrange data size
c

      rewind(nft)
      
      do i=1,L1
        call rdfuldia(nft,L2,temp)
        do j=1,n_active
          do k=1,n_active
            dm1(k,j,i)=temp(k,j)
          end do
        end do
      end do

c
c Make SA-DM1
c

      do i=1,n_active
        do j=1,n_active
          dm1(j,i,L1+1)=0.0d0  ! zero clear
        end do
      end do

      ist_dm=1 ! index of DM1 of the istat-th state
      do istat=1,nstate
        do i=1,n_active
          do j=1,n_active
            dm1(j,i,L1+1)=dm1(j,i,L1+1)+dm1(j,i,ist_dm)
          end do
        end do
        ist_dm=ist_dm+nstate-istat+1 ! to the next state (skip t-DM1)
      end do

      do i=1,n_active
        do j=1,n_active
          dm1(j,i,L1+1)=dm1(j,i,L1+1)/dble(nstate)
        end do
      end do


      return 

      end




      subroutine caldel_maxD3(nstate,n_active,dm1,
     &           alpha_n,alpha_r,alpha_t,jac1,jac2,a_ij,b_ij)
  
      implicit real*8 (a-h,o-z)

      data zero /0.0d0/
      data one  /1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/
    
      dimension dm1(n_active,n_active,nstate*(nstate+1)/2+1) 

      L1= nstate*(nstate+1)/2+1

c
c Calculate Ruedenberg term
c

      ar_ij=zero
      br_ij=zero
      kk=1      ! kk is index of dm1 between kstat-kstat
      do kstat=1,nstate
        d_ii=dm1(jac1,jac1,kk)
        d_jj=dm1(jac2,jac2,kk)
        d_ij=dm1(jac1,jac2,kk)
        d_ji=dm1(jac2,jac1,kk)
        ar_ij=ar_ij+pt25*d_ij*d_ij+pt25*d_ji*d_ji+pt5*d_ij*d_ji
     &      +pt5*d_ii*d_jj-pt25*d_ii*d_ii-pt25*d_jj*d_jj
        br_ij=br_ij+pt5*d_ij*d_jj+pt5*d_ji*d_jj-pt5*d_ii*d_ij
     &       -pt5*d_ii*d_ji
        kk=kk+nstate+1-kstat
      end do

      ar_ij=alpha_r*ar_ij !scale by parameter 
      br_ij=alpha_r*br_ij

c
c Calculate Natural orbital term
c

      d_ii=dm1(jac1,jac1,L1)
      d_jj=dm1(jac2,jac2,L1)
      d_ij=dm1(jac1,jac2,L1)
      d_ji=dm1(jac2,jac1,L1)
      an_ij=pt25*d_ij*d_ij+pt25*d_ji*d_ji+pt5*d_ij*d_ji
     &      +pt5*d_ii*d_jj-pt25*d_ii*d_ii-pt25*d_jj*d_jj
      bn_ij=pt5*d_ij*d_jj+pt5*d_ji*d_jj-pt5*d_ii*d_ij
     &       -pt5*d_ii*d_ji
          
      an_ij=alpha_n*dble(nstate)*an_ij  !Scale by parameter 
      bn_ij=alpha_n*dble(nstate)*bn_ij  !and # of states

c
c Calculate Transition term (kstat < istat)
c


      at_ij=zero
      bt_ij=zero

      if (nstate.eq.1) goto 100
     
      kk=1   
      do kstat=1,nstate
        do istat=kstat+1,nstate
          kistat=kk+istat-kstat
          d_ii=dm1(jac1,jac1,kistat)
          d_jj=dm1(jac2,jac2,kistat)
          d_ij=dm1(jac1,jac2,kistat)
          d_ji=dm1(jac2,jac1,kistat)
          at_ij=at_ij+pt25*d_ij*d_ij+pt25*d_ji*d_ji+pt5*d_ij*d_ji
     &      +pt5*d_ii*d_jj-pt25*d_ii*d_ii-pt25*d_jj*d_jj
          bt_ij=bt_ij+pt5*d_ij*d_jj+pt5*d_ji*d_jj-pt5*d_ii*d_ij
     &       -pt5*d_ii*d_ji
        end do
        kk=kk+nstate+1-kstat
      end do

      at_ij=alpha_t*dble(2/(nstate-1))*at_ij  !Scale by parameter 
      bt_ij=alpha_t*dble(2/(nstate-1))*bt_ij  !and # of states

100   continue

c
c Get a_ij and b_ij
c

      a_ij=ar_ij+an_ij+at_ij
      b_ij=br_ij+bn_ij+bt_ij



      return
      end

      subroutine caldel_mormo1(num,v_i,v_j,vref_i,vref_j,w,a_ij,b_ij)
  
      implicit real*8 (a-h,o-z)
      common/ijpair/ia(1) 

      data zero /0.0d0/
      data one  /1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/
    
      dimension v_i(num),v_j(num)
      dimension vref_i(num), vref_j(num)
      dimension w(num*(num+1)/2)


c
c num    .... # of AO (basis set)
c n_overlap...# of reference DMOs
c nmo_mix ... # of current MOs to be mixed with
c vref_i, vref_j.... reference DMO
c v_i, v_j    .... current MOs 
c w      .... AO overlap
c
       
       
c
c Calculate a_ij and b_ij correlated with  MO Ovrelap term 
c

c  
c jac1 is <i> and jac2 is <j>:-->calculate s_i<i>,s_j<i>,s_i<j>,s_j<j>
c      

cc--- s_i<i> and s_j<i> -----


cc s_i<i>

      s_ii=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else 
            ind_kl=ia(l)+k
          end if
          s_ii=s_ii+vref_i(k)*v_i(l)*w(ind_kl)
        end do
      end do

cc s_j<i>

      s_ji=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else
            ind_kl=ia(l)+k
          end if
          s_ji=s_ji+vref_i(k)*v_j(l)*w(ind_kl)
        end do
      end do


cc--- s_i<j> and s_j<j> -----


cc s_i<j>

      s_ij=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else 
            ind_kl=ia(l)+k
          end if
          s_ij=s_ij+vref_j(k)*v_i(l)*w(ind_kl)
        end do
      end do

cc s_j<j>


      s_jj=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else 
            ind_kl=ia(l)+k
          end if
          s_jj=s_jj+vref_j(k)*v_j(l)*w(ind_kl)
        end do
      end do


cc Get a_ij and b_ij

      a_ij=-pt5*s_ii**2+pt5*s_ji**2+pt5*s_ij**2-pt5*s_jj**2
      b_ij=-s_ii*s_ji+s_ij*s_jj


      return
      end


      subroutine caldel_mormo2(num,v_i,v_j,vref_i,w,a_ij,b_ij)
  
      implicit real*8 (a-h,o-z)
      common/ijpair/ia(1) 

      data zero /0.0d0/
      data one  /1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/
    
      dimension v_i(num),v_j(num)
      dimension vref_i(num)
      dimension w(num*(num+1)/2)



c
c num    .... # of AO (basis set)
c n_overlap...# of reference DMOs
c nmo_mix ... # of current MOs to be mixed with
c vref_i ... reference DMO
c v_i, v_j   .... current MOs 
c w      .... AO overlap
c
       
       
c
c Calculate a_ij and b_ij correlated with  MO Ovrelap term 
c

c
c jac1 is <i>: <> is refernce DMO--> calculate only s_i<i>, s_j<i>
c


cc s_i<i>

      s_ii=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else 
            ind_kl=ia(l)+k
          end if
          s_ii=s_ii+vref_i(k)*v_i(l)*w(ind_kl)
        end do
      end do

cc s_j<i>

      s_ji=zero
      do k=1,num
        do l=1,num
          if (l.le.k) then
            ind_kl=ia(k)+l
          else
            ind_kl=ia(l)+k
          end if
          s_ji=s_ji+vref_i(k)*v_j(l)*w(ind_kl)
        end do
      end do


cc Get  a_ij,b_ij

      a_ij=-pt5*s_ii**2+pt5*s_ji**2
      b_ij=-s_ii*s_ji



      return
      end


      subroutine calsum_maxD3(nstate,n_active,dm1,
     &                       alpha_n,alpha_r,alpha_t,sum)
  
      implicit real*8 (a-h,o-z)


      data zero /0.0d0/
      data one  /1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/
    
      dimension dm1(n_active,n_active,nstate*(nstate+1)/2+1) 


      L1=nstate*(nstate+1)/2+1 ! index where SA DM1 is stored

c
c Ruedenberg term
c
      sum_r=zero
      kk=1      ! kk is index of dm1 between kstat-kstat
      do kstat=1,nstate
        do i=1,n_active
          sum_r=sum_r+dm1(i,i,kk)*dm1(i,i,kk)
        end do
        kk=kk+nstate+1-kstat
      end do

      sum_r=alpha_r*sum_r !scale by parameter 

c
c Natural orbital term
c
      sum_n=zero
      do i=1,n_active     
        sum_n=sum_n+dm1(i,i,L1)**2
      end do
 
      sum_n=alpha_n*dble(nstate)*sum_n  !Scale by parameter 
                                        !and # of states


c
c Transition term (kstat < istat)
c
      if (nstate.eq.1) goto 100
     
      sum_t=zero
      kk=1       ! kk is index of DM1 between kstat-kstat
      do kstat=1,nstate
        do istat=kstat+1,nstate
          kistat=kk+istat-kstat   ! kistat is index of only TD DM1 
          do i=1,n_active
            sum_t=sum_t+dm1(i,i,kistat)*dm1(i,i,kistat)
          end do
        end do
        kk=kk+nstate+1-kstat
      end do

      sum_t=alpha_t*dble(2/(nstate-1))*sum_t  !Scale by parameter 
                                              !and # of states

100   continue

c
c Get sum  of all terms. i.e. D=Ruedenberg+Natural+Transition
c

      sum=sum_r+sum_n+sum_t




      return
      end

      subroutine calsum_mormo(num,n_active,n_overlap,indmo_mix,
     &                        ihead_mo,vec_ref,vec,w,sum)
  
      implicit real*8 (a-h,o-z)

      common/ijpair/ia(1) 
     

      data zero /0.0d0/
      data one  /1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/
    
      dimension vec_ref(num,n_overlap)
      dimension vec(num,n_active)
      dimension w(num*(num+1)/2)

      dimension indmo_mix(n_active)

c
c num    .... # of AO (basis set) (input & output)
c n_overlap...# of MOs to be overlaped (input & output)
c n_active
c vec_ref.... reference DMO/AO (input & output)
c vec    .... current MO/AO    (input & output)
c w      .... AO overlap       (input & output)
c indmo_mix
c ihead_mo
c

       
c
c Calculate MORMO term
c

      sum=zero
      do i=1,n_overlap
        ind_v=indmo_mix(ihead_mo+i-1) 
          dum=zero
          do j=1,num
            do k=1,num
              if (k.le.j) then
                ind_jk=ia(j)+k
              else
                ind_jk=+ia(k)+j
              end if 
              dum=dum+vec_ref(j,i)*vec(k,ind_v)*w(ind_jk)
          end do
        end do                          
        sum=sum+dum**2
      end do


      return
      end


      subroutine chcksymord(num,n_active,moslab,w,q,s,temp)

      implicit real*8 (a-h,o-z)

      parameter(mxioda=255)

C#g77 ------------------------------------------
      CHARACTER*8 dfrmvec,endwrd,word
C#g77 ------------------------------------------
      
      common/iofile/ir,iw,ip
      common/dafile/idaf,nav,ioda(mxioda)
      common/ijpair/ia(1)


      dimension moslab(n_active)     ! irreps label for the active MOs

      dimension moslab_ref(n_active+1),mosirp_ref(n_active),
     &          mosdeg_ref(n_active)! irreps labels for ref DMOs


      dimension w(num*n_active)        ! ref DMO/AO coefficient
      dimension q(num,num)          ! canonical vec
      dimension s(num*(num+1)/2)    ! AO overlap
     
      dimension temp(num)           ! temporary region 

      data zero,one /0.0d0,1.0d0/

      data dfrmvec   /' $DFMVEC'/
c      data diahead  /' DMO SET'/
      data endwrd   /' $END'/

c
c This routine assign the irreps of $DFMVEC and 
c check each irreps array component is same with current MOs or not.
c This procedure os necessary to use -NTN- algorithm when DMO-baesd CI
c is carried out.
c



c
c Read $DFMVEC
c

      rewind(ir)
5     continue 
      read(ir,9800,end=10,err=10) word
      goto 20
10    write(iw,*) 'Can not find $DFMVEC. Stop'
      stop
20    continue
      if (dfrmvec.ne.word) goto 5
 
      do j=1,n_active
        max=0
30      min=max+1
        max=max+5
        if (max.gt.num) max=num
        if (num.le.99) read(ir,9810) idum,idum,
     &     (w(i+num*(j-1)),i=min,max)
        if (num.gt.99) read(ir,9820) idum,idum,
     &     (w(i+num*(j-1)),i=min,max)
        if (max.lt.num) goto 30
      end do

!
! Now all $DFMVEC data stored in w
!



!-- Assign symmetrty (irreps) for reference DMOs 
!   and sort refernce DMOs by irreps with using moslab_ref

      call daread(idaf,ioda,q,num*num,13) ! read canonical vector


      call symmos(moslab_ref,mosirp_ref,mosdeg_ref,q,s,w,
     &            temp,ia,num,num,n_active,num)
      
! -- Check symmetry label beween the current MOs and $DFMVEC for
! -  active space 
!

      do i=1,n_active
        if (moslab(i).ne.moslab_ref(i)) then
          write(iw,'(''Error. $DFMVEC irreps ordering is different''/
     &         '' with the ordering of MOs used for -NTN- algorithm ''/
     &         '' (e.g. $VEC)''/
     &         ''$DRT cannot be correct when DMO/CI is carried out.'')')
          write(iw,*)'Symmetry labels used for -NTN- (active)'
          write(iw,9800) (moslab(j),j=1,n_active)
          write(iw,*) 'Symmetry labels of $DFMVEC (active)'
          write(iw,9800) (moslab_ref(j),j=1,n_active)
          write(iw,*) 'Stop'
          stop
        end if 
      end do
       

      return


9800  format(a8)
9810  format(i2,i3,5e15.8)
9820  format(i3,i2,5e15.8)

      end

      subroutine detblock(nft,ndfrm,num,n_active,nstate,moslab_active,
     &                    moslab_charact,avocc,th_doc,th_vir,v,w,s,dm,
     &                    ndoc_cand,moslab_pseudo)           !HN 2/2/02

      implicit real*8 (a-h,o-z)

c
c This program classifies the (candidate) active DMOs to the three type,
c doc, valence, and virtual type with refering averaged occupation
c numbers ( and $DFMVEC, if available, i.e. ndfrm.ne.0)
c


      common /iofile/ir,iw,ip
      common /symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir

 
      dimension moslab_charact(n_active)
      dimension moslab_active(n_active)
      dimension moslab_pseudo(n_active)                       !HN 2/2/02
      dimension avocc(n_active)

      dimension v(num,n_active)
      dimension w(num,n_active) 

      dimension dm(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension s(num*(num+1)/2)

      dimension ind_mo(n_active)

      dimension indtemp0(n_active)
      dimension indtemp1(n_active)
      dimension indtemp2(n_active)

c
c
c moslab_charact(i).... label to identify the i-th active MO 
c                       (detemined by the threefold density criterion)
c                       belongs to 
c                       =2  dobouly occupied (doc)
c                       =1  valence 
c                       =0  virtual  
c                       (input & output)
c
c moslab_pseudo(i).... label to identify the i-th active MO belongs to 
c                   =1 (not 2!)  target of catch and release as doc (not Vl!) block   
c                   =0  not target (i.e. apparently vl and vir
c                                       or not apply C-R algorithm )
c                       (input & output)
c
c ndoc_cand      ..... # of candidate MO of doc block based on C-R
c                      algorithm. If C-R algorithm is Not applied,
c                      ndoc_cand=0 (input & output) 
c


c v     .... MOs of the threefold density criterion (input)
c            output is reorderd by taking overlap between $DFMVEC
c            for each block (doc/via/vir)
c
c avocc .... averaged occupation number of MOs of the threefold
c            density criterion (input). Output is reorderd.
c
c dm    .... density matrices of MOs of the threefold density criterion
c            (input) Output is updated based on the new ordering 
c            of the MOs
c
c w     .... temporary region e.g. to store (1) sym. orthogonalized 
c            $DFMVEC etc 
c                                      
c s     .... AO overlap
c
c
c ind_mo ... index label to identify one-to-one correspondence
c            between v and w
c            Notice !!  ind_mo > 0----> the same sign
c                              < 0----> different sign 
c            i-th current MO is odered to ind_mo(i)-th position
c            (indtemp0 is supported array for ind_mo)
c


      if (ndfrm.eq.0) then
        write(iw,*)'$DFMVEC is not available. All active MOs are treated
     &  as valence block, and the fourfold way will continue. '
        do i=1,n_active
          moslab_charact(i)=nsymbol_val
        end do
        return ! 
      end if

c
c Read (sym. orthogonalized) $DFMVEC from the scrach file 
c

      rewind(nft)
      call rdfuldia(nft,num*n_active,w)
     
c
c Check input of moslab_charact
c

      do i=1,n_active
        if ((moslab_charact(i).ne.nsymbol_doc).and.
     &      (moslab_charact(i).ne.nsymbol_val).and.
     &      (moslab_charact(i).ne.nsymbol_vir)) then
          write(iw,*) '$DIABAT moslab_charact has illegal values.'
          stop
        end if 
      end do

c
c Initialization of ind_mo array as ) (dummy)
c

      do i=1,n_active
        ind_mo(i)=0
      end do
     


c
c ------  Doc block ------
c


cc Pick up doc MO of $DFMVEC

      n_doc=0
      do i=1,n_active
!        if (moslab_charact(i).eq.2) then
        if (moslab_charact(i).eq.nsymbol_doc) then
          n_doc=n_doc+1
          indtemp1(n_doc)=i
        end if
      end do
       

cc-- Select the candidate of doc MOs --

!-- case A -- Catch and Release algorithm was applied 
      if (ndoc_cand.ne.0) then                               !HN 2/2/02
        icount=0                                             !HN 2/2/02
        do i=1,n_active                                      !HN 2/2/02
          if (moslab_pseudo(i).eq.nsymbol_val) then          !HN 2/2/02
            icount=icount+1                                  !HN 2/2/02
            indtemp2(icount)=i                               !HN 2/2/02
          end if                                             !HN 2/2/02
        end do                                               !HN 2/2/02
        ntemp_doc=ndoc_cand                                  !HN 2/2/02
!-- case B -- Catch and Release algorithm was Not applied 
      else if (n_doc.ne.0) then
        ntemp_doc=0
        do i=1,n_active
          if (th_doc.lt.avocc(i)) then 
            ntemp_doc=ntemp_doc+1
            indtemp2(ntemp_doc)=i
          end if
        end do
      end if 

!--- Now in both cases, ntemp_doc is # of candidates MO fpr Dc block

      if (n_doc.le.ntemp_doc) then 

cc Determine n_doc MOs as doc MOs from ntemp_doc candidates

        do i=1,n_active 
          indtemp0(i)=0
        end do
    
        call dfrmcheck(num,n_active,n_doc,ntemp_doc,indtemp1,indtemp2,
     &                   indtemp0,v,w,s)

cc Doc part of ind_mo array is determined 
        do i=1,n_active
          if (indtemp0(i).ne.0) ind_mo(i)=indtemp0(i)
        end do

      else 
        write(iw,*) '# of candidates MO for doc block is too small'
        write(iw,*) 'DC block is neglected and treated as val block'
        do i=1,n_active
          if (moslab_charact(i).eq.nsymbol_doc) 
     &          moslab_charact(i)=nsymbol_val
        end do
      end if


c
c ------  Vir block ------
c


cc Pick up virtual  MO of $DFMVEC


      n_vir=0
      do i=1,n_active
!        if (moslab_charact(i).eq.0) then
        if (moslab_charact(i).eq.nsymbol_vir) then
          n_vir=n_vir+1
          indtemp1(n_vir)=i
        end if
      end do
       
      if (n_vir.ne.0) then 

cc Select the candidate of vir MOs 

        ntemp_vir=0
        do i=1,n_active
!          if (th_vir.lt.avocc(i)) then 
          if (avocc(i).lt.th_vir) then 
            ntemp_vir=ntemp_vir+1
            indtemp2(ntemp_vir)=i
          end if
        end do

        if (n_vir.le.ntemp_vir) then 

cc Determine n_vir MOs as vir MOs from ntemp_vir candidates

          do i=1,n_active
            indtemp0(i)=0
          end do
    
          call dfrmcheck(num,n_active,n_vir,ntemp_vir,indtemp1,indtemp2,
     &                   indtemp0,v,w,s)

cc Vir part of ind_mo is determined
          do i=1,n_active
            if (indtemp0(i).ne.0) ind_mo(i)=indtemp0(i)
          end do 
        
        else 
          write(iw,*) '# of candidates MO for vir block is too small'
          write(iw,*) 'Vir block is neglected and treated as Vl block'
          do i=1,n_active
            if (moslab_charact(i).eq.nsymbol_vir) 
     &          moslab_charact(i)=nsymbol_val
          end do
        end if 

      end if 


c
c---- The rest is valence orb. ---
c

cc-- Pick up valence DMOs--
      n_val=0
      do i=1,n_active
        if (moslab_charact(i).eq.nsymbol_val) then
          n_val=n_val+1
          indtemp1(n_val)=i
        end if
      end do

      ntemp_val=0
      do i=1,n_active
        if (ind_mo(i).eq.0) then 
          ntemp_val=ntemp_val+1
          indtemp2(ntemp_val)=i
        end if
      end do

!cc ---  for debug 
      if (ntemp_val.ne.n_val) then  
        write(iw,*) ntemp_val,n_val, "Something is wrong in DETBLOCK"
        stop
      end if 
!cc ---

      do i=1,n_active
        indtemp0(i)=0
      end do

      call dfrmcheck(num,n_active,n_val,ntemp_val,indtemp1,indtemp2,
     &               indtemp0,v,w,s)

cc Val part of ind_mo is determined

      do i=1,n_active
        if (indtemp0(i).ne.0) ind_mo(i)=indtemp0(i)
      end do

c
c  Reorder the MOs 
c

      
      do  i=1,n_active
        if (ind_mo(i).lt.0) then 
          do j=1,num
            w(j,-ind_mo(i))=-v(j,i)
          end do
        else 
          do j=1,num
            w(j,ind_mo(i))=v(j,i)
          end do
        end if
      end do

      do i=1,n_active
        do j=1,num
          v(j,i)=w(j,i)
        end do
      end do

c
c Reorder elements of DMs (Notice: With sign change of MOs, each
c                                  DM elements change)
c

      do idm=1,nstate*(nstate+1)/2+1
cc--- change row and column
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
          else
            ind_i=ind_mo(i)
          end if
          do j=1,n_active
            if (ind_mo(j).lt.0) then
              ind_j=-ind_mo(j)
            else 
              ind_j=ind_mo(j)
            end if
            w(ind_j,ind_i)=dm(j,i,idm)
          end do
        end do
cc--- change sign of elements
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
            do j=1,n_active
              w(j,ind_i)=-w(j,ind_i)
            end do
          end if
        end do
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
            do j=1,n_active
              w(ind_i,j)=-w(ind_i,j)
            end do
          end if
        end do
cc--- up date dm         
        do i=1,n_active
          do j=1,n_active
            dm(j,i,idm)=w(j,i)
          end do
        end do
      end do

c
c Reorder irreps label
c

      do i=1,n_active
        indtemp1(i)=moslab_active(i)
      end do
      do i=1,n_active
        if (ind_mo(i).lt.0) then
          moslab_active(-ind_mo(i))=indtemp1(i)
        else 
          moslab_active(ind_mo(i))=indtemp1(i)
        end if
      end do
 
c
c Reorder avocc
c

      do i=1,n_active
        avocc(i)=dm(i,i,nstate*(nstate+1)/2+1)
      end do

      return

9800  format(a8)

      end 


      subroutine dfrmcheck2(num,n_active,nb_det,nb_temp,
     &                      indb_crrt,istack,v,w,s)

      implicit real*8 (a-h,o-z)
 
c
c This subroutine make index array to represent one-to-one 
c correspondence between some of v and w 

c
c Note; dfrmcheck2 is slightly different with dfrmcheck
c       The first nb_det orbital's order is defined by
c       reference MOs. The rest is arbitray, but 
c       necessary to be defined !
c


      common/iofile/ir,iw,ip
      common/ijpair/ia(1)

      dimension v(num,n_active) ! not (num,nb_temp)
      dimension w(num,nb_det)
      dimension s(num*(num+1)/2)

      dimension istack(nb_temp)
      dimension store_dot(nb_temp,nb_temp)

      dimension indb_crrt(nb_temp)


      data zero,one  /0.0d0,1.0d0/

c
c v  ... MO coefficient of active space at the current geometry
c        return without modification
c
c w  ... reference MOs i.e $DIAVEC or $DPSVEC not $DFMVEC
c
c s  ... AO overlap
c     

c
c n_active   ... # of active orbitals
c 
c nb_det     ... # of reference ($DIAVEC or $DPSVEC) MOs 
c
c nb_temp    ... # of MOs per the doc or val block
c 
c

c
c
c indb_crrt(i) ... i-th MO counted within the block is stored
c                   on indb_crrt(i)-th array
c 
c



c
c store_dot(i,j): overlap-like quantity <u_i(ref)|u_j(current)>
c
c istack(1:nb_temp): i-th MO counted within the block is reordered 
c                     to the istack(i)-th MO counted within the block
c                    
c THIS DEFINITION OF istack is different with dfrmcheck                     
c
c set
c

!--- zero--
       do j=1,nb_temp
         do i=1,nb_det
           store_dot(i,j)=zero
         end do
       end do 

!--- dummy value to take consistency for the rest
!    nb_temp-nb_set MOs
       do j=1,nb_temp
         do i=nb_det+1,nb_temp
           store_dot(i,j)=0.001
         end do
       end do

    
c
c ---- Check of deformation ------
c

c
c Step(1) Calculate overlap matrix between reference's DMO
c block and current MO block
c

      
      do i=1,nb_det
        do j=1,nb_temp
          dum=zero
          do k=1,num
            do l=1,num
              if (l.le.k) then
                ind_kl=ia(k)+l
              else 
                ind_kl=ia(l)+k
              end if
              dum=dum+w(k,i)*v(l,indb_crrt(j))*s(ind_kl)
            end do
          end do 
          store_dot(i,j)=dum
        end do
      end do
 

c
c
c Step(2) Search maximum overlaping and determine the one-to-one
c correspondence of nb_det MOs. The rest MO's postion 
c is specified arbitratry.
c Thus, scan is performed nb_temp * nb_temp
c
c

      do i=1,nb_temp
!
! Search max element of store_dot : max value = store_dot(i_ref,i_crrt)
!

        dum_max=zero
        do j=1,nb_temp
!          do k=1,nb_det
          do k=1,nb_temp
            dum=dabs(store_dot(k,j))
            if (dum_max.lt.dum) then
              dum_max=dum
              i_ref=k
              i_crrt=j
            end if
          end do
        end do
         
!
! Now correspondence between i-th reference MO  
! indb_crrt(i_crrt)-th MO are defined
!

!
! Index array 
!

!
! i_crrt th MO counted within block must move to i_ref th.
! the sign of indb_crrt, istack is FIXED.
! The sign is NOT affected by this procedure
!
! 

         istack(i_crrt)=i_ref  

!
! The all values of store_dot(*,i_crrt) and 
! store_dot(i_ref,*) are replaced by 
! dummy value zero because the above correspondence are determied
!
        do j=1,nb_temp
          store_dot(j,i_crrt)=zero
        end do
        do j=1,nb_temp
          store_dot(i_ref,j)=zero
        end do

        

      end do
         

      return 
      end 

      subroutine dfrmcheck(num,n_active,nb_det,nb_temp,indb_ref,
     &                     indb_crrt,istack,v,w,s)

      implicit real*8 (a-h,o-z)
 
c
c This subroutine make index array to represent one-to-one 
c correspondence between some of v and w 
c

      common/iofile/ir,iw,ip
      common/ijpair/ia(1)

      dimension v(num,n_active)
      dimension w(num,n_active)
      dimension s(num*(num+1)/2)

      dimension istack(n_active)
      dimension store_dot(nb_det,nb_temp)

      dimension indb_ref(nb_det)
      dimension indb_crrt(nb_temp)


      data zero,one  /0.0d0,1.0d0/

c
c v  ... MO coefficient of active space at the current geometry
c        return without modification
c
c w  ... DMO coefficients at the reference geometry etc.
c
c s  ... AO overlap
c     

c
c n_active   ... # of active orbitals
c 
c nb_det     ... # of MOs to be assigned 
c
c nb_temp    ... # of MOs of candidates to be assinged
c                i.e. nb_det MO are assigned (nb_det < or = nb_temp)
c 
c

c
c indb_ref   ... index array to identify the referenced MO to be
c                used to assign nb_det MOs
c
c indb_crrt   ... index array to identify the current nb_det MOs to be 
c                assigned 
c 
c



c
c store_dot(i,j): overlap-like quantity <u_i($DFMVEC)|u_j(current)>
c
c istack(1:n_active): i-th stored current DMO is ordered to the
c                     istack(i)-th position 
c
c (Notice (1) Also this array shows the sign. if the overlap-like
c             quantity is smaller than 0, the value of istack(i) is <0 
c         (2) As dummy value istack(i) is 0 when input.
c             In this subroutine the nb_det MO's positions are
c             determined; thus nb_det elements of istack array become
c             non 0 (i.e. non dummy) values.)
c                    
c

c
c Zero clear
c
       do j=1,nb_temp
         do i=1,nb_det
           store_dot(i,j)=zero
         end do
       end do 

    
c
c ---- Check of deformation ------
c

c
c Step(1) Calculate overlap matrix between reference's DMO
c block and current MO block
c

      
      do i=1,nb_det
        do j=1,nb_temp
          dum=zero
          do k=1,num
            do l=1,num
              if (l.le.k) then
                ind_kl=ia(k)+l
              else 
                ind_kl=ia(l)+k
              end if
              dum=dum+w(k,indb_ref(i))*v(l,indb_crrt(j))*s(ind_kl)
            end do
          end do 
          store_dot(i,j)=dum
        end do
      end do
 

c
c
c Step(2) Search maximum overlaping and determine the one-to-one
c correspondence of nb_det MOs 
c
c

      do i=1,nb_det
!
! Search max element of store_dot  : maxvalue = store_dot(i_ref,i_crrt)
!

        dum_max=zero
        do j=1,nb_temp
          do k=1,nb_det
            dum=dabs(store_dot(k,j))
            if (dum_max.lt.dum) then
              dum_max=dum
              i_ref=k
              i_crrt=j
            end if
          end do
        end do
         
!
! Now correspondence between indb_ref(i_ref)-th $DFMVEC (i-th MO of the 
! block defined by indb_ref) and indb_crrt(i_crrt)-th MO
! are determined
!

!
! Index array 
!
        if (store_dot(i_ref,i_crrt).lt.zero) then 
c!          istack(indb_ref(i_ref))=-indb_crrt(i_crrt)
          istack(indb_crrt(i_crrt))=-indb_ref(i_ref)
        else 
c!          istack(indb_ref(i_ref))=indb_crrt(i_crrt)
          istack(indb_crrt(i_crrt))=indb_ref(i_ref)
        end if 
       
!
! The all values of store_dot(*,i_crrt) and 
! store_dot(i_ref,*) are replaced by 
! dummy value zero because the above correspondence are determied
!
        do j=1,nb_det
          store_dot(j,i_crrt)=zero
        end do
        do j=1,nb_temp
          store_dot(i_ref,j)=zero
        end do

        

      end do
         





      return 
      end 

      subroutine diabmqdr

c
c This subroutine is the driver between diabatization procedure
c and MC-QDPT. 
c Read the information from the preliminary CAS-based diabatization 
c from the FT scratch file.
c Next, check and modify some parameters of the namelist
c $MCQDPT if necessary.
c If diabatization is not required (i.e. $WFN WFNFLG.ne.16), this 
c program do nothing.
c 


      implicit real*8(a-h,o-z)

      integer wfnflg

      logical kst

      common /iofile/ir,iw,ip
      common /diatmpfil/nft21,nft22,nft23,nft24
      common /drtpar/nco,nmcco,nci,msym 
      common /mq2par/dmqpar(200),avecoe(20),
     &               imqpar(400),maincs(3),lpconf(20),kst(20)
      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,alpha_t,
     &              slctth,moslab_charact,th_doc,th_vir,conv_th,max_iter,
     &              ngrstate                                              !RV 9/21/06

      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)


      namelist /wfn/wfnflg

      dimension temp(20)


c
c Read namelist -WFN-
c

      rewind(ir)
      read(ir,WFN,end=5,err=5)
5     continue
    
      if ((wfnflg.ne.16) .and. (wfnflg.ne.17)) return


c
c Read parameters for diabatization
c


c
c Reset some parameters of namelist -MCQDPT-
c
 
c      
c - inorb, iforb -  MOs are read from DA10 (DMOs) and used for MCQDPT
c
  
      inorb=0 
      iforb=0

      imqpar(1)=iforb
      imqpar(3)=inorb

c
c - arrange # of target states etc.
c Note: Generally QDPT target states is part of CAS-CI target states
c       and they are not necessarly equal.
c       Therefore lofical array 'kst' may be modified. 
c

c-- Save original QDPT target state index (i.e.$MCQDPT 'kstate' input 
c- value)--

      do i=1,imqpar(24)
        if (kst(i)) then 
          kstcas_mq(i)=1
        else
          kstcas_mq(i)=0
        end if
      end do

cc--- arrange start ---

c-- case: symmetry is on --

      if (ndmosym.eq.0) then 
        istsym=msym ! set target state symmetry from -NTN- algorithm
        imqpar(6)=istsym
        nstci=imqpar(24)
c        do i=1,nstci
c          if (kst(i)) then  ! kst(i) is true
c            kstci_mq(i)=1
c          else
c            kstci_mq(i)=0
c          end if
c        end do
c        do i=nstci+1,nstcas
c          kstci_mq(i)=0
c        end do
        do i=1,nstci
          kstci_mq(i)=kstcas_mq(i)
        end do
      end if
       

c-- case: symmetry is off --
   
      if (ndmosym.ne.0) then 
        istsym=1
        imqpar(6)=istsym
        imqpar(24)=nstci
        icount=0
        do i=1,nstci
          if (kstci_cas(i).eq.1) then
            icount=icount+1
            if (kst(icount)) then 
              kstci_mq(i)=1
            else
              kstci_mq(i)=0
            end if
          else
            kstci_mq(i)=0
          end if 
        end do

        do i=1,nstci
          if (kstci_mq(i).eq.0) then
            kst(i)=.false.
          else 
            kst(i)=.true.
          end if
        end do 

      end if 

c
c Set parameter
c

      nstmq=imqpar(23)


c
c CI max coef for CAS-CI is resored for only QDPT-target states
c

      do i=1,nstcas
        temp(i)=coef_max1(i)
      end do

      icount=0
      do i=1,nstcas
        if (kstcas_mq(i).ne.0) then 
          icount=icount+1
          coef_max1(icount)=temp(i)
        end if
      end do


!---for debug
      if (icount.ne.nstmq) then
        write(iw,*) 'Error in diabmqdr. Inconsistent '
        stop
      end if 
!----
        
c
c
c Warning important message for changing input value etc 
c

      write(iw,*) '********Routine diabmqrd (driver routine) ******'
      write(iw,*) '                                                  '
      write(iw,'(''================ Notice ====================''/
     & ''MC-QDPT calculation is carried out by using ''/
     & ''Fock orbital for inactive, active, and external orbitals''/
     & ''that are stored in DA10 #15 at CASSCF step''/
     & ''==========================================='')')
      write(iw,*) '                                                  '
      write(iw,'(''Input parameters iforb and inorb are fixed as ''/
     &  ''iforb=0, inorb=0'')')

      if (ndmosym.ne.0) then 
        write(iw,*) '                                            '
        write(iw,'(''=============== Warning ====================''/
     &  ''The active DMOs have no symmetry, therefore all ''/
     &  ''calculation of MC-QDPT is carried out without symmetry.''/ 
     &  ''To keep the same QDPT target states with the input, ''/
     &  ''MC-QDPT calculation carried out is equivalent to using''/
     &  ''the below $MCQDPT parameters.'')') 
        write(iw,*) '------------------------------'
        write(iw,*) 'istsym=',istsym
        write(iw,*) 'nstci=',nstci
        write(iw,*) 'kstate=',(kstci_mq(i),i=1,nstci)
        write(iw,*) '------------------------------'
        write(iw,*) '============================================='
        write(iw,*) '********* End of Routine diabmqrd ***************'
      end if 
        
      
        
      return

      end

               
      subroutine diabmqwr1(nft12,nft21,ndmosym,
     &                     nstry,eci_sav,eci,nwks,civec)
 
      implicit real*8 (a-h,o-z)


      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)

      common/iofile/ir,iw,ip
      data zero   /0.0d0/
      data cvgtol /1.0d-05/

      dimension civec(nwks)
      dimension eci_sav(nstry)
      dimension eci(nstry)
      dimension temp(nstry)



c
c Copy DMO-based CAS-CI energy to temp
c

      do i=1,nstry
        temp(i)=eci(i)
      end do 

c
c Write parameters to FT scratch file
c


c
c (1) Search the cooresponding states between CASMO CAS-CI states and
c DMO-based CAS-CI states, and write CI-vector to FT21.
c This procedure is meaningful only if CAS-CI states are constrained
c by symmetry but DMOs is not (i.e.ndmosym=1). 
c

        
      do ist=1,nstcas
        do jst=1,nstry
          dum=abs(temp(jst)-eci_sav(ist))
          if (dum.lt.cvgtol) then
            temp(jst)=zero ! dummy vale 
            nstci=jst
            goto 10 ! exit loop of jst
          end if 
        end do
10      continue 
      end do


      icount=0 ! for Debug
      do ist=1,nstci
        call rdfuldia(nft12,nwks,civec)
        if (temp(ist).eq.zero) then
          icount=icount+1 ! for Debug
          coef_max=zero
          do i=1,nwks
            coef=civec(i)
            if (dabs(coef_max).lt.dabs(coef)) coef_max=coef
          end do
          coef_max1(icount)=coef_max
          write(nft21) civec
        end if
      end do

      if (icount.ne.nstcas) then
        write(iw,*) 'Error, Traget CAS-CI states cannot be found.'
        stop
      end if 
 
c-- total # of DMO-based CAS-CI states


c-- kstci_cas=1-->target CAS-CI states. =0---> not
      do ist=1,nstci
c!        write(iw,*) 'temp',temp(ist)
        if (temp(ist).eq.zero) then
          kstci_cas(ist)=1
        else 
          kstci_cas(ist)=0
        end if 
      end do


!--- Debug print ----
c      write(6,*) 'Values in /diabatarget/'
c      write(6,*)'nstci=',nstci
c      write(6,*)'nstcas=',nstcas
c      write(6,*)'nstmq=',nstmq
c      write(6,*) 'kstci_cas'
c      do i=1,nstci
c        write(6,*)kstci_cas(i)
c      end do
c      write(6,*) 'kstci_mq'
c      do i=1,nstci
c        write(6,*) kstci_mq(i)
c      end do
c      write(6,*) 'kstcas_mq'
c      do i=1,nstcas
c        write(6,*) kstcas_mq(i)
c      end do
!--------------------



         

      return

      end 

      subroutine diabmqwr2(nstate_mq,nwks,casvec,eigval,diavec)     

c
c This subroutine is the driver between diabatization procedure
c and MC-QDPT. 
c Read the information obtained by the previous driver routine 
c diabmqdr from FT scrach file and add information obtained by
c MC-QDPT calculation. These information is written to FT scrach file
c
c If diabatization is not required (i.e. $WFN WFNFLG.ne.16), this 
c program do nothing.
c 

      implicit real*8(a-h,o-z)

      integer wfnflg

      common /iofile/ir,iw,ip
      common /diatmpfil/nft21,nft22,nft23,nft24

      data zero /0.0d0/
      namelist /wfn/wfnflg

      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)

      dimension casvec(nwks,nstate_mq)!MC-QDPT's CAS-CI vec(QDPT target)
      dimension eigval(nstate_mq)
      dimension diavec(nstate_mq,nstate_mq)
     
      dimension temp(nstate_mq)


c
c Read namelist -WFN-
c

      rewind(ir)
      read(ir,WFN,end=5,err=5)
5     continue
    
      if ((wfnflg.ne.16) .and. (wfnflg.ne.17)) return


!--for Debug
      if (nstmq.ne.nstate_mq) then
        write(iw,*) 'Error in diabmqwr2'
        stop
      end if 
!--for Debug


c
c Search max CI component of CAS-CI state in MC-QDPT calculation 
c

      do i=1,nstate_mq
        coef_max=zero
        do j=1,nwks
          coef=casvec(j,i)
          if (abs(coef_max).lt.abs(coef)) then 
            coef_max=coef
            coef_max2(i)=coef_max
          end if
        end do
      end do

c
c Set the result values of MC-QDPT calculations
c

      do i=1,nstate_mq
        e_mcqdpt(i)=eigval(i)
      end do
!-- 
      icount=0
      do i=1,nstate_mq
        do j=1,nstate_mq
          icount=icount+1
          eigvec(icount)=diavec(j,i)
        end do
      end do

!--- Debug print ----
c      write(6,*) 'Values in /diabatarget/'
c      write(6,*)'nstci=',nstci
c      write(6,*)'nstcas=',nstcas
c      write(6,*)'nstmq=',nstmq
c      write(6,*) 'kstci_cas'
c      do i=1,nstci
c        write(6,*)kstci_cas(i)
c      end do
c      write(6,*) 'kstci_mq'
c      do i=1,nstci
c        write(6,*) kstci_mq(i)
c      end do
c      write(6,*) 'kstcas_mq'
c      do i=1,nstcas
c        write(6,*) kstcas_mq(i)
c      end do
c
c      do i=1,nstmq
c        write(6,*) 'coef_max1,2',coef_max1(i),coef_max2(i)
c      end do
c
c      write(6,*) 'Heff eigvec'
c
!--------------------



      return

      end

               
      subroutine diabset

      implicit real*8 (a-h,o-z)

      parameter(ndm_size=16)
      parameter(mxn_ref=10)
 
      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,alpha_t,
     &              slctth,moslab_charact(ndm_size),th_doc,th_vir,
     &              conv_th,max_iter,ngrstate                          !RV 9/21/06
      common/prototype/n_ref,nmlap(mxn_ref),ndlap,norient  
      common/diatmpfil/nft21,nft22,nft23,nft24    
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir

      dimension imlap(mxn_ref)
      dimension imoslab_charact(ndm_size)
c---

c--- Default values for /diabat/
      data idmosym /0/  
      data istry,ifrm,ircsf /0,0,0/
      data alp_n,alp_r,alp_t/2.0d0,1.0d0,0.5d0/
      data sslctth /0.2d0/
      data tth_doc,tth_vir /1.8,0.2/
      data cconv_th /1.0d-06/
      data imax_iter /200/
      data imoslab_charact /ndm_size*1/   ! 1=nsymbol_val

c--- Default values for /prototype/
      data imlap /mxn_ref*0/
      data idlap /0/      
      data iorient /0/   
      data i_ref /1/

c--- setting (i.e. definition) of /diatmpfil/
      data ift21,ift22,ift23,ift24/21,22,23,24/

c--- setting (i.e.definition) of /symboldmo/
      data isymbol_doc,isymbol_val,isymbol_vir/2,1,0/


      ndmosym=idmosym
      nstry=istry
      ndfrm=idfrm
      nrcsf=ircsf
      alpha_n=alp_n
      alpha_r=alp_r
      alpha_t=alp_t
      slctth=sslctth
      th_doc=tth_doc
      th_vir=tth_vir
      conv_th=cconv_th
      max_iter=imax_iter
      
      do i=1,ndm_size
        moslab_charact(i)=imoslab_charact(i)
      end do

      do i=1,mxn_ref
        nmlap(i)=imlap(i)
      end do

      ndlap=idlap
      norient=iorient
      n_ref=i_ref

      nft21=ift21
      nft22=ift22
      nft23=ift23
      nft24=ift24

      nsymbol_doc=isymbol_doc
      nsymbol_val=isymbol_val
      nsymbol_vir=isymbol_vir







      return
      end

      subroutine dmosymord(v,w,s,num,n_active,moslab_active,avocc)

      implicit real*8 (a-h,o-z)
 
      common/iofile/ir,iw,ip
      common/ijpair/ia(1)

      dimension v(num*n_active)
      dimension w(num*n_active)
      dimension s(num*(num+1)/2)
      dimension avocc(n_active)  
      dimension moslab_active(n_active)
      dimension istack(n_active)
      dimension store_dot1(n_active,n_active)
      dimension itemp(n_active),temp(n_active)

      data zero /0.0d0/
      
c
c v  ... DMO/AO coefficient for active space at the current geometry
c        return after ordering
c w  ... Original CAS MO/AO coefficient 
c s  ... AO overlap
c mcmoslab_active ... irreps labels of active MOs. retrun after ordering
c avocc..averaged DM occ # for active space
c        return after sorting
c
c

c
c n_active   ... # of active orbitals
c


       write(iw,*) '--- Ordering  of  DMO set starts' 
    

c
c ---- Check of deformation ------
c

c
c  Step(i)
c  Calculate overlap matrix between current DMO and original CAS MO
c

      do i=1,n_active
        ishift=num*(i-1)
        do j=1,n_active
          jshift=num*(j-1)
          dum=zero
          do k=1,num
            do l=1,num
              if (l.le.k) then
                ind_kl=ia(k)+l
              else 
                ind_kl=ia(l)+k
              end if
              dum=dum+v(k+ishift)*w(l+jshift)*s(ind_kl)
            end do
          end do 
          store_dot1(i,j)=dum
        end do
      end do
 


c
c Search maximum overlaping 
c

      do k=1,n_active
        dum_max=zero
        do i=1,n_active
          do j=1,n_active
            if (dum_max.lt.abs(store_dot1(i,j))) then
              dum_max=abs(store_dot1(i,j))
              if (zero.lt.store_dot1(i,j)) then
                ind_v=i
                istack(ind_v)=j
              else
                ind_v=i
                istack(ind_v)=-j
              end if
            end if
          end do
        end do
        ind_w=abs(istack(ind_v))
        do j=1,n_active
          store_dot1(ind_v,j)=zero ! input dummy value
          store_dot1(j,ind_w)=zero
        end do
      end do
         

         

!
! Now istack is complete sort index table for DMOs by ordering
! deforming
!

c
c Reorder DMOs/AO (active) with rearrenging sigin
c

      do i=1,n_active
        ishift=num*(i-1)
        if (0.lt.istack(i)) then
          istshift=num*(istack(i)-1)
          do j=1,num
            w(j+istshift)=v(j+ishift)
          end do
        else 
          istshift=num*(-istack(i)-1)
          do j=1,num
            w(j+istshift)=-v(j+ishift)
          end do
        end if
      end do

      do i=1,num*n_active
        v(i)=w(i)
      end do



c
c Reorder symmetry label and occupation number of averaged density 
c matrix
c

      do i=1,n_active
        itemp(i)=moslab_active(i)
        temp(i)=avocc(i)
      end do

      do i=1,n_active
        ind=abs(istack(i))
        moslab_active(ind)=itemp(i)  
        avocc(ind)=temp(i)  
      end do


      write(iw,*) '--- End of Ordering of DMOs  ---' 
         
      return


      end 

      SUBROUTINE DMSETMOD(ishift_paired)         
      IMPLICIT REAL*8 (A-H,O-Z)
      parameter(ndm_size=16)                               !HN 01/21/02
      LOGICAL OUT
C
C     ----- INITIALIZE AND PREPARE TO MAKE THE DENSITY MATRIX -----
C
c The original is DMSET, but slightly modified to calculate 
c and extract all type 1-DM
c 
c

       COMMON/WFNPRT/NPFLG1,NPFLG2,NPFLG3,NPFLG4,NPRINT,
     1               NPFLG6,NPFLG7,NPFLG8,NPFLG9,NPFLG0
       COMMON/LOOPS1/ACOEF,NWKS,NORBS,IB,JB,IUWK,JUWK,NUWK,NLWK
       COMMON/IOFILE/IR,IW
       DIMENSION IOFF(128)
       dimension ioff_dum(128)                             !HN 01/21/02
       COMMON/SCM/DM(1)
       DIMENSION INDX(1),VEC(1)
       dimension full_DM(ndm_size,ndm_size)                !HN 01/21/02
cc    ---  NORBS < ndm_size
       EQUIVALENCE (DM(1),INDX(1)),(DM(1),VEC(1))
       DATA ZERO /0.0D+00/
       DATA PT5  /0.5D+00/

c
c     ----- check size of some varaibles ---
c
c?      if ( (norbs.gt.ndm_size).or.(norbs*(norbs-1)/2.gt.128) ) then
      if ( norbs.gt.ndm_size ) then
        write(iw,*) 'too large # of valence orbiyals. STOP'
        stop
      end if
       

C
C     ----- ZERO OUT THE DENSITY MATRIX -----
C
       MIN = NWKS + NWKS + 1
       max = min-1 + norbs*(norbs+1)
       do ik=min,max
         dm(ik)=zero 
       end do

C
C     ----- ACCOUNT FOR POSITION OF DENSITY MATRIX -----
C
! dm(ioff(i)+j) is DM1(i,j) of lower triangle part and
! dm(ioff_dum(i)+j) is DM1(j,i) of upper triangle part.
       do i=1,128
         ioff(i)=(i*(i-1))/2+(nwks+nwks)
         ioff_dum(i)=ioff(i)+(norbs+1)*norbs/2
       end do

       ICOUNT=0

       RETURN
C
C     ----- CREATE THE DENSITY MATRIX -----
C

       ENTRY MAKEDMMOD(ishift_paired) !add                  !RV 9/21/06

       ICOUNT=ICOUNT+1
C
       II0 = NWKS
       JJ0 = NWKS
       IND = IUWK
       JND = JUWK
       DC = ZERO
       DO 140 I=1,NLWK
       II = INDX(IND) + II0
       JJ = INDX(JND) + JJ0
       DO 120 J=1,NUWK
       DC = DC + VEC(II+ishift_paired) * VEC(JJ) !add
       II = II + 1
 120   JJ = JJ + 1
       IND = IND + 1
 140   JND = JND + 1

       IF(IUWK.NE.JUWK) DC=DC+DC 
       LAD = IOFF(IB) + JB
C
       DM(LAD) = DM(LAD) + ACOEF * DC
       

cc -- create lower trinagle of DM1 too ---                 !HN 01/21/02

       ii0=nwks
       jj0=nwks
       ind=iuwk
       jnd=juwk
       dc=zero
       
       do i=1,nlwk
         ii=indx(ind)+ii0
         jj=indx(jnd)+jj0
         do j=1,nuwk
           dc=dc+vec(jj+ishift_paired)*vec(ii) 
           ii=ii+1
           jj=jj+1
         end do
         ind=ind+1
         jnd=jnd+1
       end do
       if (iuwk.ne.juwk) dc=dc+dc

       lad=ioff(ib)+jb
       lad_dum=lad+norbs*(norbs+1)/2
     
       dm(lad_dum)=dm(lad_dum)+acoef*dc
       
       RETURN


C
C     ----- FINISH UP -----
C
       ENTRY CLOSDMMOD(full_DM)                            !HN 01/21/02


       do i=1,norbs
         ind_scratch=ioff(i)+i
         full_DM(i,i)=dm(ind_scratch) !diagonal element
       end do

       do i=1,norbs
         do j=1,i
           if (i.ne.j) then
             ind_scratch=ioff(i)+j !lower triangle part except diag
             full_DM(i,j)=PT5*dm(ind_scratch) 
           end if
         end do
       end do
  
       do j=1,norbs
         do i=1,j
           if (i.ne.j) then
             ind_scratch=ioff_dum(j)+i !upper triangle part except diag
             full_DM(i,j)=PT5*dm(ind_scratch)
           end if 
         end do
       end do
         

       RETURN
       END
      subroutine fourfold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &                    nstate,num,n_active,nmlap,moslab_active,
     &                    moslab_charact,moslab_refdmo,dm,v,w,s,
     &                    avocc_act,temp)


      implicit real*8 (a-h,o-z)

c
c     This routine carry out the fourfold way 
c     and update MOs (categorized as "valence") then return.
c     Also all DM are transformed then return.
c

c
c Notice: the fourfold way is applied for the MOs of only the
c         valence block. For other block's MOs are not changed from
c         preliminary threefold density criterion's
c


  
      common/iofile/ir,iw,ip
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir

      data zero,one /0.0d0,1.0d0/

      dimension v(num,n_active)
      dimension w(num,nmlap)
      dimension s(num*(num+1)/2)

      dimension dm(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension moslab_active(n_active)
      dimension moslab_charact(n_active)
      dimension moslab_refdmo(n_active)
      dimension avocc_act(n_active)

      dimension indmo_mix(n_active)
      dimension msegment(n_active)
      dimension lab_block(n_active)
      
      dimension msegment_ref(nmlap)
      dimension labref_block(nmlap)

      dimension u(n_active,n_active) ! used only in this subroutine
      dimension temp(num,n_active)   ! temporary (max) region
      dimension itemp(n_active)

c
c nstate .... total # of states for diabatization 
c num  .... number of basis set (AO)
c n_active ... number of the active MOs
c nmlap ... number of reference DMOs for the MORMO criterion
c

c
c v  ... MO vec (active) : return as updated (threefold density's) MO
c w  .... reference MOs for the MORMO criterion 
c s  .... AO overlap
c dm ... Density matrices (including transition DM and SA-DM)
c (dm are stored as (11),(12),...(1 nstate),(22),(23),.... )
c moslab_active .... irreps label of the active MOs
c moslab_charact
c moslab_refdmo
c

c ---- Index arrays for current MOs ----
c
c lab_block(i) .... irreps label of MOs belong to i-th block 
c                  (lab_block(i).NE.lab_block(j) when i.NE.j)
c
c msegment(i).... number (dimension) of the i-th block to be mixed 
c
c indmo_mix ... indmo_mix(1:msegment(1))=MO index to be mixed for the
c                                        threefold density criterion
c indmo_mix(msegment(1)+1:msegment(1)+msegment(2))=MO index of the
c                                        next block
c               .... 
c                       where n_block (<n_active) is total # of block
c                      (defined bellow)
c 

c ---- Index arrays for reference DMOs ($DIAVEC)  ---
c
c labref_block(i)
c
c megment_ref(i)
c
c 



c
c Define the block to mixing MOs for MORMO criterion 
c based on information of irreps (by using moslab_active) and  
c valence block (by using moslab_charact)
c


      call mkmoind2(n_active,moslab_active,moslab_charact,
     &              indmo_mix,msegment,lab_block,n_block)


c
c Preliminary procedure for the MORMO criterion 
c Sort data in w ($DIAVEC) to have mach symmetry of block 
c to use the subroutine mormo
c


      call sort_mormo(num,nmlap,nref_block,moslab_refdmo,labref_block,
     &                msegment_ref,w,temp)
     

        
c
c Carry out the MORMO criterion to the valence block.
c The nmlap current MOs are determined at this stage.
c (In the subroutine mormo, MOs and DMs are updated.)
c
c Also n_block,lab_blcok,indmo_mix,and msegment are updated to be used
c in the next stage i.e. threefold density criterion for the REST MOs
c 
c


      call mormo(conv_th,max_iter,nstate,num,n_active,nmlap,n_block,
     &           nref_block,indmo_mix,msegment,lab_block,msegment_ref,
     &           labref_block,dm,v,w,s,temp)


c
c Carry out the threefold density criterion to the rest valence
c block MOs and determine MO/base transformation matrix U
c


cc Initialization of U (i.e. unit matrix)

      do i=1,n_active
        do j=1,n_active
          u(j,i)=zero
        end do
      end do
      do i=1,n_active
        u(i,i)=one
      end do


cc threefold density criterion for the rest MOs 
 
      call max_D3(alpha_n,alpha_r,alpha_t,conv_th,max_iter,nstate,
     &           n_active,n_block,indmo_mix,msegment,dm,u,avocc_act)


c
c Transform the MO coefficients of the new MO by using the matrix U
c 

      call dgemm('N','N',num,n_active,n_active,one,v,num,u,n_active,
     &            zero,temp,num)

      do i=1,n_active
        do j=1,num
          v(j,i)=temp(j,i)
        end do
      end do


      return

      end      


      subroutine gethdia(nrcsf,e_adia,nstdia,nlist,list_group,
     &           list_csf,dcicoef,dcicoef_ref,t,tp,ngrstate)         !RV 9/21/06

      implicit real*8 (a-h,o-z)

      parameter (maxlist=64)
      parameter (maxci=50)

      common/iofile/ir,iw,ip

      data zero /0.0d0/
      data one  /1.0d0/
      data err_th/1.0d-04/
    
      dimension list_group(maxlist)
      dimension list_csf(maxlist)
      dimension dcicoef_ref(maxlist,maxci)
      dimension dcicoef(maxlist,maxci)
      dimension dcicoef2(maxlist,maxci)
      dimension e_adia(maxci)
      dimension t(nstdia-ngrstate,nstdia-ngrstate)                   !RV 9/21/06
      dimension tp(nstdia-ngrstate,nstdia-ngrstate)                  !RV 9/21/06

!-- Arrays used within only this subroutine---
      dimension s(nstdia-ngrstate,nstdia-ngrstate)                   !RV 9/21/06
      dimension eigval(nstdia-ngrstate)                              !RV 9/21/06
      dimension eig_buff(nstdia-ngrstate)                            !RV 9/21/06
      dimension u(nstdia-ngrstate,nstdia-ngrstate)                   !RV 9/21/06
      dimension buff(nstdia-ngrstate,nstdia-ngrstate)                !RV 9/21/06
      dimension work(nlist,nstdia-ngrstate)                          !RV 9/21/06
!-- usually very small size -----------------


      
      L1=nstdia

c
c Symmetrized Orthogonalization for truncated CI satets
c

cc Make S (approximated overlap) matrix

      ii = 1 + ngrstate                                              !RV 9/21/06

      do istat=ii,L1
        do jstat=ii,L1
          s(istat-ngrstate,jstat-ngrstate)=zero                      !RV 9/21/06
          do k=1,nlist
            s(istat-ngrstate,jstat-ngrstate)=s(istat-ngrstate,       !RV 9/21/06
     $      jstat-ngrstate) +dcicoef(k,istat) *dcicoef(k,jstat)      !RV 9/21/06
          end do
        end do
      end do

cc Diagonalize S matrix

      call diajac(s,u,eigval,L1-ngrstate,L1-ngrstate,buff)           !RV 9/21/06

      do i=ii,L1
        if (eigval(i-ngrstate).lt.err_th) then                       !RV 9/21/06
          write(iw,'(''Cannot symmetric/orthogonalizate. Stop'')')
          stop
        end if
        eigval(i-ngrstate)=one/dsqrt(eigval(i-ngrstate))             !RV 9/21/06
      end do 


c
c Project to each (diabatic) subspace i.e group G_k
c 

      do k_grp=ii,L1
cc Make S_k matrix
        do istat=ii,L1
          do jstat=ii,L1
            s(istat-ngrstate,jstat-ngrstate)=zero                    !RV 9/21/06
            do k=1,nlist
              if (list_group(k).eq.k_grp) then
                s(istat-ngrstate,jstat-ngrstate)=s(istat-ngrstate,   !RV 9/21/06
     $          jstat-ngrstate) +dcicoef(k,istat)*dcicoef(k,jstat)   !RV 9/21/06
              end if
            end do
          end do
        end do
cc Make R_k matrix
        call dgemm('T','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,u, !RV 9/21/06
     $  L1-ngrstate,s,L1-ngrstate,zero,buff,L1-ngrstate)              !RV 9/21/06
        call dgemm('N','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,buff, !RV 9/21/06
     $  L1-ngrstate,u,L1-ngrstate,zero,s,L1-ngrstate)                    !RV 9/21/06
! Now s is Us^{t}*S_k*Us
        do i=ii,L1
          do j=ii,L1
            s(i-ngrstate,j-ngrstate)=eigval(i-ngrstate)               !RV 9/21/06
     $      *s(i-ngrstate,j-ngrstate)                                 !RV 9/21/06
          end do
        end do
        do j=ii,L1
          do i=ii,L1
            s(i-ngrstate,j-ngrstate)=eigval(j-ngrstate)*s(i-ngrstate, !RV 9/21/06
     $      j-ngrstate)                                               !RV 9/21/06
          end do
        end do

        call dgemm('N','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,u, !RV 9/21/06
     $  L1-ngrstate,s,L1-ngrstate,zero,buff,L1-ngrstate)              !RV 9/21/06
        call dgemm('N','T',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,buff, !RV 9/21/06
     $  L1-ngrstate,u,L1-ngrstate,zero,s,L1-ngrstate)                 !RV 9/21/06

! Now s is R_k matrix
cc Diagonalize R_k matrix

        call diajac(s,buff,eig_buff,L1-ngrstate,L1-ngrstate,t)        !RV 9/21/06

cc determine T' matrix 's k_grp-th column

        do i=ii,L1
          tp(i-ngrstate,k_grp-ngrstate)=buff(i-ngrstate,1)            !RV 9/21/06
        end do

      end do


c
c Symmetrized Orthogonalization of T' matrix i.e. Make Unitary 
c adiabatic/diabatic transform matrix
c

      call dgemm('T','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,tp,  !RV 9/21/06
     $ L1-ngrstate,tp,L1-ngrstate,zero,buff,L1-ngrstate)              !RV 9/21/06
      call diajac(buff,u,eigval,L1-ngrstate,L1-ngrstate,t)            !RV 9/21/06


      do i=ii,L1
        if (eigval(i-ngrstate).lt.err_th) then                        !RV 9/21/06
          write(iw,'(''Cannot symmetric/orthogonalizate. Stop'')')
          stop
        end if
        eigval(i-ngrstate)=one/dsqrt(eigval(i-ngrstate))              !RV 9/21/06 
      end do 

      call dgemm('N','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,tp,  !RV 9/21/06
     $ L1-ngrstate,u,L1-ngrstate,zero,s,L1-ngrstate)                  !RV 9/21/06

      do j=ii,L1
        do i=ii,L1
          buff(i-ngrstate,j-ngrstate)=eigval(j-ngrstate)              !RV 9/21/06
     $    *s(i-ngrstate,j-ngrstate)                                   !RV 9/21/06
        end do
      end do

      call dgemm('N','T',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,buff, !RV 9/21/06
     $ L1-ngrstate,u,L1-ngrstate,zero,t,L1-ngrstate)                   !RV 9/21/06
      

! Now t is adiabatic/diabatic transforma matrix (not sign arranged)


c
c Sign arrangement for each column vector of t and make ci coefficients
c for 'truncated' diabatic states
c

      if (nrcsf.eq.0) goto 100 ! skip this procedure 

! Reamrk for this  adjustment size 
      LDA=maxlist
      LDB=nstdia-ngrstate                                              !RV 9/21/06
      LDC=nlist

      do j=1,nlist
       do i=ii,nstdia
        dcicoef2(j,i-ngrstate)=dcicoef(j,i)                            !RV 9/21/06
       end do
      end do

      call dgemm('N','N',nlist,nstdia-ngrstate,nstdia-ngrstate,one,    !RV 9/21/06
     &     dcicoef2,LDA,t,LDB,zero,work,LDC)

      do k=ii,nstdia
        dum=zero
        do j=1,nlist
          dum=dum+work(j,k-ngrstate)*dcicoef_ref(j,k)                  !RV 9/21/06
        end do
        if (dum.lt.zero) then
          do j=ii,nstdia
            t(j-ngrstate,k-ngrstate)=-t(j-ngrstate,k-ngrstate)         !RV 9/21/06
          end do
          do j=1,nlist
            dcicoef(j,k)=-work(j,k-ngrstate)                           !RV 9/21/06
          end do
        else
          do j=1,nlist
            dcicoef(j,k)=work(j,k-ngrstate)                            !RV 9/21/06
          end do
        end if
      end do

! Now dcicoef is transformed to the ci coefficients of 
! diabatic states (Diabatic template)
          
     
100   continue


c
c Make diabatic Hamiltonian matrix
c
 
      do i=ii,L1
        do j=ii,L1
          buff(i-ngrstate,j-ngrstate)=t(j-ngrstate,i-ngrstate)         !RV 9/21/06
        end do
      end do

      do j=ii,L1
        do i=ii,L1
          buff(i-ngrstate,j-ngrstate)=e_adia(j)                        !RV 9/21/06
     &    *buff(i-ngrstate,j-ngrstate)                                 !RV 9/21/06
        end do
      end do

      call dgemm('N','N',L1-ngrstate,L1-ngrstate,L1-ngrstate,one,      !RV 9/21/06
     $ buff,L1-ngrstate,t,L1-ngrstate,zero,tp,L1-ngrstate)             !RV 9/21/06
! Now tp is diabatic Hamiltonoan matrix    

c
c Print out Diabatic Hamiltonian matrix elements
c

      write(iw,'(''------------------------------------------''/
     &           '' - Diabatic Hamiltonian matrix elements -   ''/
     &           ''-------------------------------------------''/
     &           ''                                           ''/
     &           ''--- Diabatic Energies (diagonal element) ---'')')

      do i=ii,nstdia
        write(iw,'('' Diabatic state # (Group #}'',i2,4x,'' Energy='',
     &             f18.9)')  i,tp(i-ngrstate,i-ngrstate)              !RV 9/21/06
      end do 
      
      write(iw,'(''                                              ''/
     &          ''--- Diabatic Couplings (off diagonal elements)---'')')

      do i=ii,nstdia-1
        do j=i+1,nstdia
          write(iw,'('' State #'',''('',i2,i2,'') Coupling ='',f12.9)')
     &    i,j,tp(i-ngrstate,j-ngrstate)                               !RV 9/21/06
        end do
      end do


c
c Print out Diabatic templates 
c

      write(iw,*) '     '
      write(iw,'(''------------------------------------------''/
     &           ''      -   Diabatic State -            ''/
     &           '' -------------------------------------------'')')

      do i=ii,nstdia
        write(iw,'('' State # '',i2,/
     &             ''------------------------'')') i
        write(iw,'('' <CSF>      <Coefficient> '')')
        do j=1,nlist
          if (list_group(j).eq.i) then
            write(iw,'(2x,i5,3x,f12.6)') list_csf(j),dcicoef(j,i)
          end if
        end do
      end do

c
c Print out Transformation matrix
c
      write(iw,*) '                    '
      write(iw,'(''------------------------------------------''/
     &      ''      - Transformation matrix elements -   ''/
     &      '' -------------------------------------------'')')

      do i=ii,nstdia
        do j=ii,nstdia
          write(iw,'(''('',i2,i2,'') = '',5e15.8)') i,j,
     &    t(i-ngrstate,j-ngrstate)                                   !RV 9/21/06
        end do
      end do



      return

      end


      SUBROUTINE GUGADGMOD(offsym,nstry) 
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXIODA=255)
      PARAMETER (MXATOM=128)
      PARAMETER (NWKMOD=512) 
      PARAMETER (MAXCI=50)
      CHARACTER*8 ERRMSG
      LOGICAL SOME,OUT,DBUG
      LOGICAL GUESS
      LOGICAL NOFIL,DAFIL
      logical offsym 
      INTEGER DIAFIL
c!      COMMON/OUTPUT/IPRINT
      COMMON/MEMORY/MAXCOR,MAXLCM
      COMMON/IOFILE/IR,IW,IP
      COMMON/SQFILE/IJKO,IJKT
      COMMON/DAFILE/IDAF,NAV,IODA(MXIODA)
      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
c!      COMMON/WFNPRT/NPFLG1,NPFLG2,NPFLG3,NPRINT,NPFLG5,
c!     1              NPFLG6,NPFLG7,NPFLG8,NPFLG9,NPFLG0
      COMMON/HNDFIL/NEWFIL,IFT13,IFT14
      COMMON/IJPAIR/IA(1)
      COMMON/INFOA/NAT,ICH,MUL,NUM,NX,NE,NA,NB,ZAN(MXATOM),
     1                                         C(3,MXATOM)
      COMMON/SCM/X(1)
      COMMON/LCM/XL(1)
      DIMENSION EIG(NWKMOD)
      DIMENSION ARRAY(5)
      DIMENSION TITLE(10),TITLE1(10)
      DIMENSION TITLA(10),TITLA1(10)
      DIMENSION TITLB(10),TITLB1(1)
c!      DIMENSION ERRMSG(3)
c!      DATA ERRMSG       /'PROGRAM ','STOP IN ','-GUGADG-'/
      DATA ZERO,PT5,ONE /0.0D+00,0.5D+00,1.0D+00/
      DATA NFT11  /11/
      DATA NFT12  /12/
      DATA NFT15  /15/
      DATA NFT16  /16/
      DATA IDAF20 /20/
      DATA DIAFIL /0/
      DATA NPFLG  /0/
      DATA NWORD  /0/
      DATA NCORE  /0/
      DATA NDAVID /0/
      DATA NOVEC  /0/
      DATA PRTTOL /8.5D-02/
c!      DATA NSTATE /1/                                    !HN 2/11/02
      DATA MNXPAN /1/
      DATA MXXPAN /30/
      DATA MAXIT  /50/
      DATA CVGTOL /5.0D-06/
C
      NAMELIST /GUGDIA/ NSTATE,MNXPAN,MXXPAN,MAXIT,CVGTOL,PRTTOL,
     1                  NDAVID,NCORE,NWORD,NOVEC,DIAFIL,NPFLG

c--
c This original of this routine is GUGADG.
c 
c--

      nstate=1 ! dummy                                      !HN 2/11/02

C
C     ----- READ NAMELIST - $GUGDIA - -----
C
      REWIND IR
      READ(IR,GUGDIA,END=5,ERR=5)
    5 CONTINUE

c!      SOME=NPRINT.NE.-5.AND.IPRINT.NE.-5
c!      OUT =NPRINT.GT.0 .OR .NPFLG .GT.0
c!      DBUG=NPRINT.GT.1 .OR .NPFLG .GT.1
c!      IF(DBUG) OUT =.TRUE.
c!      IF(OUT ) SOME=.TRUE.

!--- No print out for debug information --
      out=.false.  
      some=.false.
!-----------------------------------------
     
C
c!      IF(SOME) WRITE(IW,9999)
      if (offsym) then  
        nstate=nstry
      end if

      if (nstate.lt.2) then                                 !HN 2/11/02
        write(iw,*) 'Error. NSTATE must be larger than 1'   !HN 2/11/02
        stop                                                !HN 2/11/02
      end if                                                !HN 2/11/02 

C
      IF(NSTATE.GT.MAXCI   ) NSTATE=MAXCI
      IF(MNXPAN.LT.NSTATE  ) MNXPAN=NSTATE
      IF(MXXPAN.GT.NWKMOD  ) MXXPAN=NWKMOD
      IF(MXXPAN.LE.3*NSTATE) MXXPAN=3*NSTATE+1
      NSTAT=NSTATE
      NOFIL =(DIAFIL.EQ.0).AND.(NEWFIL.NE.0)
      NEWFIL=0
C
C     ----- GET CORE MEMORY -----
C
      CALL CMEM(LOADCM)
      LOCX1=LOCCM(X(1))
      NGOTMX=MAXCOR-LOCX1
      NGOT=NWORD
      IF(NGOT.LE.0) NGOT=NGOTMX
      NEED=LOCX1+NGOT
      CALL SETC(NEED)
C
      CALL REWFIL(NFT15)
      CALL REWFIL(NFT16)
      READ(NFT15) ISET1,NWKS,NEMEMX,CUTOFF,TITLA,TITLA1
      READ(NFT16) ISET2,NWKS,NEMEMX,CUTOFF,TITLE,TITLE1
C
      IF(ISET1.NE.1.OR.ISET2.NE.2) GO TO 9000
      DO I=1,10
         IF(TITLE (I).NE.TITLA (I)) GO TO 9000
         IF(TITLE1(I).NE.TITLA1(I)) GO TO 9000
      ENDDO    
      IF(NSTATE.GT.NWKS) THEN
         NSTATE=NWKS
c!         IF(SOME) THEN  
c!            WRITE(IW,9986)
c!         ENDIF
      ENDIF
      IF(MNXPAN.GT.NWKS) THEN
         MNXPAN=NWKS
      ENDIF
      IF(NWKS.GT.NWKMOD) THEN
         NDAVID=1
      ENDIF
c!      IF(SOME) THEN
c!         WRITE(IW,9989) NSTATE,MNXPAN,MXXPAN,MAXIT,CVGTOL
c!         WRITE(IW,9997) TITLE,TITLE1,NWKS
c!      ENDIF

      if (offsym) then  
        write(iw,'(''--------------------------------------''/
     &          '' Number of walks in DMO/CI is '',i6,'' ''/
     &          ''---------------------------------------'')') nwks
      end if 

C
C     ----- CHECK AVAILABILITY OF -CI- VECTORS -----
C
      IF(NOFIL) GO TO 20
      CALL REWFIL(NFT12)
      READ(NFT12,END=20,ERR=20) NVEC12,NWKS12,TITLB,TITLB1
C
      IF(NVEC12.LT.NSTATE) GO TO 9500
      IF(NWKS12.NE.NWKS  ) GO TO 9500
      IF(NOVEC.NE.0) GO TO 20
c!      IF(SOME.OR.OUT) WRITE(IW,9980)
      GUESS=.TRUE.
      GO TO 30
   20 CONTINUE
      NVEC12=0
      NWKS12=0
      CALL REWFIL(NFT12)
      WRITE(NFT12) NSTATE,NWKS,TITLE,TITLE1
c!      IF(SOME.OR.OUT) WRITE(IW,9979)
      GUESS=.FALSE.
   30 CONTINUE
      MXX1= MXXPAN
      MXX2=(MXXPAN*(MXXPAN+1))/2
      MXX3= MXXPAN* MXXPAN
C
      IF(NDAVID.NE.0) GO TO 50
C
C     ----- CHECK WHETHER ALL THE EXPANSION VECTORS FIT IN CORE -----
C
      IF(NCORE.EQ.0) THEN
         NEED=     (NWKS*(NWKS+1))/2
         NEED=NEED+(MXX2+MXX2+MXX3+MXX1)
         NEED=NEED+(NWKS+NWKS+NEMEMX+2*NEMEMX+NWKS)
         NEED=NEED+ NWKS*NSTATE
         LEFT=NGOT-NEED
         NVECMX=LEFT/NWKS
         MXSTAT=NVECMX/2
         IF(MXSTAT.GT.MXXPAN) GO TO 100
      ELSE
         NEED=     (NWKS*(NWKS+1))/2
         NEED=NEED+ NWKS* NWKS
         NEED=NEED+ NWKS
         NEED=NEED+ NWKS
         NEED=NEED+(NWKS+NWKS+NEMEMX+2*NEMEMX)
         LEFT=NGOT-NEED
         IF(LEFT.GE.0) GO TO 150
      ENDIF
C
C     ----- OUT OF CORE DIAGONALIZATION -----
C
   50 CONTINUE
C
C     ----- CONSTRUCT DIAGONAL MATRIX ELEMENTS -----
C
c!      IF(OUT) WRITE(IW,9996)
      NDAR20=(2*MXXPAN+1)
      IODA20=1
      I10=IODA20+NDAR20
      I50=I10+(MXX2+MXX2+MXX3+MXX1)
      I60=I50+NWKS
      I70=I60+NWKS
      I80=I70+NEMEMX
      I90=I80+NEMEMX*2
      NUSED=I90-1
      IF(NUSED.GT.NGOT) GO TO 9100
C
      CALL DGHII(NFT15,NWKS,X(I50),X(I60),NEMEMX,X(I70),X(I80))
      HMIN=ZERO
      DO IWKS=1,NWKS
         IF(X(IWKS+I50-1).LT.HMIN) IMIN=IWKS
         IF(X(IWKS+I50-1).LT.HMIN) HMIN=X(IWKS+I50-1)
      ENDDO    
c!      IF(SOME) WRITE(IW,9988) HMIN
C
C     ----- FIND ROOTS OF CI-MATRIX BY DAVIDSON'S METHOD -----
C
c!      IF(OUT) WRITE(IW,9993)
      LEFT=NGOT-NUSED
      NVECMX=LEFT/NWKS
      MXSTAT=NVECMX/2
      MXSTAT = MIN(MXSTAT,MXXPAN)
      IF(MXSTAT.LT.NSTATE) GO TO 9200
      if (MXSTAT.LT.MNXPAN) then 
        write(iw,'(''Waring ... Min # of Expansion vector is reduced''/
     &        ''to'',i6)') mxstat
      end if 
      IF(MXSTAT.LT.MNXPAN) MNXPAN=MXSTAT
C
      IODA20=1
      I10=IODA20+NDAR20
      I20=I10+(MXXPAN*(MXXPAN+1))/2
      I30=I20+(MXXPAN*(MXXPAN+1))/2
      I40=I30+ MXXPAN* MXXPAN
      I50=I40+ MXXPAN
      I60=I50+ NWKS
      I70=I60+ NWKS
      I80=I70+ NEMEMX
      I90=I80+ NEMEMX*2
      J10=I90
      J20=J10+ NWKS*MXSTAT
      J30=J20+ NWKS*MXSTAT
      LAST=J30
      NUSED=LAST-1
      DAFIL = .TRUE.
      IF(NGOT-LAST.GT.2*MXXPAN*NWKS) THEN
         DAFIL = .FALSE.
         NUSED  = NUSED + 2*MXXPAN*NWKS
      ENDIF
c!      IF(SOME) WRITE(IW,9990) NGOT,NUSED,NEMEMX
c!      IF(SOME.AND.MXSTAT.LE.MAXCI) WRITE(IW,9981) MXSTAT
C
C     ----- OPEN SCRATCH -DA- FILE -----
C
      IF(DAFIL) THEN
         NAV20=0
         CALL DAOPEN(IDAF20,X(IODA20),NAV20,NDAR20)
      ENDIF
C
      IF(NVEC12.LT.MNXPAN) GUESS=.FALSE.
      IF(GUESS) CALL DGRDCI(NFT12,MNXPAN,NWKS,X(J10),X(J20))
C
C     ----- DAVIDSON'S DRIVER -----
C
      CALL DVDSON(NFT16,NSTATE,NWKS,X(J10),X(J20),X(I50),X(I60),
     1 NEMEMX,X(I70),X(I80),
     2 MNXPAN,MXXPAN,X(I10),X(I20),X(I30),X(I40),IA,
     3 IDAF20,X(IODA20),NAV20,MXSTAT,CVGTOL,MAXIT,
     4 SOME,OUT,DBUG,PRTTOL,GUESS,DAFIL,X(J30))
C
C     ----- CLOSE SCRATCH -DA- FILE -----
C
      IF(DAFIL) THEN
         CALL DACLOS(IDAF20,NAV20,IW)
         CALL DARTRN(IDAF20)
      ENDIF
C
C     ----- SAVE -CI- VECTORS ON -NFT12- -----
C
      CALL DGWTCI(NFT12,NSTATE,NWKS,X(J10),X(J20))
      DO ISTAT=1,NSTAT
         ECI(ISTAT)=X(ISTAT+I40-1)
      ENDDO
      C1=X(IMIN+J10-1)
C
      GO TO 200
C
  100 CONTINUE
C
C     ----- IN CORE -CI- DIAGONALIZATION -----
C
      I00=1
      I10=I00+(NWKS*(NWKS+1))/2
      I20=I10+(MXXPAN*(MXXPAN+1))/2
      I30=I20+(MXXPAN*(MXXPAN+1))/2
      I40=I30+ MXXPAN* MXXPAN
      I50=I40+ MXXPAN
      I60=I50+ NWKS
      I70=I60+ NWKS
      I80=I70+ NEMEMX
      I90=I80+ NEMEMX*2
      J10=I90+ NWKS
      J20=J10+ NWKS*MXXPAN
      J30=J20+ NWKS*MXXPAN
      J40=J30+ NWKS*NSTATE
      LAST=J40
      NUSED=LAST-1
c!      IF(SOME) WRITE(IW,9990) NGOT,NUSED,NEMEMX
c!      IF(SOME.AND.MXSTAT.LE.MAXCI) WRITE(IW,9981) MXSTAT
c!      IF(OUT) WRITE(IW,9996)
      CALL DGHII(NFT15,NWKS,X(I50),X(I60),NEMEMX,X(I70),X(I80))
      HMIN=ZERO
      DO IWKS=1,NWKS
         IF(X(IWKS+I50-1).LT.HMIN) IMIN=IWKS
         IF(X(IWKS+I50-1).LT.HMIN) HMIN=X(IWKS+I50-1)
      ENDDO
c!      IF(SOME) WRITE(IW,9988) HMIN
C
c!      IF(OUT) WRITE(IW,9983)
      CALL DGHIJ(NFT16,NWKS,X(I00),X(I60),NEMEMX,
     1                      X(I70),X(I80),X(I90) )
      IF(OUT) CALL PRTR(X(I00),NWKS)
C
      IF(NVEC12.LT.MNXPAN) GUESS=.FALSE.
      IF(GUESS) CALL DGRDCI(NFT12,MNXPAN,NWKS,X(J10),X(J20))
C
C     ----- DAVIDSON'S DRIVER -----
C
      CALL DVDSNC(NSTATE,NWKS,X(I00),X(I50),X(I90),
     1 X(J10),X(J20),X(J30),
     2 MNXPAN,MXXPAN,X(I10),X(I20),X(I30),X(I40),IA,
     3 CVGTOL,MAXIT,SOME,OUT,DBUG,PRTTOL,GUESS)
C
C     ----- SAVE -CI- VECTORS ON -NFT12- -----
C
      CALL DGWTCI(NFT12,NSTATE,NWKS,X(J10),X(J20))
      DO ISTAT=1,NSTAT
         ECI(ISTAT)=X(ISTAT+I40-1)
      ENDDO
      C1=X(IMIN+J10-1)
C
      GO TO 200
C
  150 CONTINUE
C
C     ----- USE DENSE MATRIX DIAGONALIZATION -----
C
      I00=1
      I10=I00+(NWKS*(NWKS+1))/2
      I20=I10+ NWKS* NWKS
      I30=i20+ NWKS
      I40=I30+ NWKS
      I50=I40
      I60=I50+ NWKS
      I70=I60+ NWKS
      I80=I70+ NEMEMX
      I90=I80+ NEMEMX*2
c!      IF(SOME) THEN
c!         WRITE(IW,9978)
c!      ENDIF
      CALL DGHII(NFT15,NWKS,X(I50),X(I60),NEMEMX,X(I70),X(I80))
      HMIN=ZERO
      DO IWKS=1,NWKS
         IF(X(IWKS+I50-1).LT.HMIN) IMIN=IWKS
         IF(X(IWKS+I50-1).LT.HMIN) HMIN=X(IWKS+I50-1)
      ENDDO
c!      IF(SOME) WRITE(IW,9988) HMIN
c!      IF(OUT ) WRITE(IW,9983)
      CALL DGHIJ(NFT16,NWKS,X(I00),X(I60),NEMEMX,
     1                      X(I70),X(I80),X(I90) )
      IF(OUT ) CALL PRTR(X(I00),NWKS)
      CALL DGDIAG(X(I00),X(I10),EIG,X(I30),NSTATE,NWKS,NWKS,
     1            X(I50),X(I00))
      DO ISTAT=1,NSTATE
         X(ISTAT+I40-1)=EIG(ISTAT)
      ENDDO
C
C     ----- SAVE -CI- VECTORS ON -NFT12- -----
C
      CALL DGWTCI(NFT12,NSTATE,NWKS,X(I10),X(I10))
      DO ISTAT=1,NSTAT
         ECI(ISTAT)=X(ISTAT+I40-1)
      ENDDO
      C1=X(IMIN+I10-1)
C
      J10=I10
      J20=I90
C
  200 CONTINUE
C
C     ----- PRINT MOST IMPORTANT ELECTRONIC CONFIGURATIONS -----
C
      IF(SOME) THEN       
         REWIND NFT11
         READ(NFT11) NORBMX,NORBS,NCORBS,NSYM,NROWS
         K10=J20
         K20=K10+NORBMX
         K30=K20+NORBMX
         K40=K30+NORBMX
         K50=K40+4*NROWS
         CALL DGPRCI(NFT11,NSTATE,NWKS,X(I40),X(J10),PRTTOL,
     1               X(K10),X(K20),X(K30),X(K40),
     2               NORBMX,4*NROWS,NROWS)
      ENDIF    
C
C     ----- SAVE DATA ON RECORD -2- OF -IDAF- -----
C
      SZ=(MUL-1)*PT5
      S2=SZ*(SZ+ONE)
      ARRAY(1)=EN
      ARRAY(2)=ECI(1)-EN
      ARRAY(3)=ECI(1)
      ARRAY(4)=SZ
      ARRAY(5)=S2
      CALL DAWRIT(IDAF,IODA,ARRAY,5,2,NAV)
C
C     ----- CALCULATE DAVIDSON'S CORRECTION FOR QUADRUPLE EXCITATIONS --
C
      ED=ECI(1)
      DED=HMIN-ED
      DEQ=(ONE-C1*C1)*DED
      EQ=ED-DEQ
c!      IF(SOME) WRITE(IW,9987) HMIN,ED,DED,DEQ,EQ
C
c!      IF(SOME) WRITE(IW,9995)
      IF(SOME) CALL TEXIT(2,2)
C
C     ---- RESET CORE MEMORY -----
C
      CALL CMEM(NGOTCM)
      IF(NGOTCM.NE.LOADCM) CALL SETC(LOADCM)
C
      CALL FIFLSH(IW)
C
      RETURN
 9000 CONTINUE
c!      WRITE(IW,9994) ISET1,TITLA,TITLA1,
c!     1               ISET2,TITLE,TITLE1
c!      CALL HNDERR(3,ERRMSG)

      write(iw,*) ' -NFT15- AND -NFT16- DO NOT MATCH. '
      write(iw,*) 'Program Stop In DMO/CI diagonalization step'
 9100 CONTINUE
c!      WRITE(IW,9992) NGOT,NUSED
      write(iw,'(''NOT ENOUGH CORE TO CONSTRUCT DIAGONAL CI-ELEMENTS.''/
     &      ''NGOT AND NUSED ='',2I10)') ngot, nused
c!      CALL HNDERR(3,ERRMSG)
      write(iw,*) 'Program Stop In DMO/CI diagonalization step'
 9200 CONTINUE
      NEED=NEED+2*NSTATE*NWKS
c!      WRITE(IW,9991) NGOT,NEED
c!      CALL HNDERR(3,ERRMSG)
      write(iw,'(''NOT ENOUGH CORE TO GET THE EIGENVECTORS.''/
     &      '' NGOT AND NEED ='',2i10)') ngot, need
      write(iw,*) 'Program Stop In DMO/CI diagonalization step'
 9500 CONTINUE
      write(iw,*) 'CI vec tape does not match the present run. Stop'
c!      WRITE(IW,9982)
c!      CALL HNDERR(3,ERRMSG)
      write(iw,*) 'Program Stop In DMO/CI diagonalization step'
C
c! 9999 FORMAT(/,10X,25(1H-),20X,72(1H-),
c!     1       /,10X,'CI-MATRIX DIAGONALIZATION',
c!     2         20X,'DAVIDSON-LIU METHOD. PROGRAM WRITTEN BY',
c!     3             ' B.R.BROOKS, MODIFIED BY M.DUPUIS',
c!     4       /,10X,25(1H-),20X,72(1H-))
c! 9997 FORMAT(/,' NAME OF THE ENERGY MATRIX ... ',10A8,
c!     1       /,' NAME OF THE -DRT- TABLE   ... ',10A8,
c!     2       /,' NUMBER OF CONFIGURATIONS  ... ',I10)
c! 9996 FORMAT(' CONSTRUCT DIAGONAL MATRIX ELEMENTS ')
c! 9995 FORMAT(' ...... END OF CI-MATRIX DIAGONALIZATION ......')
c! 9994 FORMAT(' -NFT15- AND -NFT16- DO NOT MATCH. ',/,
c!     1 ' NFT15 HEADER RECORD @ ISET1 = ',I5,2X,10A8,/,38X,10A8,/,
c!     2 ' NFT16 HEADER RECORD @ ISET2 = ',I5,2X,10A8,/,38X,10A8)
c! 9993 FORMAT(' FIND ROOTS OF ENERGY MATRIX ')
c! 9992 FORMAT(' NOT ENOUGH CORE TO CONSTRUCT DIAGONAL CI-ELEMENTS.',
c!     1 ' NGOT AND NUSED = ',2I10)
c! 9991 FORMAT(' NOT ENOUGH CORE TO GET THE EIGENVECTORS.',
c!     1 ' NGOT AND NEED = ',2I10)
c! 9990 FORMAT(/,' NUMBER OF WORDS AVAILABLE = ',I10,/,
c!     1         ' NUMBER OF WORDS USED      = ',I10,/,
c!     2         ' ENERGY MATRIX BUFFER SIZE = ',I10)
c! 9989 FORMAT(' NUMBER OF STATES REQUESTED = ',I5,
c!     1     /,' MIN. NUMB. OF EXPAN. VEC   = ',I5,
c!     1     /,' MAX. NUMB. OF EXPAN. VEC   = ',I5,
c!     2     /,' MAX. NUMB. OF ITERATIONS   = ',I5,
c!     3     /,' CONVERGENCE CRITERION      = ',E10.1)
c! 9988 FORMAT(/,' E(HF) = ',F20.12)
c! 9987 FORMAT(/,' DAVIDSON CORRECTION ( VALID FOR -CISD- ONLY )',
c!     1       /,' ---------------------------------------------',
c!     1       /,31H                       E(HF) = ,F18.9,
c!     2       /,31H                        E(D) = ,F18.9,
c!     3       /,31H        DE(D) = E(HF) - E(D) = ,F18.9,
c!     4       /,31H DE(Q) =(1.-C(1)**2) * DE(D) = ,F18.9,
c!     5       /,31H                  E( D + Q ) = ,F18.9)
c! 9986 FORMAT(' -NSTATE- RESET EQUAL TO -NWKS- ')           
c!9985 FORMAT(' WARNING... MIN. NUMB. OF EXPAN. VEC. IS REDUCED TO ',I5)
c! 9983 FORMAT(' CONSTRUCT OFF-DIAGONAL MATRIX ELEMENTS ')
c!9982 FORMAT(' -CI- VECTOR TAPE DOES NOT MATCH THE PRESENT RUN. STOP.')
c! 9981 FORMAT(' CORE COULD HANDLE ',I2,' STATE(S)')
c! 9980 FORMAT('    INITIAL GUESS -CI- VECTORS EXIST ON -NFT12-.')
c! 9979 FORMAT(' NO INITIAL GUESS -CI- VECTORS EXIST ON -NFT12-.')
c! 9978 FORMAT(' DENSE MATRIX DIAGONALIZATION ALGORITHM')
      END
      subroutine gugdmt(n_active)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXIODA=255)
      PARAMETER (MXATOM=128)
      PARAMETER (MAXCI=50)
      parameter(ndm_size=16) 
      LOGICAL ERROR
      COMMON/MEMORY/MAXCOR,MAXLCM
      COMMON/IJPAIR/IA(1)
      COMMON/IOFILE/IR,IW,IP
      COMMON/SQFILE/IJKO,IJKT
      COMMON/DAFILE/IDAF,NAV,IODA(MXIODA)
      COMMON/INFOA/NAT,ICH,MUL,NUM,NNX,NE,NA,NB,ZAN(MXATOM),
     1                                          C(3,MXATOM)
      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/LOOPS1/ACOF,NWKS,NORBS,IB,JB,IUWK,JUWK,NUWK,NLWK
      COMMON/SCM/H(1)
      common/diifil/idaf30,nav30,ioda30(maxci)             !HN 01/21/02
      common/diatmpfil/nft21,nft22,nft23,nft24             !HN 01/21/02

      DIMENSION TITLE(10),TITLE1(10)
      DIMENSION MAP(128),ISYM(128),LEVNR(128),LEVPT(128)
      DIMENSION ISTATE(MAXCI)
      dimension full_DM(ndm_size,ndm_size)                 !HN 01/21/02
      dimension map_rev(128),work(ndm_size,ndm_size)       !HN 01/21/02
      DATA ISTATE /1,49*0/
      DATA NWORD  /0/
      DATA NFT11  /11/
      DATA NFT12  /12/
      data zero   /0.0d0/

c
c The original of this routine is GUGADM, that calculates one electron
c density matrices of CI wave functions. 
c

!
! norbs.... # of valence orbitals (active space)
! This value is read from File (nft12) in this routine
!


c
c Set some parameters ! modefy ISTATE(i)
c
       do i=1,NSTATE
         ISTATE(i)=1
       end do

       do i=1,ndm_size
         do j=1,ndim_size
           full_DM(i,j)=zero  !zero clear
         end do
       end do

 
C
C     ----- GET CORE MEMORY -----
C
      CALL CMEM(LOADCM)
      LOCH1=LOCCM(H(1))
      NGOTMX=MAXCOR-LOCH1
      NGOT=NWORD
      IF(NWORD.LE.0) NGOT=NGOTMX
      NEED=LOCH1+NGOT
      CALL SETC(NEED)
C
C     ----- READ HEADER RECORD OF CI VECTOR TAPE -----
C
      CALL REWFIL(NFT12)
      READ(NFT12) NSTATE,NWKS,TITLE,TITLE1
c!      IF(SOME) WRITE(IW,9997) TITLE,TITLE1,NSTATE,NWKS
C
C     ----- READ -DRT- DATA -----
C
      CALL REWFIL(NFT11)
      READ(NFT11) NORBMX,NORBS,NCORBS,NSYM,NROWS,NWKS0
      READ(NFT11) TITLE
      ERROR=.FALSE.
      DO 20 I=1,10
      ERROR=ERROR.OR.(TITLE(I).NE.TITLE1(I))
   20 CONTINUE
      IF(ERROR.OR.NWKS.NE.NWKS0) GO TO 910
C
      n_active=norbs                                       !HN 01/21/02
      L1= NUM
      L2=(NUM*(NUM+1))/2
      L3= NUM* NUM
      IHA=1
      IHB=IHA+NWKS
      IHC=IHB+NWKS 
      IHD=IHC+L3
      IHE=IHD+L1
      IHF=IHE+L3
      IH1=IHF+L1
      IH2=IH1+NROWS
      IH3=IH2+NROWS*4
      IH4=IH3+NROWS
      IH5=IH4+NROWS
      IH6=IH5+NROWS
c---  h(ihbb) is scratch for paired CI vector to calculate transition DM
      ihbb=ih6+nrows*4                                     !HN 01/21/02
      last=ihbb+nwks                                       !HN 01/21/02
      ishift_paired=ihbb-ihb                               !HN 01/21/02
cc      LAST=IH6+NROWS*4                                   !NH 01/21/02
      IF(LAST.GT.NGOT) GO TO 920
C
C     ----- READ MORE -DRT- DATA -----
C
      NROWS4=NROWS*4
      CALL DMDRT(H(IHA),H(IH1),H(IH2),H(IH3),H(IH4),H(IH5),
     1 H(IH6),MAP,ISYM,LEVNR,LEVPT,NROWS,NROWS4,NWKS,NFT11)
      NFZC=0
      DO 50 I=1,NORBMX
      IF(MAP(I).LT.0) NFZC=NFZC+1
   50 CONTINUE

c
c Rewrite CI vectors to direct acsess file                 !HN 01/21/02
c
      idaf30=30                                            !HN 01/21/02
      nav30=0                                              !HN 01/21/02
      ndar30=nstate+1                                      !HN 01/21/02
      call raopen(idaf30,ioda30,nav30,ndar30)              !HN 01/21/02
        
      do kstat=1,nstate                                    !HN 01/21/02
        idar30=kstat                                       !HN 01/21/02
        call dmrdci(nft12,nwks,h(ihb))                     !HN 01/21/02
        call rawrit(idaf30,ioda30,h(ihb),nwks,idar30,nav30)!HN 01/21/02
      end do                                               !HN 01/21/02

c
c Make mapping array of MO's index between DRT and DA10 
c (core orbitals not included)
c

      do i=1,norbs                                         !HN 01/21/02
        map_rev(map(i+ncorbs))=i                           !HN 01/21/02
      end do                                               !HN 01/21/02

c
c Rewind nft21 to write all DM1                          
c

      rewind(nft21)                                        !HN 01/21/02

C
C     -----           LOOP OVER STATES            -----
C     ----- GET DENSITY MATRIX + NATURAL ORBITALS -----
C
      do kstat=1,nstate                                    !HN 01/21/02
        do istat=kstat,nstate                              !HN 01/21/02
C
C     ----- READ -CI- VECTOR ----- 
C
          call raread(idaf30,ioda30,h(ihb),nwks,kstat)     !HN 01/21/02
          call raread(idaf30,ioda30,h(ihbb),nwks,istat)    !HN 01/21/02
   
C
C     ----- INITIALIZE THE DENSITY MATRIX -----
C

cc check norbs*(norbs+1) < num*num
         if (norbs*(norbs+1).gt.num*num) then              !HN 01/21/02
           write(iw,*) 'Error too large # of valence orbitals STOP'
           stop                                            !HN 01/21/02
         end if                                            !HN 01/21/02

         call dmsetmod(ishift_paired)                      !HN 01/21/02
       
C
C     ----- MAKE THE DENSITY MATRIX -----
C
         call loopy1mod(H(IH1),H(IH2),H(IH3),H(IH4),H(IH5),H(IH6),
     1           NROWS,ISYM,LEVPT,LEVNR,NROWS4,ishift_paired)            !HN 01/21/02 !RV 9/21/06

C
C     ----- FINISH THE DENSITY MATRIX -----
C

cc work is DM1 whose index is defined by DRT

         call closdmmod(work)                              !HN 01/21/02

cc Sort (mapping). Now DM1's index agree with the sequence stored DA10

         do i=1,norbs                                      !HN 01/21/02
           do j=1,norbs                                    !NH 01/21/02
             full_DM(map_rev(i),map_rev(j))=work(i,j)      !HN 01/21/02
           end do                                          !HN 01/21/02
         end do                                            !HN 01/21/02
         

c
c Save matrix elements to FT21 temporary  
c

         write(nft21) full_DM                              !HN 01/21/02
         
        end do                                             !HN 01/21/02
      end do                                               !HN 01/21/02

cc--- end of loop over CI root pairs

c
c Close Direct access file  
c
      call raclos(idaf30,nav30,iw)
      call rartrn(idaf30)

C
c      IF(SOME) WRITE(IW,9996)
c      IF(SOME) CALL TEXIT(2,2)
C
C     ----- RESET CORE MEMORY -----
C
      CALL CMEM(NGOTCM)
      IF(NGOTCM.NE.LOADCM) CALL SETC(LOADCM)
      RETURN
  910 CONTINUE
      write(iw,'('' Error in CI vector tape and DRT tape ''/
     &         '' nwks, nwks0, title ='',2i7,''  '',10a8)')

      write(iw,*) 'Error in calculation of DM1'
      return

  920 CONTINUE
      MORE=LAST-NGOT

      write(iw,*) 'Not enough core to estimate DM1'       

      write(iw,*) 'Error in calculation of DM1'
      
      return

      end 
      SUBROUTINE LOOPY1MOD(NABC,IARC,NLWKS,NUWKS,PUWK,IWGHT,
     1 NROWS,ISYM,LEVPT,LEVNR,NROWS4,ishift_paired)                 !RV 9/21/06
      IMPLICIT REAL*8 (A-H,O-Z)
C
C     ----- THIS ROUTINE IS A STRIPPED DOWN VERSION OF LOOPY -----
C     ----- FOR THE ONE-ELECTRON LOOPS                       -----
C
      CHARACTER*8 ERRMSG
      INTEGER PUWK
       COMMON/IOFILE/IR,IW
       COMMON/LOOPS1/ACOF,NWKS,NORBS,IB,JB,IUWK,JUWK,NUWK,NLWK
       DIMENSION ISEGM(128),JSEGM(128),IMAIN(128),ISUB(128)
       DIMENSION ACOEF(128),IUWKMN(128),IUWKSB(128),ISHIFT(4)
       DIMENSION ISYM(128),LEVPT(128),LEVNR(128)
       DIMENSION NABC(NROWS),IARC(NROWS4),NLWKS(NROWS)
       DIMENSION NUWKS(NROWS),PUWK(NROWS),IWGHT(NROWS4)
       DIMENSION JSEGNR(3),JSEGPT(3),IARCMN(21),IARCSB(21)
       DIMENSION NXTSEG(21),JXT(21),JMN(21),JSB(21)
       DIMENSION COEFFS(20,5),CFS(100)
       DIMENSION ERRMSG(3)
       EQUIVALENCE (COEFFS(1,1),CFS(1))
       EQUIVALENCE (JXT(1),NXTSEG(1)),(JMN(1),IARCMN(1))
       EQUIVALENCE (JSB(1),IARCSB(1))
       DATA ERRMSG /'PROGRAM ','STOP IN ','-LOOPY1-'/
       DATA JSEGNR /7,14,21/
       DATA JXT    /0,0,0,2,2,3,3,0,0,2,2,2,2,3,0,0,3,3,3,3,2/
       DATA JMN    /2,3,4,3,4,2,4,1,2,1,2,3,4,2,1,3,1,2,3,4,3/
       DATA JSB    /2,3,4,1,2,1,3,3,4,1,2,3,4,3,2,4,1,2,3,4,2/
       DATA ZERO   /0.0D+00/
       DATA ONE    /1.0D+00/
       DATA TWO    /2.0D+00/
C
       JSEGPT(1)=0
       DO 130 I=1,2
 130   JSEGPT(I+1)=JSEGNR(I)
       DO 135 I=1,2
       DO 135 J=1,5
 135   COEFFS(I,J)=ZERO
C
C     ----- THESE ARE THE POSSIBLE SEGMENT COEFFICIENTS -----
C
       DO 140 I=3,20
       A = REAL(I-2)
       COEFFS(I,1)= ONE/A
       COEFFS(I,2)=-ONE/A
       COEFFS(I,3)=SQRT((A+ONE)/A)
       COEFFS(I,4)=SQRT(A/(A+ONE))
       COEFFS(I,5)=SQRT(A*(A+TWO))/(A+ONE)
 140   CONTINUE
       DO 150 K=1,4
       ISHIFT(K)=(K-1)*NROWS
 150   CONTINUE
       NLEVS=NORBS+1
C
C     ----- START THE MAIN LOOP -----
C
       DO 500 LEVI=2,NLEVS
       LEV=LEVI
       LEVM=LEV-1
       JSM=ISYM(LEVM)
       IAD=LEVM
       NR=LEVNR(LEV)
       NPT=LEVPT(LEV)
       DO 480 IROW=1,NR
       NPT=NPT+1
       ISEGM(LEV)=1
       ISEG=1
       IMN=NPT
       ISB=NPT
       KSEG=0
       KSEGMX=JSEGNR(ISEG)
       IUWKMN(LEV)=PUWK(NPT)
       IUWKSB(LEV)=PUWK(NPT)
       IMAIN(LEV)=NPT
       ISUB(LEV)=NPT
       NUWK=NUWKS(NPT)
       ACOEF(LEV)=ONE
C
C     ----- SEARCH FOR NEXT LOOP SEGMENT -----
C
 200   KSEG = KSEG + 1
       IF(KSEG.GT.KSEGMX) GO TO 440
       KMN = IARCMN(KSEG)
       IARPT = IMN + ISHIFT(KMN)
       KMN = IARC(IARPT)
       IF(KMN.EQ.0) GO TO 200
       KSB = IARCSB(KSEG)
       JARPT = ISB + ISHIFT(KSB)
       KSB = IARC(JARPT)
       IF(KSB.EQ.0) GO TO 200
       JSEGM(LEV) = KSEG
       IUWKMN(LEVM) = IUWKMN(LEV) + IWGHT(IARPT)
       IUWKSB(LEVM) = IUWKSB(LEV) + IWGHT(JARPT)
C
C      -----     HAVING FOUND A VALID SEGMENT    -----
C      ----- UPDATE THE VALUE OF THE COEFFICIENT -----
C
       GO TO (1,1,4,1,12,1,8,1,7,1,17,2,2,3,1,13,1,2,19,2,6),KSEG
   1   ACOEF(LEVM) = ACOEF(LEV)
       GO TO 20
   2   ACOEF(LEVM) = -ACOEF(LEV)
       GO TO 20
   3   IA=NABC(IMN)+2
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   4   ACOEF(LEVM)=ACOEF(LEV)+ACOEF(LEV)
       GO TO 20
   6   IA=NABC(IMN)+24
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   7   IA=NABC(IMN)+42
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   8   IA=NABC(IMN)+43
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   12  IA=NABC(IMN)+62
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   13  IA=NABC(IMN)+63
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   17  IA=NABC(IMN)+81
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   19  IA=NABC(IMN)+83
       ACOEF(LEVM)=ACOEF(LEV)*CFS(IA)
       GO TO 20
   20  CONTINUE
       IF(NXTSEG(KSEG).GT.0) GO TO 400
       IF(ISYM(LEVM).NE.JSM)GO TO 200
       IF(KMN-KSB) 300,380,300
C
C     ----- UPDATE ALL VARIABLES FOR THE NEW SEGMENT -----
C
 300   LEVL=LEVM
       KSEGMX=4
 310   LEV=LEVM
       LEVM=LEV-1
       IF(LEVM.GT.0) GO TO 315
       WRITE(IW,313)
 313   FORMAT(' PROBLEMS WITH PARTIAL SPACE')
       CALL HNDERR(3,ERRMSG)
       RETURN
 315   CONTINUE
       KSEG=0
       IMN=KMN
       IMAIN(LEV)=KMN
       ISB=KSB
       ISUB(LEV)=KSB
 320   KSEG = KSEG + 1
       IF(KSEG.GT.KSEGMX) GO TO 360
       IARPT = IMN + ISHIFT(KSEG)
       KMN = IARC(IARPT)
       IF(KMN.LE.0) GO TO 320
       JARPT = ISB + ISHIFT(KSEG)
       KSB = IARC(JARPT)
       IF(KSB.LE.0) GO TO 320
       JSEGM(LEV) = KSEG
       IMAIN(LEVM)=KMN
       IUWKMN(LEVM)=IUWKMN(LEV)+IWGHT(IARPT)
       ISUB(LEVM)=KSB
       IUWKSB(LEVM)=IUWKSB(LEV)+IWGHT(JARPT)
       IF(KMN-KSB) 310,340,310
C
C     -----             A LOOP HAS BEEN CONSTRUCTED             -----
C     ----- FIND ITS CONTRIBUTION AND ADD TO THE DENSTIY MATRIX -----
C
 340   IB = IAD
       JB = LEVL
       NLWK = NLWKS(KMN)
       IUWK = IUWKMN(LEVM)
       JUWK = IUWKSB(LEVM)
       ACOF = ACOEF(LEVL)
       CALL MAKEDMMOD(ishift_paired)                             !RV 9/21/06
       GO TO 320
 360   IF(LEV.EQ.LEVL) GO TO 440
       LEVM=LEV
       LEV=LEVM+1
       IMN=IMAIN(LEV)
       ISB=ISUB(LEV)
       KSEG=JSEGM(LEV)
       GO TO 320
 380   IB = IAD
       JB = LEVM
       NLWK = NLWKS(KMN)
       IUWK = IUWKMN(LEVM)
       JUWK = IUWKSB(LEVM)
       ACOF = ACOEF(LEVM)
       CALL MAKEDMMOD(ishift_paired)                             !RV 9/21/06
       GO TO 200
 400   CONTINUE
       LEV = LEVM
       LEVM = LEV - 1
       ISEG = NXTSEG(KSEG)
       ISEGM(LEV) = ISEG
       KSEG = JSEGPT(ISEG)
       IMN = KMN
       IMAIN(LEV) = KMN
       ISB = KSB
       ISUB(LEV) = KSB
       KSEGMX = JSEGNR(ISEG)
       GO TO 200
 440   CONTINUE
       IF(LEV.EQ.LEVI) GO TO 480
       LEVM = LEV
       LEV = LEVM + 1
       ISEG = ISEGM(LEV)
       IMN = IMAIN(LEV)
       ISB = ISUB(LEV)
       KSEG = JSEGM(LEV)
       KSEGMX = JSEGNR(ISEG)
       GO TO 200
 480   CONTINUE
 500   CONTINUE
       RETURN
       END
      subroutine max_D3(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &         nstate,n_active,n_block,indmo_mix,msegment,dm1,u,avocc)

      implicit real*8 (a-h,o-z)

      common/iofile/ir,iw,ip
c
c This subroutine maximize the D3 function of the threefold density
c criterion and determine MO/base transformatiom matrix.
c Also density matrices are updated.
c The restriction of MOs to be mixed with are defined by "indmo_mix"
c (as input).
c


      dimension indmo_mix(n_active)
      dimension msegment(n_active)
      dimension u(n_active,n_active)
      dimension dm1(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension avocc(n_active)

      dimension temp1(n_active)
      dimension temp2(n_active)
      dimension temp3(n_active)
      dimension temp4(n_active)
     

      data zero,one /0.0d0,1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/


      write(iw,'(''-------------------------------------------''/
     &           ''      Maximizing D3 function                ''/
     &           ''-------------------------------------------'')')

c
c Jacobi rotation for Maximizing D3 function start.
c The rotation is carried out for each block
c
      
      indhead=1
      do iblock=1,n_block  ! loop for the block
        do istep=1,max_iter ! loop for the one-sircle of the rotation
          call calsum_maxD3(nstate,n_active,dm1,alpha_n,alpha_r,alpha_t,
     &                    D_pre)
          indtail=indhead+msegment(iblock)-1
        
          do i=indhead,indtail-1 ! loop for applying the rotation matrix
! The rotation matrix mixes jac1 and jac2-th MOs as J(ij) step
            jac1=indmo_mix(i)
c            do j=i,indtail
            do j=i+1,indtail ! mod 11/1
              jac2=indmo_mix(j)
              call caldel_maxD3(nstate,n_active,dm1,alpha_n,alpha_r,
     &                          alpha_t,jac1,jac2,a_ij,b_ij)
              del_D=dsqrt(a_ij**2+b_ij**2)+a_ij 
              gamma_ij=pt25*dacos(-a_ij/dsqrt(a_ij**2+b_ij**2))
              if (b_ij.lt.zero) gamma_ij=-gamma_ij
              if (conv_th.lt.del_D) then
! transform density matrix element
                c=dcos(gamma_ij) 
                s=dsin(gamma_ij)

                do idm=1,nstate*(nstate+1)/2+1
                  tmp_ii=dm1(jac1,jac1,idm)*c**2
     &            -dm1(jac1,jac2,idm)*c*s
     &            -dm1(jac2,jac1,idm)*c*s
     &            +dm1(jac2,jac2,idm)*s**2

                  tmp_jj=dm1(jac1,jac1,idm)*s**2
     &            +dm1(jac1,jac2,idm)*c*s
     &            +dm1(jac2,jac1,idm)*c*s
     &            +dm1(jac2,jac2,idm)*c**2
               
                  tmp_ij=dm1(jac1,jac1,idm)*c*s
     &            -dm1(jac2,jac2,idm)*c*s
     &            +dm1(jac1,jac2,idm)*c**2
     &            -dm1(jac2,jac1,idm)*s**2

                  tmp_ji=dm1(jac1,jac1,idm)*c*s
     &            -dm1(jac2,jac2,idm)*c*s
     &            -dm1(jac1,jac2,idm)*s**2
     &            +dm1(jac2,jac1,idm)*c**2
              
                  do k=1,n_active
                    temp1(k)=dm1(jac1,k,idm)*c
     &              -dm1(jac2,k,idm)*s
                    temp2(k)=dm1(k,jac1,idm)*c
     &              -dm1(k,jac2,idm)*s
                    temp3(k)=dm1(jac1,k,idm)*s
     &              +dm1(jac2,k,idm)*c
                    temp4(k)=dm1(k,jac1,idm)*s
     &              +dm1(k,jac2,idm)*c
                  end do

                  do k=1,n_active
                    dm1(jac1,k,idm)=temp1(k)
                    dm1(k,jac1,idm)=temp2(k)
                    dm1(jac2,k,idm)=temp3(k)
                    dm1(k,jac2,idm)=temp4(k)
                  end do
            
                  dm1(jac1,jac1,idm)=tmp_ii
                  dm1(jac2,jac2,idm)=tmp_jj
                  dm1(jac1,jac2,idm)=tmp_ij
                  dm1(jac2,jac1,idm)=tmp_ji

                end do    


! Up Date Unitary MO-trans matrix
             
                do k=1,n_active
                  temp2(k)=u(k,jac1)*c-u(k,jac2)*s
                  temp4(k)=u(k,jac1)*s+u(k,jac2)*c
                  u(k,jac1)=temp2(k)
                  u(k,jac2)=temp4(k) 
                end do

              end if 

            end do ! end of loop of j
          end do   ! end of loop of i (i.e end of one circle )

          call calsum_maxD3(nstate,n_active,
     &                      dm1,alpha_n,alpha_r,alpha_t,D_aft)

          write(iw,'(i5,''  '',f10.5,''  '',f13.8)')istep,D_aft,
     &        D_aft-D_pre
          if (dabs(D_aft-D_pre).lt.conv_th) then
            write(iw,*)'The rotation of ',iblock,'-th block was done'
            write(iw,*) '      '
            goto 100
          end if 

        end do ! end of loop of istep 
 
        if (dabs(D_aft-D_pre).gt.conv_th) then 
          write(iw,*) 'Jacobi rotation was not converged. Stop '
          stop
        end if 

100   continue ! Done of optimization for one-block (iblock loop)

cc    to next block...
  
        indhead=indhead+msegment(iblock)

      end do ! end of loop of iblock 



c
c Store occupation # for state averaged density matrix 
c (i.e diagonal elements of SA DM1) for sorting DMOs
c

      n1=nstate*(nstate+1)/2+1

      do i=1,n_active
        avocc(i)=dm1(i,i,n1)
      end do


      return


      end


      subroutine mkmoind1(n_active,moslab_active,indmo_mix,msegment,
     &                    n_block)

      implicit real*8 (a-h,o-z)

c
c This subroutine makes index arrays that define the "mixing block"
c of the active MOs to apply the simle threefold density criterion,
c i.e., the blocks are defined by only the irreps label. 
c 

      dimension moslab_active(n_active)
      dimension indmo_mix(n_active)
      dimension msegment(n_active)
     
      dimension itemp(n_active)

c
c n_active ... number of the active MOs (input and output)
c n_block  ... number of the blocks (output)
c moslab_active ... irreps label
c 


c
c msegment(i).... number (dimension) of the i-th block to be mixed 
c
c indmo_mix ... indmo_mix(1:msegment(1))=MO index to be mixed for the
c                                        threefold density criterion
c indmo_mix(msegment(1)+1:msegment(1)+msegment(2))=MO index of the
c                                        next block
c               .... 
c                       where n_block (<n_active) is total # of block
c                      (defined bellow)
c 



       do i=1,n_active
         itemp(i)=moslab_active(i)
       end do


       icount=0
       ishift=0
       do i=1,n_active ! this loop is correlated to picking up one-block
         if (itemp(i).eq.0) goto 100  ! skip 0 is dummy value  
         icount=icount+1
         idumlab=itemp(i)
         jcount=0
         do j=1,n_active ! this loop is correlated to determination of
                         ! the members of the above one-block
           if (itemp(j).eq.idumlab) then
             jcount=jcount+1
             indmo_mix(jcount+ishift)=j
             itemp(j)=0 ! replace to null value for the loop of i
           end if
         end do
         msegment(icount)=jcount
         ishift=ishift+msegment(icount)
100      continue
       end do
       n_block=icount


       return

       end 




      subroutine mkmoind2(n_active,moslab_active,moslab_charact,
     &                    indmo_mix,msegment,lab_block,n_block)
     &                   

      implicit real*8 (a-h,o-z)

c
c This subroutine makes index arrays that define the "mixing block"
c of the active MOs to apply the MORMO criterion in the fourfold 
c way step.
c i.e., the blocks are defined by character (i.e.valence),
c       the irreps label of the current DMO and irreps label of the
c       reference DMO ($DIAVEC) 
c 

      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir

      dimension moslab_active(n_active)
      dimension moslab_charact(n_active)

      dimension lab_block(n_active)
      dimension indmo_mix(n_active)
      dimension msegment(n_active)
   
     
      dimension itemp(n_active)

c
c n_active ... number of the active MOs (input and output)
c n_block  ... number of the blocks (output)
c 


c
c msegment(i).... number (dimension) of the i-th block to be mixed 
c
c indmo_mix ... indmo_mix(1:msegment(1))=MO index to be mixed for the
c                                        threefold density criterion
c indmo_mix(msegment(1)+1:msegment(1)+msegment(2))=MO index of the
c                                        next block
c               .... 
c                       where n_block (< n_active) is total # of block
c                       (defined bellow)
c
c lab_blok(i) ... irreps label of i-th block 
c                 Notice: lab_block(i).NE.lab_block(j) (i.NE.j)
c


       do i=1,n_active
!         if (moslab_charact(i).eq.1) then
         if (moslab_charact(i).eq.nsymbol_val) then
           itemp(i)=moslab_active(i)
         else 
           itemp(i)=0 ! null (zero) dummy value
         end if 
       end do


       icount=0
       ishift=0
       do i=1,n_active ! this loop is correlated to picking up one-block
         if (itemp(i).eq.0) goto 100  ! skip. 0 is dummy value 
         icount=icount+1
         idumlab=itemp(i)
         lab_block(icount)=idumlab ! irreps of icount-th block 
         jcount=0
         do j=1,n_active ! this loop is correlated to determination of
                         ! the members of the above one-block
           if (itemp(j).eq.idumlab) then
             jcount=jcount+1
             indmo_mix(jcount+ishift)=j
             itemp(j)=0 ! replace to null value for the loop of i
           end if
         end do
         msegment(icount)=jcount
         ishift=ishift+msegment(icount)
100      continue
       end do
       n_block=icount


       return

       end 


      subroutine mkqdptvec(nft,civec,eigvec,temp,nwks,nst,coef_max1,
     &                     coef_max2)

      implicit real*8 (a-h,o-z)

      dimension civec(nwks,nst)
      dimension eigvec(nst,nst)
      dimension temp(nwks)
      dimension coef_max1(nst) ! max CI coef from HONDO
      dimension coef_max2(nst) ! max CI coef from MC-QDPT program

      data zero /0.0d0/

c
c Take consistencty for total sign of each CI wavefunctions 
c between by the results of two CI programs
c
c eigenvector of Heff is updated to the eigenvectors
c of Hilbert space spaned by the basis set which are -NTN- CAS-CI 
c wavefunctions, not MC-QDPT program's CAS-CI.
c


      do i=1,nst
        if (coef_max1(i)*coef_max2(i).lt.zero) then
          do j=1,nst
            eigvec(i,j)=-eigvec(i,j)
          end do
        end if
      end do


      
c
c Calculate new CI vectors of projected MC-QDPT wavefunctions
c

      do i=1,nst
        do j=1,nwks
          temp(j)=zero  ! zero clear
        end do
        do j=1,nst
          do k=1,nwks
            temp(k)=temp(k)+eigvec(j,i)*civec(k,j)
          end do
        end do
! CI vector of i-th projected MCQDPT wavefunction 
        write(nft) temp 
      end do


      return 


      end 
      subroutine mocoefmix(v,w,work,num,gamma)
      
      implicit real*8(a-h,o-z)

c
c mixing MO coefficients by using rotational matrix represented 
c by gamma
c


      dimension v(num),w(num),work(num,2)

      c=dcos(gamma)
      s=dsin(gamma)
  
      do i=1,num
        work(i,1)=c*v(i)-s*w(i)
        work(i,2)=s*v(i)+c*w(i)
      end do
      
      do i=1,num
        v(i)=work(i,1)
        w(i)=work(i,2)
      end do

        
      return
      end

      
      subroutine mormo(conv_th,max_iter,nstate,num,n_active,nmlap,
     &                n_block,nref_block,indmo_mix,msegment,lab_block,
     &                msegment_ref,labref_block,dm1,v,v_ref,s,temp)

      implicit real*8 (a-h,o-z)

      common/iofile/ir,iw,ip
      common/ijpair/ia(1)

c
c This subroutine carries out the MORMO criterion 
c All MOs are updated by the criterion, and also
c density matrices are updated in this routine.
c The restriction of MOs to be mixed with are defined by "indmo_mix"
c etc.
c


      dimension indmo_mix(n_active)
c      dimension msegment(n_active)
c      dimension msegment_ref(nmlap)
      dimension msegment(n_block)
      dimension msegment_ref(nref_block)


      dimension dm1(n_active,n_active,nstate*(nstate+1)/2+1)
c      dimension avocc(n_active)

      dimension s(num*(num+1)/2)
      dimension v(num,n_active)
      dimension v_ref(num,nmlap)

      dimension lab_block(n_block) ! lab_block(n_active)
      dimension labref_block(nref_block)

      dimension msegment_tmp(n_block)
      dimension temp(num,2) 
      dimension temp1(n_active)
      dimension temp2(n_active)
      dimension temp3(n_active)
      dimension temp4(n_active)
      dimension itemp(n_active)

      data zero,one /0.0d0,1.0d0/
      data pt5,pt25 /0.5d0,0.25d0/

c
c v ... MOs of current geometry (input). Output is updated in this 
c       subroutine and retruned
c
c v_ref ... reference DMO ($DIAVEC) (input & output)
c          (Note: this data is orderd by irreps as MOs of each block are
c                 labeled by unique irreps.)         
c                
c s ... AO overlap (input & output)
c 
c dm1 ... density matrices (input) of the current MOs. Output is updated
c         in this subroutine and retruned.
c

c
c n_block ... total # of block ( only within each block, Jacobi 
c             rotation is done)
c
c msegment(i)... number (dim) of the i-th block to be mixed
c
c indmo_mix ... MO index to be mixed for the MORMO criterion 
c 
c nref_block ... total # of block to be referenced (blocked by irreps)
c
c msegment_ref(i) ... number (dim) of the i-th block of reference DMO
c
c lab_block
c
c labref_block
c
c

c
c Set parameters
c

! ntot_val : total number of "valence" MOs i.e. to be used for MORMO
      ntot_val=0
      do i=1,n_block
        ntot_val=ntot_val+msegment(i)
      end do

c
c Copy
c

      do i=1,n_block
        msegment_tmp(i)=msegment(i)
      end do


      write(iw,'(''-------------------------------------------''/
     &           ''      Maximizing MORMO term                 ''/
     &           ''-------------------------------------------'')')


c
c Jacobi rotation for Maximizing reference overlap term (MORMO) is
c start.
c The rotation is carried out for each block.
c The block to be picked up first is reference DMO's ($DFMVEC) one
c and then suitable block of the current MOs is searched.
c The Jacobi rotation carried out within this current MOs's block
c
      
      ihead_ref=1 ! header index of the reference DMO of the iblock-th
                  ! block

      do iblock=1,nref_block  ! loop for the block of the reference DMOs
       
        n_overlap=msegment_ref(iblock) !number of ref DMOs in this block

c
c Pick up corresponding block of current MOs (index=n_search)
c Set up parameters for current MOs block
c 


        call srchmormo(n_search,ihead,n_mixmo,n_block,lab_block,
     &                 msegment,labref_block(iblock))

!
! n_search : index of current MO's block corresponding to the ref DMO
!            block
! n_mixmo  : number of MOs of the above block
!
! ihead    : header index of current MOs (physical address 
!                                         indmo_mix(indhead)      
!

cc Jacobirotation step

        do istep=1,max_iter ! loop for the one-sircle of the rotation

          call calsum_mormo(num,n_active,n_overlap,indmo_mix,
     &                      ihead,v_ref(1,ihead_ref),v,s,D_pre)
          do i=1,n_overlap
            jac1=indmo_mix(ihead+i-1) ! index of the current MO for i
            jac1_ref=ihead_ref+i-1   ! index of the reference DMO for i
            do j=i+1,n_mixmo
              jac2=indmo_mix(ihead+j-1) ! index of the current MO for j
              if (j.le.n_overlap) then
                jac2_ref=ihead_ref+j-1 !index of the reference DMO for j
                call caldel_mormo1(num,v(1,jac1),v(1,jac2),
     &              v_ref(1,jac1_ref),v_ref(1,jac2_ref),s,a_ij,b_ij)
              else 
                ! in this case, reference DMO for j is not necessary
                call caldel_mormo2(num,v(1,jac1),v(1,jac2),
     &              v_ref(1,jac1_ref),s,a_ij,b_ij)
              end if

              del_D=dsqrt(a_ij**2+b_ij**2)+a_ij 

              gamma_ij=pt5*dacos(-a_ij/dsqrt(a_ij**2+b_ij**2)) 

              if (conv_th.lt.del_D) then

! transform current MO coefficients

                call mocoefmix(v(1,jac1),v(1,jac2),temp,num,gamma_ij)

! update DMs
                cval=dcos(gamma_ij) 
                sval=dsin(gamma_ij)

                do idm=1,nstate*(nstate+1)/2+1
                  tmp_ii=dm1(jac1,jac1,idm)*cval*cval
     &            -dm1(jac1,jac2,idm)*cval*sval
     &            -dm1(jac2,jac1,idm)*cval*sval
     &            +dm1(jac2,jac2,idm)*sval*sval

                  tmp_jj=dm1(jac1,jac1,idm)*sval*sval
     &            +dm1(jac1,jac2,idm)*cval*sval
     &            +dm1(jac2,jac1,idm)*cval*sval
     &            +dm1(jac2,jac2,idm)*cval*cval
               
                  tmp_ij=dm1(jac1,jac1,idm)*cval*sval
     &            -dm1(jac2,jac2,idm)*cval*sval
     &            +dm1(jac1,jac2,idm)*cval*cval
     &            -dm1(jac2,jac1,idm)*sval*sval

                  tmp_ji=dm1(jac1,jac1,idm)*cval*sval
     &            -dm1(jac2,jac2,idm)*cval*sval
     &            -dm1(jac1,jac2,idm)*sval*sval
     &            +dm1(jac2,jac1,idm)*cval*cval
              
                  do k=1,n_active ! notice k should be run ALL active
                    temp1(k)=dm1(jac1,k,idm)*cval
     &              -dm1(jac2,k,idm)*sval
                    temp2(k)=dm1(k,jac1,idm)*cval
     &              -dm1(k,jac2,idm)*sval
                    temp3(k)=dm1(jac1,k,idm)*sval
     &              +dm1(jac2,k,idm)*cval
                    temp4(k)=dm1(k,jac1,idm)*sval
     &              +dm1(k,jac2,idm)*cval
                  end do

                  do k=1,n_active
                    dm1(jac1,k,idm)=temp1(k)
                    dm1(k,jac1,idm)=temp2(k)
                    dm1(jac2,k,idm)=temp3(k)
                    dm1(k,jac2,idm)=temp4(k)
                  end do
            
                  dm1(jac1,jac1,idm)=tmp_ii
                  dm1(jac2,jac2,idm)=tmp_jj
                  dm1(jac1,jac2,idm)=tmp_ij
                  dm1(jac2,jac1,idm)=tmp_ji

                end do    

              end if 

            end do ! end of loop of j
          end do   ! end of loop of i (i.e end of one circle )

          call calsum_mormo(num,n_active,n_overlap,indmo_mix,
     &                      ihead,v_ref(1,ihead_ref),v,s,D_aft)

          write(iw,'(i5,''  '',f10.5,''  '',f13.8)')istep,D_aft,
     &        D_aft-D_pre
          if (dabs(D_aft-D_pre).lt.conv_th) then
            write(iw,*) 'The rotation of ',iblock,'-th block was done'
            write(iw,*) '           '
            goto 100
          end if 

        end do ! end of loop of istep 
 
        if (dabs(D_aft-D_pre).gt.conv_th) then 
          write(iw,*) 'Jacobi rotation was not converged. Stop '
          stop
        end if 


100     continue ! Done of optimization for one-block (iblock loop)

cc Define (update) msegment_tmp array to use the final
c-- threefold density criterion, i.e, remove MOs determined by this 
c-- MORMO step

        msegment_tmp(n_search)=n_mixmo-n_overlap

cc    to next block...

        ihead_ref=ihead_ref+msegment_ref(iblock)

      end do ! end of loop of iblock 



c
c Update indmo_mix, msegment, and n_block by using msegment_tmp array
c


      do i=1,ntot_val
        itemp(i)=indmo_mix(i)
      end do

      ishift1=0
      ishift2=0
      do i=1,n_block
        ival=msegment(i)
        id3free=msegment_tmp(i)
        ioverlap=ival-id3free
        do j=1,id3free
          indmo_mix(j+ishift1)=itemp(j+ioverlap+ishift2)
        end do
        ishift1=ishift1+id3free
        ishift2=ishift2+ival
      end do


      do i=1,n_block
        itemp(i)=lab_block(i)
      end do

      icount=0
      do i=1,n_block
        if (msegment_tmp(i).ne.0) then
          icount=icount+1
          msegment(icount)=msegment_tmp(i)
          lab_block(icount)=itemp(i)
        end if
      end do

      n_block=icount


      return


      end

      subroutine orddmo(nft,num,n_active,nstate,moslab_charact,
     &                  moslab_active,dm,avocc,v,w,s)

      implicit real*8 (a-h,o-z)

c
c This subroutine determine final ordering ( after threefold or
c fourfold way) of DMOs by using $DFMVEC
c The target DMOs for ordering belong to "valence" block.
c The positions of other DMOs (doc, and vir block) are already
c determined before this statge, i.e. in the subroutine detblock.
c
c
      
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir

      dimension v(num,n_active)
      dimension s(num*(num+1)/2)
      dimension w(num,n_active)

      dimension moslab_charact(n_active)
      dimension moslab_active(n_active)
      dimension dm(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension avocc(n_active)

      dimension ind_mo(n_active)
    
      dimension indtemp0(n_active)
      dimension indtemp1(n_active)

 
c
c v .... DMOs at current geometry 
c w .... $DFMVEC (sym. orthogonalized) to be used for deformation check.
c
c


c
c Initializaton of index array ind_mo 
c
c Note: The order of Doc and Vir block MOs is already fixed, but 
c       to use ind_mo(1:n_active), this initialization is required.
c       


      do i=1,n_active
        ind_mo(i)=i
      end do
       
      
c
c Read sym. orthogonalized $DFMVEC (ndfrm=1) or av Fock orbitals 
c (ndfrm=0)
c

 
      rewind(nft)

      call rdfuldia(nft,num*n_active,w)
  

c
c Pick up Val block MOs
c                     

      n_val=0
      do i=1,n_active
!        if (moslab_charact(i).eq.1) then
        if (moslab_charact(i).eq.nsymbol_val) then
          n_val=n_val+1
          indtemp1(n_val)=i
        end if
      end do



c
c deformation check and sort
c

      do i=1,n_active
        indtemp0(i)=0
      end do

      call dfrmcheck(num,n_active,n_val,n_val,indtemp1,indtemp1,
     &               indtemp0,v,w,s)

cc Val part of ind_mo array is determined

      do i=1,n_active
        if (indtemp0(i).ne.0) ind_mo(i)=indtemp0(i)
      end do

c
c Now, ind_mo(1:n_active) array is completely determined 
c (i.e. All (doc+val+vir) block part DMOs are identified.)
c


c
c  Reorder the MOs 
c

      do  i=1,n_active
        if (ind_mo(i).lt.0) then 
          do j=1,num
            w(j,-ind_mo(i))=-v(j,i)
          end do
        else 
          do j=1,num
            w(j,ind_mo(i))=v(j,i)
          end do
        end if
      end do

      do i=1,n_active
        do j=1,num
          v(j,i)=w(j,i)
        end do
      end do

c
c
c Reorder elements of DMs (Notice: With sign change of MOs, each
c                                  DM elements change)
c

      do idm=1,nstate*(nstate+1)/2+1
cc--- change row and column
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
          else
            ind_i=ind_mo(i)
          end if
          do j=1,n_active
            if (ind_mo(j).lt.0) then
              ind_j=-ind_mo(j)
            else 
              ind_j=ind_mo(j)
            end if
            w(ind_j,ind_i)=dm(j,i,idm)
          end do
        end do
cc--- change sign of elements
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
            do j=1,n_active
              w(j,ind_i)=-w(j,ind_i)
            end do
          end if
        end do
        do i=1,n_active
          if (ind_mo(i).lt.0) then
            ind_i=-ind_mo(i)
            do j=1,n_active
              w(ind_i,j)=-w(ind_i,j)
            end do
          end if
        end do
cc--- up date dm         
        do i=1,n_active
          do j=1,n_active
            dm(j,i,idm)=w(j,i)
          end do
        end do
      end do

c
c Reorder irreps label
c

      do i=1,n_active
        indtemp1(i)=moslab_active(i)
      end do
      do i=1,n_active
        if (ind_mo(i).lt.0) then
          moslab_active(-ind_mo(i))=indtemp1(i)
        else 
          moslab_active(ind_mo(i))=indtemp1(i)
        end if
      end do
 
c
c Reorder avocc
c

      do i=1,n_active
        avocc(i)=dm(i,i,nstate*(nstate+1)/2+1)
      end do


     
!--- for debug ---
!      write(6,*) 'Debug in orddmo'
!      write(6,*) 'i, ind_mo(i)'
!      do i=1,n_active
!        write(6,*) i,ind_mo(i)
!      end do
!-----------------


      return 

      end




      subroutine presort(nstate,num,n_active,nb_can,n_refmo,indx1,
     &                   v,w,s,temp,dm,avocc,moslab_active)

      implicit real*8 (a-h,o-z)

c
c This routine sorts MOs within each (candidate) block,
c based on reference MOs ($DPSVEC or $DIAVEC)
c in order to accelate convergence of the MORMO step
c

c
c nb_can ... # of candidate MO of the block
c n_refmo ... # of reference MOs
c
c indx1(i) i-th MO of the specified block is indx1(i)-th
c          stored MO
c istack(i) i-th MO of the spcecified block is sorted 
c          to istack(i)-th MO of the specifeid block 
c
c temp(num,n_active)
c v(num,n_active) .... active MOs
c w(num,n_refmo)  .... reference MOs
c s(num*(num+1)/2)... AO overlap
c dm(n_active,n_active,nstate*(nstate+1)/2+1
c avocc(n_active)
c
   

      dimension v(num,n_active)
      dimension w(num,n_refmo)
      dimension s(num*(num+1)/2)
      dimension temp(num,n_active)
      dimension indx1(nb_can)
      dimension itemp(n_active)
      dimension istack(nb_can)
      dimension dm(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension avocc(n_active)
      dimension moslab_active(n_active)

      call dfrmcheck2(num,n_active,n_refmo,nb_can,
     &                indx1,istack,v,w,s)

!---- Debug
      write(6,*) 'Debug in presort'
      write(6,*) 'istack'
      do i=1,nb_can
        write(6,*) i,istack(i)
      end do
      write(6,*) 'actual position sort new<--- old' 
      do i=1,nb_can
        write(6,*) indx1(istack(i)),indx1(i)
      end do
!-----

c
c Sort
c

!--- MO ----

      do i=1,n_active
        do j=1,num
          temp(j,i)=v(j,i)
        end do
      end do
 
      do i=1,nb_can
        do j=1,num
          v(j,indx1(istack(i)))=temp(j,indx1(i))
        end do
      end do

!--- irreps labela--

      do i=1,n_active
        itemp(i)=moslab_active(i)
      end do
      do i=1,nb_can
        moslab_active(indx1(istack(i)))=itemp(indx1(i))
      end do


!--- DMs ---

      do idm=1,nstate*(nstate+1)/2+1
        do i=1,n_active
          do j=1,n_active
            temp(j,i)=dm(j,i,idm)
          end do
        end do
        do i=1,nb_can
          ind_a=indx1(i)
          ind_b=indx1(istack(i))
          do j=1,nb_can
            jnd_a=indx1(j)
            jnd_b=indx1(istack(j))
            dm(ind_b,jnd_b,idm)=temp(ind_a,jnd_a)
          end do
        end do
      end do
 
!--- average occ # --
      do i=1,nstate
        avocc(i)=dm(i,i,nstate*(nstate+1)/2+1)
      end do


      return 
 
      end 
     


      subroutine prlapdmo(nft,num,n_active,moslab_charact,v,v_ref,s)

      implicit real*8(a-h,o-z)

      common/iofile/ir,iw,ip
      common/ijpair/ia(1)

      dimension v(num,n_active)
      dimension w(num,n_active)
      dimension s(num*(num+1)/2)
      dimension moslab_charact(n_active)
 
      dimension store_dot(n_active,n_active)

      data zero,one /0.0d0,1.0d0/

c
c v ... the current DMO coefficients (input and output)
c w ... sym. orth. $DFMVEC
c s ... AO overlap (input and output)
c moslab_charact ... character of DMOs doc, val, vir
c store_dot(i,j) ... <$DIAVEC_i|DMO_j>
c


c
c Read sym. orth. $DFMVEC
c



      rewind(nft)
      call rdfuldia(nft,num*n_active,w)

c
c Calculate overlap-like quantity
c

      
      do i=1,n_active
        do j=1,n_active
          store_dot(j,i)=zero
        end do
      end do


      do i=1,n_active
        do j=1,n_active
          dum=zero
          do k=1,num
            do l=1,num
              if (l.le.k) then
                ind_kl=ia(k)+l
              else
                ind_kl=ia(l)+k
              end if
              dum=dum+w(k,i)*v(l,j)*s(ind_kl)
            end do
          end do
          store_dot(i,j)=dum
        end do
      end do


c
c Print out overlap-like quantity matrix
c

      do i=1,n_active
        write(iw,*)'Between ',i,' th symmetirc orthogonalized $DFMVEC'
        write(iw,9800) (store_dot(i,j),j=1,n_active)
      end do


      return

9800  format(1x,5f8.5)

      end

      subroutine prmcdia(nrcsf,NFT11,NFT12,nft21,nst,NWKS,
     1        eigval,EIGVEC,PRTTOL,LEVIR,ICASE,IECONF,IECASE,IARC,
     2        list_csf,list_group,dcicoef,nlist,NOMX,NROWS4,nrows)
                      
      IMPLICIT REAL*8 (A-H,O-Z)

      parameter (maxlist=64)

      COMMON/IOFILE/IR,IW,IP
      COMMON/ORBDRT/NORBMX,NORBS,NCORBS,NLEVS,NA,NB,NC,NSYM,MSYM,
     1 IACT,IDOC,IVAL,IMCC,ICFA,ICFB,MCFA,MCFB,IGVB(10),NGVB,
     2 ISYM(128),ICODE(128),NLCS(128),LEVPT(128),LEVNR(128),IOUT(128),
     3 NREFS,IEXCT,NFOCI,ICF,INTACT,IPP,NOPSH

      data zero /0.0d0/
      DIMENSION EIGVEC(NWKS), EIGVAL(nst)
      DIMENSION LEVIR(NOMX),ICASE(NOMX),IECONF(NOMX),IECASE(NOMX),
     1          IARC(NROWS4)

      DIMENSION ISHIFT(4)
      DIMENSION IOUTIV(128)

      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,nst)
      dimension tab_coef(maxlist)


c
c Zero clear (input dummy value) for list array, if list_csf etc
c are not given as input (i.e. nrcsf = 0 )
c

      if (nrcsf.eq.0) then
        do i=1,maxlist
          list_csf(i)=0
          list_group(i)=0
          tab_coef(i)=zero
        end do
      end if 

C
C     ----- CREATE INVERSE MAPPING OF ORBITALS -----
C
      N=0
      do I=1,128
         IF(IOUT(I).LT.0) N=N+1
      end do

      do I=1,128
         IF(IOUT(I).GT.0) THEN
            IOUTIV(IOUT(I))=I-N
         END IF
      end do

C
C     ----- PRINT COEFFICIENTS OF CI VECTORS LARGER THAN PRTTOL -----
C           THE ELECTRON OCCUPANCY IS ALSO PRINTED
C
      do  K = 1,4
        ISHIFT(K) = (K-1)*NROWS
      end do

      REWIND NFT11
      do I=1,13
        READ(NFT11) ! skip Header etc
      end do
      READ(NFT11) IARC
      REWIND NFT11
C
      NCORB = 0
      do I=1,NORBMX
        IF(IOUT(I).LT.0) NCORB = NCORB+1
      end do
C

c
c Search Domain CSF and make some list for grouping by reading CI 
c vectors
c
      rewind(nft21) 
      if (nft21.eq.nft12) read(nft21) idum,idum ! FT12 has header

      icount=0
      do  KSTAT = 1,nst
        READ(nft21) EIGVEC
        WRITE (IW,9999) KSTAT,EIGVAL(KSTAT)
        IWKS = 0
        LEV = 1
        LEVM = 1
        IR0 = 1
  100   CONTINUE
        IF (LEV .EQ. NLEVS) GO TO 140
        LEVIR(LEV) = IR0
        LEVM = LEV
        LEV = LEVM+1
        LEVIR(LEV) = LEVNR(LEV)+1
  110   CONTINUE
        IR0 = LEVIR(LEV)
        NPTX = LEVPT(LEV)
        NPTM = LEVPT(LEVM)
        IRM = LEVIR(LEVM)
  120   CONTINUE
        IR0 = IR0-1
        IF (IR0 .EQ. 0) GO TO 300
        NPT = IR0+NPTX
        DO 130 K = 1,4
          IARPT = NPT+ISHIFT(K)
          JARC = IARC(IARPT)
          IF (JARC .EQ. 0) GO TO 130
          JARC = JARC-NPTM
          ICASE(LEVM) = K
          IF (IRM .EQ. JARC) GO TO 100
  130   CONTINUE
        GO TO 120
C
  140   CONTINUE
C
        DO 250 ILEV = 1,NORBS
          ICAS = ICASE(ILEV)
          GO TO (200,210,210,220), ICAS
  200     IOCC = 0
          GO TO 230
  210     IOCC = 1
          GO TO 230
  220     IOCC = 2
  230     CONTINUE
          IORB=-2**20
          DO 240 I = 1,NORBMX
            IF (IOUT(I) .EQ. ILEV) IORB = I
  240     CONTINUE
          IECONF(IORB) = IOCC
          IECASE(ILEV) = ICAS
  250   CONTINUE
C
        IWKS = IWKS+1
        DUM = EIGVEC(IWKS)
        IF(ABS(DUM).GE.PRTTOL) THEN
           WRITE(IW,9998) IWKS,DUM,
     1                    (IECONF(NCORB+IORB),IORB=1,NORBS)

cc make list array if list_csf etc are not given as input
          if (nrcsf.eq.0) then
            iupdate=0
            do ilist=1,icount
              if (list_csf(ilist).eq.iwks) then
                iupdate=ilist ! check double count about the CSF
              end if
            end do
            if (iupdate.eq.0) then 
              icount=icount+1
              list_csf(icount)=iwks
              tab_coef(icount)=abs(dum)
              list_group(icount)=kstat
            else if (tab_coef(iupdate).lt.abs(dum)) then ! up date list
              tab_coef(iupdate)=abs(dum)
              list_group(iupdate)=kstat
            end if
          end if 
         
        END IF
300     continue
        lev=levm
        levm=lev-1
        if (levm.gt.0) goto 110
      end do
c!      nlist=icount
      nlist0=icount
   
      if (nrcsf.eq.0) nlist=nlist0

c
c Pick up CI coefficents of dominant CSFs for all target states
c and store them to dcicoef array
c

      rewind(nft21)
      if (nft21.eq.nft12) read(nft21) idum,idum ! read header of FT12

          
      do i=1,nst
        read(nft21) eigvec
        do icsf=1,nlist
          dcicoef(icsf,i)=eigvec(list_csf(icsf))
        end do
      end do



      RETURN


 9999 FORMAT(/,1X,'Adiabatic State #',I5,'  ENERGY =',F18.9,
     1       /,1X,'----------',5X,'  ------  ',
     2       /,6X,'CSF',6X,'Coefficient',4X,'Occupancy'/
     3        ,6X,'---',6X,'-----------',11X,'---------')
 9998 FORMAT(1X,I8,F12.6,2X,110I1,/,23X,18I1)
 9996 FORMAT(23X,36I3)
 9995 FORMAT(23X,36(2X,A))

      END


      subroutine pucsfdia(nstate,nlist,coef,list_csf,list_group,
     &                    maxlist,maxci)

      implicit real*8(a-h,o-z)

      common/iofile/ir,iw,ip
    
      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension coef(maxlist,maxci)

C#g77 ------------------------------------------
      CHARACTER*8 csfdat,endwrd
C#g77 ------------------------------------------
      
      data csfdat /' $CSFDAT'/
      data endwrd /' $END   '/


c
c punch out  data of dominant CSFs 
c
 

      write(ip,9900) csfdat

      write(ip,*) nlist
      max=0
10    min=max+1
      max=max+5
      if (max.gt.nlist) max=nlist
      write(ip,9910) (list_csf(i),list_group(i),i=min,max)
      if (max.lt.nlist) goto 10
!-- punch out diabatic CI coefficients for each set of DMOs
      do j=1,nstate
        jcount=0
        max=0
20      min=max+1
        max=max+5
        jcount=jcount+1
        if (max.gt.nlist) max=nlist
        if (nlist.le.99) write(ip,9920) j,jcount,
     &                     (coef(i,j),i=min,max)
        if (nlist.gt.99) write(ip,9930) j,jcount,
     &                     (coef(i,j),i=min,max)
        if (max.lt.nlist) goto 20
      end do

      write(ip,9900) endwrd


      return

9900  format(a8)
9910  format(i8,'(',i2,')')
9920  format(i2,i3,5e15.8) 
9930  format(i3,i2,5e15.8) 


      end 

      subroutine rdcsfdia(nstate,nlist,coef,list_csf,list_group,
     &                    maxlist,maxci)
  
      implicit real*8(a-h,o-z)

      common/iofile/ir,iw,ip
    
      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension coef(maxlist,maxci)

C#g77 ------------------------------------------
      CHARACTER*8 csfdat,word
C#g77 ------------------------------------------
      
      data csfdat /' $CSFDAT'/


c
c Read data of dominant CSFs 
c
 
!-- search $CSFDAT flag--
10    continue
      read(ir,9900,end=15,err=15) word
      goto 20
15    write(iw,*) 'Can not be found $CSFDAT. Stop'
      stop
20    continue
      if (csfdat.ne.word) goto 10
!-- read dominant CSF list and group
      read(ir,*) nlist
      max=0
25    min=max+1
      max=max+5
      if (max.gt.nlist) max=nlist
      read(ir,9910) (list_csf(i),list_group(i),i=min,max)
      if (max.lt.nlist) goto 25


!-- read reference CI coefficients in $CSFDAT for each set of DMOs
      do j=1,nstate
        max=0
30      min=max+1
        max=max+5
        if (max.gt.nlist) max=nlist
        if (nlist.le.99) read(ir,9920) idum,idum,
     &                   (coef(i,j),i=min,max)
        if (nlist.gt.99) read(ir,9930) idum,idum,
     &                   (coef(i,j),i=min,max)
        if (max.lt.nlist) goto 30
      end do

      return

9900  format(a8)
9910  format(i8,'(',i2,')')
9920  format(i2,i3,5e15.8) 
9930  format(i3,i2,5e15.8) 


      end 


      subroutine rdfuldia(nft,L1,w)
      implicit real*8 (a-h,o-z)
      dimension w(L1)
      read(nft) w
      return 

      end 

      subroutine refdia(list_csf,list_group,dcicoef,
     &                  nlist,nstate_sav,eci_sav)

      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MAXCI=50)  

      parameter (maxlist=64)

      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,
     &              alpha_t,slctth,moslab_charact,th_doc,th_vir,
     &              conv_th,max_iter,ngrstate                         !RV 9/21/06

      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/IOFILE/IR,IW,IP
      COMMON/SCM/X(1)
      common/diatmpfil/nft21,nft22,nft23,nft24  

      DATA NFT11  /11/
      DATA NFT12  /12/
      DATA ZERO   /0.0D+00/
      DATA CVGTOL /1.0D-05/

      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,nstate_sav)
      dimension eci_sav(maxci)
      dimension istack(maxci)  ! stack array for various index
      
c
c list_csf(i).... i-th dominant CSF inex 
c list_group(i)...Group index where i-th dominant CSF belongs to
c nlist.... total # of dominant CSFs.
c --->If list_csf,list_group, and nlist are already defined by $CSFDAT,
c     they are input and not changed. If not, this routine makes list
c     and return. In the latter case, users should understand that
c     the return is only for dominant CSFs based on DMO set.
c     i.e, not always enough dominant CSFs for global nuclear coordinate
c     region.
c dcicoef(i,j)...CI coefficent of j-th adiabatic state of i-th 
c                      dominant CSF 
c nstate_sav... total # of target adiabatic states for diabatization
c               as input. 
c (if ndmosym=0, this is equal to nstate read from the header of FT12)
c eci_sav   ... the adiabatic CI energies for target states for
c               diabatization
c 

 
c
c Read header of FT11 (DRT file) and FT12 (CI vectors)
c

      REWIND NFT11
      READ(NFT11) NORBMX,IDUM,IDUM,IDUM,NROWS

      REWIND NFT12
      READ(NFT12) NSTATE,NWKS

c
c Search (suitable) target states if DMO's are determined by
c breaking symmetry
c

      if (ndmosym.ne.0) then
        write(iw,*) '--- Search suitable CI (CAS) states ---'

        icount=0
        do i=1,nstate_sav
          do j=1,nstate
            dum=abs(eci(j)-eci_sav(i))
            if (dum.lt.cvgtol) then
              icount=icount+1
              istack(icount)=j
            end if
          end do
        end do
        if (icount.ne.nstate_sav) then
          write(iw,'('' Error: Cannot Find Suitable States.''/
     &               '' You maybe nedd more large value of NSTRY''/
     &               '' Stop.'')')
          stop
        else
          write(iw,'('' Root searching was successful.''/
     &               '' --- You needed only NSTRY='',i2,''---'')')
     &          istack(icount)
        end if
       
      else 
        do i=1,nstate_sav
          istack(i)=i
        end do
      end if


!
! If DMOs are obtained by symmetry breaking, write only target CI
! vectors to FT21 and use this file.
! If symmetry are conserved, the CI vectors in FT12 are used
! and FT21 is not created.
! (Reading FT21 ot 12 is done in the routine prmcdia 
!



cc Get core memory


      K10 = 1
      K20 = K10 + NORBMX
      K30 = K20 + NORBMX
      K40 = K30 + NORBMX
      K50 = K40 + NROWS*4
      K60 = K50 + NWKS
      K70 = K60 + NORBMX
      need=k60+norbmx

      call cmem(loadcm)
      call setc(need)

cc Pick up target CI vectors from FT12 and write to FT21, if necessary

      if (ndmosym.eq.0) then
        nft21_sav=nft21
        nft21=nft12 ! i.e Not use FT21 file
      else
        rewind(nft21)
        icount=1
        do i=1,nstate ! nstate is read from FT12's header
c!          call diardci(nft12,nwks,x(k50))
          call rdfuldia(nft12,nwks,x(k50))
          if (i.eq.istack(icount)) then
c!            call diawtci(nft21,nwks,x(k50))
            call wtfuldia(nft21,nwks,x(k50))
            icount=icount+1
          end if
          if (nstate_sav.lt.icount) goto 100 
        end do
      end if 
100   continue

 
c
c Print -MC- Coefficient using DMOs and get list of domain CSFs
c for only suitable CI sates.
c

      write(iw,'(''----------------------------------------------''/
     &           '' CSF Expansion of DMOs Base in Active Space''/
     &           ''-----------------------------------------------'')')

      call prmcdia(nrcsf,NFT11,NFT12,nft21,nstate_sav,NWKS,
     1  eci_sav,X(K50),slctth,X(K10),X(K20),X(K30),X(K60),X(K40),
     2  list_csf,list_group,dcicoef,nlist,
     3  NORBMX,4*NROWS,NROWS)

      write(iw,'(''-----------------------------------------------''/
     &           '' CI coefficients of dominant CSFs for all Groups''/
     &           ''-----------------------------------------------'')')

      do i=1,nstate_sav
        write(iw,'('' Adiabatic State #'',i5,'' ENERGY ='',f18.9,/
     &        '' ------------------    -------------''/
     &        ''       <CSF>   <Group #>  <Coefficient>'')')
     &        i,eci_sav(i) 
        do j=1,nlist
          write(iw,'(1x,i8,i3,f12.6)') list_csf(j),list_group(j),
     &                                 dcicoef(j,i)
        end do
      end do
      

cc Reset core memory
      
      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)  

cc Back parameter nft21 to the original value  if necessary

      if (ndmosym.eq.0) nft21=nft21_sav 


      return

      end


      subroutine refdiamq(list_csf,list_group,dcicoef,
     &           nlist,nstmq,nstci,kstci_mq,coef_max_ref,
     &           e_mcqdpt,eigvec)

      IMPLICIT REAL*8 (A-H,O-Z)
c      PARAMETER (MXIODA=255)
c      PARAMETER (NUMMX=3072)
c      PARAMETER (MAXCI=50)  

      parameter (maxlist=64)

      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,
     &              alpha_t,slctth,moslab_charact,th_doc,th_vir,
     &              conv_th,max_iter,ngrstate                           !RV 9/21/06

c      COMMON/ MCPRT/CIPRT,MOPRT,PRTSPN
c      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/IOFILE/IR,IW,IP
c      COMMON/ MCPAR/NFZC,NCORBS,NCI,NORBS,NORB,NUM,NWKS
c      COMMON/INFOA/NAT,ICH,MUL,NBF,NX,NE
      COMMON/SCM/X(1)
      common/diatmpfil/nft21,nft22,nft23,nft24  

      DATA NFT11  /11/
      DATA NFT12  /12/
c      data nft21  /21/
c      DATA PT5    /0.5D+00/
      DATA ZERO   /0.0D+00/
c      DATA NDIAG  /0/
c      DATA MXXPAN /50/
c      DATA MAXDIA /512/
      DATA CVGTOL /1.0D-05/
c      DATA NWORD  /0/


      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,nstmq)
      dimension eigvec(nstmq,nstmq)
      dimension e_mcqdpt(nstmq)
      dimension coef_max(nstmq)
      dimension coef_max_ref(nstmq)
      dimension kstci_mq(nstci)
      
c
c list_csf(i).... i-th dominant CSF inex 
c list_group(i)...Group index where i-th dominant CSF belongs to
c nlist.... total # of dominant CSFs.
c --->If list_csf,list_group, and nlist are already defined by $CSFDAT,
c     they are input and not changed. If not, this routine makes list
c     and return. In the latter case, users should understand that
c     the return is only for dominant CSFs based on DMO set.
c     i.e, not always enough dominant CSFs for global nuclear coordinate
c     region.
c dcicoef(i,j)...CI coefficent of j-th adiabatic state of i-th 
c                      dominant CSF 
c nstmq... total # of QDPT target adiabatic states for diabatization
c    as input.(not necessally equal to total # of CAS-CI target state)
c 


 
c
c Read header of FT11 (DRT file) and FT12 (CI vectors)
c

      REWIND NFT11
      READ(NFT11) NORBMX,IDUM,IDUM,IDUM,NROWS

      REWIND NFT12
      READ(NFT12) NSTATE,NWKS


c
c Search (suitable) QDPT (i.e. diabatization) target states 
c

      write(iw,*) 'QDPT target states are picked up'

      icount=0
      do i=1,nstci
        if (kstci_mq(i).ne.0) then
          icount=icount+1
        end if
      end do
      if (icount.ne.nstmq) then
        write(iw,*) 'Error. QDPT target states can not be identified' 
        stop
      end if 
       
c
c Read CAS-CI vectors and make MC-QDPT projected CI vectors
c
c



cc Get core memory


      K10 = 1
      K20 = K10 + NORBMX
      K30 = K20 + NORBMX
      K40 = K30 + NORBMX
      K50 = K40 + NROWS*4
      K60 = K50 + NWKS
      K70 = K60 + NORBMX
      k80 = k70 +nwks*nstmq
      need=k80

      call cmem(loadcm)
      call setc(need)

cc Pick up target CI vectors from FT12 and write to FT21

      icount=1
      do i=1,nstci
c!        call diardci(nft12,nwks,x(k50))
        call rdfuldia(nft12,nwks,x(k50))
        if (kstci_mq(i).ne.0) then ! this CI root is QDPT target state
c!          call diawtci(nft21,nwks,x(k50))
          call wtfuldia(nft21,nwks,x(k50))
!-- search max CI coef and store all states to memory
          jshift1=k50-1
          jshift2=k70-1+(icount-1)*nwks
          dum=zero
          do j=1,nwks
            coef=x(j+jshift1)
            x(j+jshift2)=coef
            if (abs(dum).lt.abs(coef)) dum=coef
          end do
!---
          coef_max(icount)=dum
          icount=icount+1
        end if
        if (nstmq.lt.icount) goto 100 
      end do
100   continue

c
c Calculate  CI vectors of MC-QDPT projected wavefunction 
c

      rewind(nft21)
      call mkqdptvec(nft21,x(k70),eigvec,x(k50),nwks,nstmq,coef_max,
     &               coef_max_ref)


c
c Print -MC-QDPT- Coefficient using DMOs and get list of domain CSFs
c for only QDPT target states
c

      write(iw,'(''=============== Notice ========================''/
     &           '' Adiabatic state # and energies are QDPT target ''/
     &           '' state # and MC-QDPT energies.''/
     &           ''================================================'')')
      write(iw,'(''------------------------------------------------''/
     &           ''CSF Expansion of DMOs Base in Active Orbitals''/
     &           ''------------------------------------------------'')')
      call prmcdia(nrcsf,NFT11,NFT12,nft21,nstmq,NWKS,
     1  e_mcqdpt,X(K50),slctth,X(K10),X(K20),X(K30),X(K60),X(K40),
     2  list_csf,list_group,dcicoef,nlist,
     3  NORBMX,4*NROWS,NROWS)

      write(iw,'(''----------------------------------------------''/
     &           '' CI coefficients of dominant CSFs for all Groups''/
     &           ''----------------------------------------------'')')

      do i=1,nstmq
        write(iw,'('' Adiabatic State #'',i5,'' ENERGY ='',f18.9,/
     &        '' ------------------    ----------------''/
     &        ''       <CSF>   <Group #>  <Coefficient>'')')
     &        i,e_mcqdpt(i) 
        do j=1,nlist
          write(iw,'(2x,i8,4x,i2,f12.6)') list_csf(j),list_group(j),
     &                              dcicoef(j,i)
        end do
      end do
      
      write(iw,*) '---------------------------'

cc Reset core memory
      
      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)  



      return


      end


      subroutine refdmosym(n_sum,num,ndm_size,mxprim,w,q,s,temp,
     &  moslab_ref,vecwrd,endwrd,frwrd,natom,anam,num_ao,label_type,
     &  label2,norient)                                      !HN 2/5/02


      implicit real*8 (a-h,o-z)

      character*8 charatom
  
      equivalence (charatom,anam_rd)
c
c
c
c Function: (1) transform ref or pseudo ref MO coef from specified
c               orientation by input to the current standard orientation
c               (i.e. coordinates of $BASIS)  for each atom
c
c           Then
c           
c           (2) assign the irreps of the reference DMOs and 
c               make label (moslab_ref) then return it.
c

c
c Note: current version transform MO coefs of only p type.
c     : for d,f... type are eliminated!
c
c

      parameter(mxioda=255)
      
      common/iofile/ir,iw,ip
      common/dafile/idaf,nav,ioda(mxioda)
      common/ijpair/ia(1)



c
c ndm_size is fixed size
c mxprim   is fixed size
c mxioda   is fixed size
c
c natom... # of toal atoms
c label_type(i,j) ... j-th atom's i-th AO's label
c num_ao(i) .... i-th atom's total basis #
c label2.... data set of labe like s, px,....
c w     .... ref or pseudo ref MO coef of (specific orientation:input)
c                                  (current standard orientation:output)
c
c


      dimension moslab_ref(n_sum),mosirp_ref(n_sum) ! for SYMMOS
      dimension mosdeg_ref(n_sum) ! irreps label for ref DMOs ($DIAVEC)

      dimension w(num,n_sum)         ! ref DMO/AO coefficient
      dimension q(num,num)           ! canonical vec
      dimension s(num*(num+1)/2)     ! AO overlap
     
      dimension temp(num)            ! temporary region 

      dimension trans(3,3,natom)                             !HN 2/4/02
      dimension label_type(mxprim,natom)                     !HN 2/4/02
      dimension label2(12)                                   !HN 2/4/02
      dimension num_ao(natom)                                !HN 2/4/02
      dimension anam(natom)                                  !HN 2/5/02 


      data zero,one /0.0d0,1.0d0/
      data th_zero /1.0d-03/                                 !HN 2/5/02

c
c Read data group
c

      rewind(ir)
5     continue 
      read(ir,9800,end=10,err=10) word
      goto 20
10    write(iw,*)'Cannot be found $DIAVEC or $DPSVEC. Stop'
      stop
20    continue
      if (vecwrd.ne.word) goto 5                              !HN 2/1/02
      do j=1,n_sum
        max=0
30      min=max+1
        max=max+5
        if (max.gt.num) max=num
        if (num.le.99) read(ir,9810) idum,idum,
c!     &     (w(i+num*(j-1)),i=min,max)
     &     (w(i,j),i=min,max)                                 !HN 2/5/02
        if (num.gt.99) read(ir,9820) idum,idum,
c!     &     (w(i+num*(j-1)),i=min,max)
     &     (w(i,j),i=min,max)                                 !HN 2/5/02
        if (max.lt.num) goto 30
      end do

!
! Now all reference (or pseudo) MO  coeffcients 
! stored in w
!

c
c Read data group and transform ref (pseudo ref) MO coeffs sequentially
c (if necessary)
c
c (if norient=0, there are no specific orientation; thus use the current
c standard orientation) 
c


      if (norient.ne.0) then  ! if-orientation use
c-- search flag--
        rewind(ir)
35      continue
        read(ir,9800,end=40,err=40) word
        goto 50
40      write(iw,*) 'Cannot be found $LCLDC or $LCLVL'
        stop
50      continue
        if (frwrd.ne.word) goto 35
c----------------
        do iorb=1,n_sum  ! loop over ref (pseudo ref) MOs 
c-- read data group (unit vector)
          do i=1,natom
            read(ir,*)charatom,trans(1,1,i),trans(2,1,i),trans(3,1,i)
            read(ir,*)         trans(1,2,i),trans(2,2,i),trans(3,2,i) 
            read(ir,*)         trans(1,3,i),trans(2,3,i),trans(3,3,i) 
c-- check-(1)--
            if (anam_rd.ne.anam(i)) then
            write(iw,*)'Order are different betwee $BASIS and $LCLDC/VL'
              stop
            end if
          end do
c-- check (2)--
          do i=1,natom
            dum1=trans(1,1,i)**2+trans(2,1,i)**2+trans(3,1,i)**2
            dum2=trans(1,2,i)**2+trans(2,2,i)**2+trans(3,2,i)**2
            dum3=trans(1,3,i)**2+trans(2,3,i)**2+trans(3,3,i)**2
            dum12=trans(1,1,i)*trans(1,2,i)+trans(2,1,i)*trans(2,2,i)
     &           + trans(3,1,i)*trans(3,2,i)
            dum23=trans(1,2,i)*trans(1,3,i)+trans(2,2,i)*trans(2,3,i)
     &           + trans(3,2,i)*trans(3,3,i)
            dum31=trans(1,3,i)*trans(1,1,i)+trans(2,3,i)*trans(2,1,i)
     &           + trans(3,3,i)*trans(3,1,i)
            dum1=dabs(one-dum1)
            dum2=dabs(one-dum2)
            dum3=dabs(one-dum3)
            dum12=dabs(dum12)
            dum23=dabs(dum23)
            dum31=dabs(dum31)
            if ( (dum1.gt.th_zero).or.(dum2.gt.th_zero).or.
     &           (dum3.gt.th_zero).or.(dum12.gt.th_zero).or.
     &           (dum23.gt.th_zero).or.(dum31.gt.th_zero) ) then 
              write(iw,*)'$LCLDC/VL must be orthogonal unit vectors'
              write(iw,*) 'Error in ',i,' th unit vector' 
              write(iw,*) 'Norm', dum1,dum2,dum3
              write(iw,*) 'Ortho', dum12,dum23,dum31
              stop
            end if 
          end do
        
c-- transform MO coef based on the current standard orientation form the
c-- specified orientation (p-type only!!)
          indao=0
          do iatom=1,natom 
            do ibas=1,num_ao(iatom)
              indao=indao+1
              lab=label_type(ibas,iatom)
              if (lab.eq.label2(2)) then
                coef_x=w(indao,iorb)
                coef_y=w(indao+1,iorb)
                coef_z=w(indao+2,iorb)
                w(indao,iorb)=trans(1,1,iatom)*coef_x
     &                       +trans(1,2,iatom)*coef_y
     &                       +trans(1,3,iatom)*coef_z
                w(indao+1,iorb)=trans(2,1,iatom)*coef_x
     &                         +trans(2,2,iatom)*coef_y
     &                         +trans(2,3,iatom)*coef_z
                w(indao+2,iorb)=trans(3,1,iatom)*coef_x
     &                         +trans(3,2,iatom)*coef_y
     &                         +trans(3,3,iatom)*coef_z
              else if ( (lab.ne.label2(1)).and.
     &                  (lab.ne.label2(2)).and.
     &                  (lab.ne.label2(3)).and.
     &                  (lab.ne.label2(4)) ) then
                w(indao,iorb)=zero
              end if
            end do
          end do
        end do  ! loop end of ref (pseudo ref) MOs
      end if  ! end if of orientation use

!
!-- Assign symmetrty (irreps) for MOs
!

      call daread(idaf,ioda,q,num*num,13) ! read canonical vector


      call symmos(moslab_ref,mosirp_ref,mosdeg_ref,q,s,w,
     &            temp,ia,num,num,n_sum,num)
      

      return


9800  format(a8)
9810  format(i2,i3,5e15.8)
9820  format(i3,i2,5e15.8)

      end

      subroutine sort_mormo(num,nmlap,nref_block,moslab_refdmo,
     &                      labref_block,msegment_ref,v,temp)

      implicit real*8 (a-h,o-z) 

c
c This subroutine sorts the data of reference MO for the MORMO 
c criterion (i.e. $DIAVEC) to group block such as the same irreps ref
c DMOs belong to one block.
c

      dimension v(num,nmlap)
      dimension moslab_refdmo(nmlap)
      dimension labref_block(nmlap)
      dimension msegment_ref(nmlap)

      dimension temp(num,nmlap)
      dimension itemp(nmlap)
      dimension ind_temp(nmlap)

c
c n_block ... total # of block for mixing (input & output) 
c nref_block ... total # of block of reference DMOs (grouped by irreps)
c                (output)
c

c
c v ... data of the reference MOs ($DIAVEC) (input. Output is updated)
c moslab_refdmo ... irreps label of the reference MOs 
c                   (input. output is reordered with v in this routine.)
c labref_block  ... irreps label of each block  (output)
c msegment_ref  ... msegment_ref(i) is number of reference DMOs of 
c                   i-th block 
c

      
      do i=1,nmlap
        itemp(i)=moslab_refdmo(i)
      end do

      icount=0
      ishift=0
      do i=1,nmlap ! this loop is correlating with picking up one-block
        if (itemp(i).eq.0) goto 100 ! skip if 0 = dummy value
        icount=icount+1
        idumlab=itemp(i)
        labref_block(icount)=idumlab  ! irreps label of this block
        jcount=0
        do j=1,nmlap ! this loop is correlating to determination of 
                     ! the members of the above one-block
          if (itemp(j).eq.idumlab) then
            jcount=jcount+1
            ind_temp(jcount+ishift)=j
            itemp(j)=0 ! replace to null dummy value 
          end if
        end do
        msegment_ref(icount)=jcount
        ishift=ishift+msegment_ref(icount)
100     continue
      end do
      nref_block=icount


c
c Sort MOs 
c

      do i=1,nmlap
        do j=1,num
          temp(j,i)=v(j,i)
        end do
      end do
  
      do i=1,nmlap
        do j=1,num
          v(j,i)=temp(j,ind_temp(i))
        end do
      end do

c
c Sort label
c

      do i=1,nmlap
        itemp(i)=moslab_refdmo(i)
      end do
   
      do i=1,nmlap
        moslab_refdmo(i)=itemp(ind_temp(i))
      end do


      return 

      end        



      subroutine srchmormo(n_search,ihead_mo,n_mixmo,n_block,lab_block,
     &                     msegment,lab_ref)

      implicit real*8 (a-h,o-z)

c
c This subroutine search the block of current MOs corresponding 
c to the block of the reference DMOs.
c The label of target (reference DMOs) block is identified 
c as input "lab_ref" 
c 
c
   
c
c lab_block .... label array for the current MOs's block for MORMO
c n_block   .... total # of block for the current MOs for MORMO
c lab_ref   .... label of target reference DMOs's block
c n_search  .... n_search-th block of current DMOs's block is 
c                corresponding to the target block
c n_mixmo   .... total # of current MOs  in n_search-th block
c ihead_mo  .... header index of MOs in n_search-th block 
c                (physical address is identified indmo_mix(ihead_mo))
c

      dimension lab_block(n_block)
      dimension msegment(n_block)

      do i=1,n_block
        if (lab_ref.eq.lab_block(i)) then
          n_search=i
          goto 100
        end if
      end do

100   continue


      ihead_mo=1
      do i=1,n_search-1
        ihead_mo=ihead_mo+msegment(i)
      end do

      n_mixmo=msegment(n_search)


      return

      end

      subroutine symdfmvec(nft,num,n_active,vec,s,temp)

      implicit real*8 (a-h,o-z)

      common/ijpair/ia(1)
      common/iofile/ir,iw,ip

c
c This subroutine transforms the data $DFMVEC to the symmetry 
c orthogonalized one and write to the scratch file
c

      data zero,one /0.0d0,1.0d0/
      data err_th   /1.0d-04/ 

      dimension vec(num,n_active)  ! $DFMVEC's data (input)
      dimension s(num*(num+1)/2)   ! AO overlap
      dimension temp(num*num)

      dimension w(n_active,n_active) 

c
c Calculate MO overlap matrix w(j,i)=<j|i>
c

      do i=1,n_active
        do j=1,n_active
          dum=zero
          do k=1,num
            do l=1,num
              if (l.le.k) then
                ind_kl=ia(k)+l
              else
                ind_kl=ia(l)+k
              end if
              dum=dum+vec(k,i)*vec(l,j)*s(ind_kl)
            end do
          end do
          w(i,j)=dum
        end do
      end do
           
c
c Symmetric Orthogonalization of vec (by using overlap matrix w)
c

      nwork=n_active*n_active*2+3*n_active+num*n_active
      if (num.lt.(n_active+3)) then
        write(iw,*) '# of basis is too smaller than # of active orb.'
        write(iw,*) 'stop'
        stop
      end if
      call symvec(vec,w,n_active,num,temp,nwork)
      
c
c Print sym. orth. transform matrix
c

      write(iw,*) '                      '
      write(iw,*) 'Symmetric Orthogonalization of $DFMVEC data'
      do i=1,n_active
        write(iw,*) i,' th column of sym orth matrix'
        write(iw,9800) (w(j,i),j=1,n_active)
      end do 

c
c Write symmetric orthogonalized $DFMVEC to the scratch file 
c
  
      rewind(nft)
      call wtfuldia(nft,num*n_active,vec) 


      return

9800  format(1x,5f8.5)

      end 
         

      subroutine symvec(vec,s,nvec,num,work,nwork)

      implicit real*8 (a-h,o-z)
 
      common/iofile/ir,iw,ip

      dimension vec(num,nvec)

      dimension s(nvec,nvec)
      
      dimension work(nwork)


      data  zero /0.0d0/
      data  one  /1.0d0/
      data  err_th /1.0d-04/
     
c
c This routine calculate symmetric orthogonal vectors and determine
c sign of vectors as they agree with original vectors.
c

c
c nvec ....   total number of vectors to orthogonalize
c
c num  ....   dimension of vector
c
c nwork ...   scratch space size ! needed at least 
c             nvec*nvec+3*nvec+num*nvec
c
c s(i,j) .... overlap matrix between i-th vector and j-th. 
c             Assumed that full matrix elements are stored. (input)
c             Return as transformation (sym. orthogonalized) matrix
c
c vec(*,i)... i-th vector components. Return as the symmetric 
c             orthonormalized vector.
c
c work .... scratch space
c
c

     
c
c S^{-1/2} = U*diag(s^{-1/2})*U^{t}
c


cc Diagonalize S  

      iw1=1
      iw2=iw1+nvec*nvec
      iw3=iw2+nvec
      iw4=iw3+2*nvec

cc      iwork_space=iw4+nvec*nvec-1
      iwork_space=iw4+num*nvec-1

!
! w(iw1) : store nvec*nvec square matrix temporary
! w(iw2) : store nvec array
! w(iw3) : use scartch for routine diajac
! w(iw4) : store nvec*nvec or num*nvec  matrix (always nvec.le.num)
!

! for debug----
      if (nwork.lt.iwork_space) then
        write(iw,*) 'Too small work space. Set more large value
     &               nwork when you call symvec'
        stop
      end if
! for debug-----


      call diajac(s,work(iw1),work(iw2),nvec,nvec,work(iw3))

! Now work(iw1) is eigenvector matrix i.e U, 
! work(iw2) is eigenvalues 
! and work(iw3) is  work space for this routine.
! Matrix s is breaken.

      do i=1,nvec
        if (work(i+iw2-1).lt.err_th) then
          write(iw,*) 'Cannot symmetric/orthogonalize. Stop.'
          stop
        end if
        work(i+iw2-1)=one/dsqrt(work(i+iw2-1))
      end do

cc S^{-1/2}

! wrok(iw4) is U*diag(s^{-1/2})
      do i=1,nvec
        ishift1=iw1-1+(i-1)*nvec
        ishift4=iw4-1+(i-1)*nvec
        idiag=i+iw2-1
        do j=1,nvec
          work(j+ishift4)=work(j+ishift1)*work(idiag)
        end do
      end do 
      
! S^{-1/2} is overwritten to array s
      call dgemm('N','T',nvec,nvec,nvec,one,work(iw4),nvec,
     &           work(iw1),nvec,zero,s,nvec)


c
c Check sign of each column 
c

      do j=1,nvec
        if ( s(j,j).lt.zero ) then
          do i=1,nvec
            s(i,j)=-s(i,j) 
          end do
        end if
      end do


c
c Transform MO coefficients vec (non orthogobal) --->
c vec (symmetric orthogonal)
c
 

cc copy vec(*,*) to one dimarray work(iw4)
      icount4=iw4
      do j=1,nvec
        do i=1,num
          work(icount4)=vec(i,j)
          icount4=icount4+1
        end do
      end do


      call dgemm('N','N',num,nvec,nvec,one,work(iw4),num,
     &            s,nvec,zero,vec,num)


      return

      end


      subroutine threefold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &                     nstate,num,n_active,moslab_active,dm,
     &                     v,avocc_act,temp)


      implicit real*8 (a-h,o-z)

c
c This routine carry out the threefold density criterion 
c and determine the MO/base transformation matrix.
c
c The input MOs are transformed by the threefold density criterion and
c retruned as output
c Also all DM are transformed then return.
c
  
      common/iofile/ir,iw,ip
 
      data zero,one/0.0d0,1.0d0/

      dimension v(num,n_active)
      dimension dm(n_active,n_active,nstate*(nstate+1)/2+1)
      dimension moslab_active(n_active)
      dimension avocc_act(n_active)

      dimension u(n_active,n_active)  
      dimension indmo_mix(n_active)
      dimension msegment(n_active)
      
      dimension temp(num,n_active)
      dimension itemp(n_active)

c
c num  .... number of basis set (AO)
c n_active ... number of the active MOs
c

c
c v  ... MO vec (active) : return as updated (threefold density's) MO
c dm ... Density matrices (including transition DM and SA-DM)
c (dm are stored as (11),(12),...(1 nstate),(22),(23),.... )
c u  ... MO/base transformation matrix   
c moslab_active .... irreps label of the active MOs
c

c
c msegment(i).... number (dimension) of the i-th block to be mixed 
c
c indmo_mix ... indmo_mix(1:msegment(1))=MO index to be mixed for the
c                                        threefold density criterion
c indmo_mix(msegment(1)+1:msegment(1)+msegment(2))=MO index of the
c                                        next block
c               .... 
c                       where n_block (<n_active) is total # of block
c                      (defined bellow)
c 




c
c Define the block to mixing MOs for the threefold density criterion
c based on irreps (by using moslab_active)
c


      call mkmoind1(n_active,moslab_active,indmo_mix,msegment,n_block)
        

c
c Carry out the threefold density criterion to the all active space
c and determine MO/base transformation matrix U
c The MOs to be applied for the threefold density criterion 
c and to be mixed with each others are already defined by indo_mix
c and msegment.
c

cc Initialization of U (i.e. unit matrix)

      do i=1,n_active
        do j=1,n_active
          u(j,i)=zero
        end do
      end do
      do i=1,n_active
        u(i,i)=one
      end do


cc threefold density criterion
 
      call max_D3(alpha_n,alpha_r,alpha_t,conv_th,max_iter,nstate,
     &            n_active,n_block,indmo_mix,msegment,dm,u,avocc_act)


c
c Transform the MO coefficients of the new MO by using the matrix U
c 

      call dgemm('N','N',num,n_active,n_active,one,v,num,u,n_active,
     &            zero,temp,num)

      do i=1,n_active
        do j=1,num
          v(j,i)=temp(j,i)
        end do
      end do


      return

      end      



      subroutine wfndia_casdmo

      implicit real*8 (a-h,o-z)

      parameter (mxatom=128)
      parameter (mxioda=255)
      parameter (maxci=50)
      parameter (mdiis=19)
c--

      parameter (mxprim=2048)  
      parameter (ndm_size=16) 
      parameter (maxlist=64)
      parameter (maxtot_orb=1024)

c--
      parameter (mxn_ref=10) !  Max number of the reference geomrtry
                             ! (arrangement)  UNDER CONSTRUCTION 
c--
                                   
      LOGICAL FINAL,MCNOCI,MLMOCI,MCANCI
      LOGICAL NOTRF
      LOGICAL FULLCI
      LOGICAL OFFSYM
      INTEGER FOCAS
C#g77 ------------------------------------------
      CHARACTER*8 DIRECT,diavecw,dfrmvecw,dpsvecw,
     1            frdcw,frvlw,endwrd,hndtyp,typsav
      CHARACTER*4 lab_c1,moslab_active,moslab_refdmo,
     1            moslab_refps,mcmoslab
      CHARACTER*4 label2
C#g77 ------------------------------------------
      COMMON/IOFILE/IR,IW,ip 
      COMMON/DAFILE/IDAF,NAV,IODA(MXIODA)
      COMMON/INFOA/NAT,ICH,MUL,NUM,NX,NE,NA,NB,ZAN(MXATOM),
     1                                         C(3,MXATOM)
      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/CASOPT/CASHFT,CASDII,NRMCAS,FOCAS
      COMMON/NTNOPT/ENERGY,ENERG0,DEMAX,SQCDF,ITER,MICRO,
     1              FINAL,MCNOCI,MLMOCI,MCANCI,
     2              NOCAN,NOCAS,NOCI,LMOCI,NOTRF
      COMMON/QMTOPT/QMTTOL,NQMT
      COMMON/TRFOPT/MCTRF,MPTRF
      COMMON/DRTOPT/FULLCI,IFFULL
      COMMON/DRTPAR/NCO,NMCCO,NCI,MSYM
      COMMON/TRFPAR/NFRZC,NMCORB,NCORB,NOCC,NORB,NBF
      COMMON/ MCPAR/NFZC,NCORBS,NACT,NORBS,NORB1,NTOT,NWKS
      COMMON/DIICAS/NCALL,NDIIS
      COMMON/DIIFIL/IDAF30,NAV30,IODA30(4*MDIIS+5)
      COMMON/HNDOPT/HNDTYP,DSKTYP,FILTYP,PCKTYP
      COMMON/HNDFIL/IFT12,IFT13,IFT14
      COMMON/SCM   /X(1)

      common/mcmossym/mcmoslab(maxtot_orb)
      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,alpha_t,
     &              slctth,moslab_charact(ndm_size),th_doc,th_vir,
     &              conv_th,max_iter,ngrstate                           !RV 9/21/06
      common/prototype/n_ref,nmlap(mxn_ref),ndlap,norient
      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir
      common/diatmpfil/nft21,nft22,nft23,nft24  
      common/xyzlabel_ao/label_type(mxprim,mxatom), 
     &                   label_shell(mxprim,mxatom),
     &                   num_ao(mxatom),ntot_basis 
      common/mollab/anam(mxatom),bnam(mxatom)

c--- labels 
      dimension moslab_active(ndm_size)
      dimension moslab_refdmo(ndm_size)
      dimension moslab_refps(ndm_size)
      dimension moslab_pseudo(ndm_size)
      dimension label2(12)
c---

      dimension avocc_act(ndm_size) 
      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,maxci)
      dimension dcicoef_ref(maxlist,maxci)
      dimension title(10),title1(10)
      dimension itemp(ndm_size)                              !HN 2/22/02


      data zero          /0.0D+00/
      data one           /1.0d+00/ 
      data two           /2.0d0/
      DATA DIRECT        /'DIRECT  '/
      data lab_c1       /4hA   /   

      data nft11 /11/
      data nft12 /12/  

      data diavecw /' $DIAVEC'/
      data dfrmvecw /' $DFMVEC'/
      data dpsvecw /' $DPSVEC'/
      data frdcw /' $LCLDC'/
      data frvlw /' $LCLVL'/
      data endwrd  /' $END   '/
 
      data label2 /'s','px','py','pz','dxy','dyz','dzx','dxx',
     &             'dyy','dzz','f','g'/

c
c
c avocc_act(i): averaged occupation # of i-th active MOs 
c list_csf(i): dominant CSF index 
c list_group(i): Group index  where list_csf(i)-th CSF belongs to
c dcicoef_ref(i,j): CI coefficents for diabatic states of list_csf(i)-th
c                 CSF for j-th diabatic states
c                    
c These three data are read from $CSFDAT 
c 
c dcicoef(i,j): CI coefficients of list_csf(i)-th CSF
c               for j-th adiabatic states 
c
c


c
c ----- Set some parameters for CI --
c

      final=.false.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.
      notrf=.false.
      fullci=.true.

c
c ----- Open -DIIS- work file ---
c


      ncall=0
      ndiis=0
      idaf30=30
      nav30=0
      ndar30=4*mdiis+5
      if (focas.ne.0) then
        call raopen(idaf30,ioda30,nav30,ndar30)
      end if

c
c ---- Set parameters for integral transformation ----
c

!     nco:   # of frozen core
!     nmcco: # of core orbitals
!     nocc:  # of active orbitals

      mptrf=0
      nfrzc=0
      nmcorb=nco+nmcco
      ncorb=nco+nmcco
      norb=nqmt-ncorb
      nocc=nci
      nbf=num

c
c ---- Set parameters for -NTN- algorithm ----
c

!     ncorbs: # of (frozen+core) orbitals
!     norbs:  # of (frozen+core+active) orbitals
!     norb:   # total of MOs
!     num:    # total of AOs (i.e. basis size)

      nfzc=imofzc
      ncorbs=ncorb
      norbs=ncorb+nci
      norb1=nqmt
      nact=nci
      ntot=num

c
c ---- Set parameters for other modules
c

      na=norbs

c
c ---- Set parameters for this module
c

      n_active=norbs-ncorbs


      write(iw,'(''---------------------------------------------''/
     &       ''     Diabatization of MC-QDPT wave functions starts''/
     &       ''     -------------------------------------------''/
     &       ''                                                  '')')


c
c Notice: (1) Input $DIABAT and $DFMVEC is already done in wfndia_cas
c             thus skip
c         (2) By the subroutine wfndia_cas, the CI vectors of MC-QDPT
c         are in FT21, and sym. orthogonalized $DFMVEC is stored in FT22
c         already.
c         (3) Check of symmetry of was already done
c         in the wfndia_cas; thus skip.
c



c
c Read prepaerd data from scrath files to make projected MC-QDPT
c wave function 
c

!  Read QDPT target CAS-CI vectors 

      rewind(nft21)
      read(nft21) idum,nwks
      
cc-- Get core memory ----

      ix1=1                  ! ix1--ix2-1: CI vectors 
      ix2=ix1+nwks*nstmq     ! ix2--ix3-1: temp
      ix3=ix2+nwks           
      last=ix3
      call cmem(loadcm)
      need=last
      call setc(need)


      icount=0
      do i=1,nstcas
        if (kstcas_mq(i).eq.0) then
          call rdfuldia(nft21,nwks,x(ix3))  ! skip
        else 
          icount=icount+1
          ihead=ix1+nwks*(icount-1)
          call rdfuldia(nft21,nwks,x(ihead))
        end if
      end do


c
c Calculate new CI vectors of projected MC-QDPT wave function
c and write them to FT12 
c

!-- Change header part of FT12 
      rewind(nft12)
      read(nft12) idum,idum,title,title1
      rewind(nft12)
      write(nft12) nstmq,nwks,title,title1
!--

      call mkqdptvec(nft12,x(ix1),eigvec,x(ix2),nwks,nstmq,
     &               coef_max1,coef_max2)


c
c Set parameter to calculate DM1 by using gugdmt
c

      nstate=nstmq


cc--- Reset core memory 

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)
c
c ---- Now scratch FT21 is released ----
c



cc======== Diabatization procedure start ===========


c
c Get MO-basis DM1 and write to FT21 temporary 
c

      call gugdmt(n_active) ! n_active= # of active orbitals
      if (ndm_size.lt.n_active) then
        write(iw,*) 'Too Large Active Space. Stop'
        stop
      end if

c
c Determination DMOs based on MC-QDPT wave functions
c


      n_sum=nmlap(1)
       

cc --- Get core memory ---

      n0=num*n_active
      n1=num*num
      n2=num*(num+1)/2
      n3=num*n_active
      n4=n_active*n_active*(nstate*(nstate+1)/2+1) ! +1--> for SA DM1
     
      ix1=1        ! ix1--ix2-1: store reference DMOs
      ix2=ix1+n0   ! ix2--ix3-1: store AO overlap
      ix3=ix2+n2   ! ix3--ix4-1: temp region  
      ix4=ix3+n1   ! ix4--ix5-1: store DMO/AO 
      ix5=ix4+n3   ! ix5--ix6-1: store DM1
      ix6=ix5+n4   ! ix6--last-1: store pseudo reference DMOs
      last=ix6+ndlap*num

      call cmem(loadcm)
      need=last
      call setc(need)

cc Read AO overlap

      call daread(idaf,ioda,x(ix2),n2,12)

cc--- Transform $DIAVEC based on the specified orientation
cc--- Assign irreps of $DIAVEC (if necessary) and make irreps label 
cc    of $DIAVEC

cc Read $DIAVEC, Symmetry assign (moslab_refdmo)

      if (n_sum.ne.0) then
        call refdmosym(n_sum,num,ndm_size,mxprim,x(ix1),x(ix3),x(ix2),
     &                 x(ix4),moslab_refdmo,diavecw,endwrd,frvlw,nat,
     &                 anam,num_ao,label_type,label2,norient)
      end if

cc--- Transform $DPSVEC based on the specified orientation.
cc--- Assign irreps of $DPSVEC (if necessary) and make irreps label 
cc    Symmetry assign (moslab_refps)

      if (ndlap.ne.0) then
        call refdmosym(ndlap,num,ndm_size,mxprim,x(ix6),x(ix3),x(ix2),
     &                 x(ix4),moslab_refps,dpsvecw,endwrd,frdcw,nat,
     &                 anam,num_ao,label_type,label2,norient)
      end if


cc Pick up the active MO coefficients  (only active part)
c- (CASSCF-based DMOs)

      call daread(idaf,ioda,x(ix3),n1,15)
      do i=1,n_active
        ishift1=ix3-1+num*(ncorbs+i-1)
        ishift2=ix4-1+num*(i-1)
        do j=1,num
          x(j+ishift2)=x(j+ishift1)   
        end do
      end do

!-- Now x(ix1)--> reference DMOs (i.e. $DIAVEC)
!       x(ix2)--> AO overlap
!       x(ix4)--> active MO (DA10 #15)
!       x(ix6)---> pseudo reference MO (i.e. $DPSVEC)

c
c -- Optimize orbitals to MC-QDPT based Diabatic MOs ---
c

      L1=nstate*(nstate+1)/2
      L2=ndm_size*ndm_size
      L3=n_active*n_active


cc  Read all DM1 and also add SA DM1 to data

      call arrange_dm(x(ix5),x(ix3),nstate,n_active,ndm_size,nft21)

 
! Now x(ix5) is all DM1 + SA DM1

c
c Make irreps label of active MOs. This procedure is completely same
c with the prepation procedure in the subroutine wfndia_cas 
c

      do i=1,n_active
        moslab_active(i)=mcmoslab(ncorbs+i)
      end do
    
      if (ndmosym.ne.0) then
        do i=1,n_active
          moslab_active(i)=lab_c1
        end do
        do i=1,n_sum
          moslab_refdmo(i)=lab_c1
        end do
        do i=1,ndlap
C          moslab_refps(i)=lab_ci                                         !CK1105
          moslab_refps(i)=lab_c1                                          !CK1105
        end do
      end if 

c
c Set CAS-DMOs for carring out CI 
c

      call daread(idaf,ioda,x(ix3),n1,15) ! read MOs used by MC-QDPT

cc Convert active part  to CAS-DMOs

      do i=1,n3
        ishift1=ix3-1+num*ncorbs
        x(i+ishift1)=x(i+ix4-1)
      end do

cc Write CAS-DMO to #15
      call dawrit(idaf,ioda,x(ix3),n1,15,nav)

cc -- Reset core memory --

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)

     
c========= Calculation of projected MC-QDPT wave fution (CI coeffcient)
c          in CAS-DMOs starts =============================

c
c Symmetry turned OFF  If needed
c

      offsym=.false.
      if ((nstate.gt.1).and.(ndmosym.ne.0)) then ! mod 6/22
        call symoff
        typsav=hndtyp
        hndtyp=direct
        offsym=.true.
      end if


c
c Carry out CI in CAS-DMOs
c


cc Set parameter

      nstate=nstci
       
cc Do CI

      final=.true.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.

      ift12=1
      ift13=1
      ift14=1

      call gugdrt
      
      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
     
      call srtgug
      call gugaem
      call gugadgmod(offsym,nstci) 


c
c     ----- Get CI coefficient of dominant CSFs in DMO base
c

c
c Read data of dominant CSFs if necessary
c
 
      if (nrcsf.ne.0) then
        rewind(ir)
        call rdcsfdia(nstmq,nlist,dcicoef_ref,list_csf,list_group,
     &                maxlist,maxci)
      end if 


c 
c (1) Make CI vectors for projectef MC-QDPT states (QDPT target states
c     only) in CASSCF based DMOs
c (2) Make some lists for dominat CSF to make group G_k 
c     If the list is specified in the card, use it.  
c (3) Search CI coefficients of dominant CSFs and store to the array
c     dcicoef
c Note: FT21 may be used as scratch file in subroutine refdiamq
c

      call refdiamq(list_csf,list_group,dcicoef,nlist,nstmq,nstci,
     &              kstci_mq,coef_max1,e_mcqdpt,eigvec)



c
c  Make adiabatic/diabatic transformation matrix 
c
  
cc -- Get core memory
      ix1=1
      ix2=ix1+nstmq*nstmq
      ix3=ix2+nstmq*nstmq
      need=ix3


      call gethdia(nrcsf,e_mcqdpt,nstmq,nlist,list_group,
     &             list_csf,dcicoef,dcicoef_ref,x(ix1),x(ix2),
     &             ngrstate)                                      !RV 9/21/06


c
c Punch out CSF list, CI coefficients of 'truncated' Diabatic states
c to $CSFDAT (See subroutine tmset)
c

      call pucsfdia(nstmq,nlist,dcicoef,list_csf,list_group,
     &               maxlist,maxci)


cc Reset core memory

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)

c
c ============= diabatization procedure was done =============
c


c
c ======= Clearning up of the MC-QDPT diabatization procedure ======
c


c
c Switch back DA10 #125 to #15
c

cc Get core memory

      n1=num*num   
      ix1=1
      last=ix1+n1
      
      call cmem(loadcm)
      need=last
      call setc(need)

      call daread(idaf,ioda,x,n1,125)
      call dawrit(idaf,ioda,x,n1,15,nav)

cc Reset core memory
      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)
  
!
!---- Now av Fock orbitals for original CASSCF calculation
!     returns to DA10 #15  -----------
!

c
c Symmetry turned ON 
c 
      if (offsym) then 
        call symon 
        hndtyp=typsav
        offsym=.false.
        nstate=nstcas
      end if

c
c One more CI calculation with -MC- orbitals for consistency
c
      final=.false.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.
      
      ift12=1
      ift13=1
      ift14=1

      call gugdrt
  
      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
   
      call srtgug
      call gugaem
      call gugadg
      call gug2dm

c
c Close -DIIS- work file if it was used
c

      if (focas.ne.0) then
        call raclos(idaf30,nav30,iw)
        call rartrn(idaf30)
      end if



      return


      end


      subroutine wfndia_cas(nwfnflg) 

      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXATOM=128)
      PARAMETER (MXIODA=255)
      PARAMETER (MAXCI=50) 
      PARAMETER (MDIIS=19)
c--
      parameter (mxprim=2048)                                !HN 2/4/02
      parameter (ndm_size=16) ! Max size of DM1 based on MO (active)
      parameter (maxlist=64)  
      parameter (maxtot_orb=1024) 

c--
      parameter (mxn_ref=10) ! Max number of the reference geomrtry
                             ! (arrangement)  UNDER CONSTRUCTION 
c--
                                   
      LOGICAL FINAL,MCNOCI,MLMOCI,MCANCI
      LOGICAL NOTRF
      LOGICAL FULLCI
      LOGICAL OFFSYM
      INTEGER FOCAS
C#g77 ------------------------------------------
      CHARACTER*8 DIRECT,diavecw,dfrmvecw,dpsvecw,
     1            frdcw,frvlw,endwrd,hndtyp,typsav
      CHARACTER*4 lab_c1,moslab_active,mcmoslab,moslab_refdmo,
     1            moslab_refps
      CHARACTER*4 label2
C#g77 ------------------------------------------
      COMMON/IOFILE/IR,IW,ip 
      COMMON/DAFILE/IDAF,NAV,IODA(MXIODA)
      COMMON/INFOA/NAT,ICH,MUL,NUM,NX,NE,NA,NB,ZAN(MXATOM),
     1                                         C(3,MXATOM)
      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/CASOPT/CASHFT,CASDII,NRMCAS,FOCAS
      COMMON/NTNOPT/ENERGY,ENERG0,DEMAX,SQCDF,ITER,MICRO,
     1              FINAL,MCNOCI,MLMOCI,MCANCI,
     2              NOCAN,NOCAS,NOCI,LMOCI,NOTRF
      COMMON/QMTOPT/QMTTOL,NQMT
      COMMON/TRFOPT/MCTRF,MPTRF
      COMMON/DRTOPT/FULLCI,IFFULL
      COMMON/DRTPAR/NCO,NMCCO,NCI,MSYM
      COMMON/TRFPAR/NFRZC,NMCORB,NCORB,NOCC,NORB,NBF
      COMMON/ MCPAR/NFZC,NCORBS,NACT,NORBS,NORB1,NTOT,NWKS
      COMMON/DIICAS/NCALL,NDIIS
      COMMON/DIIFIL/IDAF30,NAV30,IODA30(4*MDIIS+5)
      COMMON/HNDOPT/HNDTYP,DSKTYP,FILTYP,PCKTYP
      COMMON/HNDFIL/IFT12,IFT13,IFT14
      COMMON/SCM   /X(1)

      common/mcmossym/mcmoslab(maxtot_orb)
      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,alpha_t,
     &              slctth,moslab_charact(ndm_size),th_doc,th_vir,
     &              conv_th,max_iter,ngrstate 		     !RV 9/21/06	
      common/prototype/n_ref,nmlap(mxn_ref),ndlap,norient    !HN 2/4/02
      common/diatmpfil/nft21,nft22,nft23,nft24    
      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir
      common/xyzlabel_ao/label_type(mxprim,mxatom),          !HN 2/4/02
     &                   label_shell(mxprim,mxatom),         !HN 2/4/02
     &                   num_ao(mxatom),ntot_basis           !HN 2/4/02
      common/mollab/anam(mxatom),bnam(mxatom)                !HN 2/5/02

c-- labels 
      dimension moslab_active(ndm_size)
      dimension moslab_refdmo(ndm_size)
      dimension moslab_refps(ndm_size)                        !HN 2/1/02
      dimension moslab_pseudo(ndm_size)                       !HN 2/1/02
      dimension label2(12)                                    !HN 2/4/02
c---

      dimension avocc_act(ndm_size)  
      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,maxci)
      dimension dcicoef_ref(maxlist,maxci)
      dimension eci_sav(maxci)

      dimension itemp(ndm_size)                             !HN 2/22/02 
c

      DATA ZERO          /0.0D+00/
      data one           /1.0d+00/ 
      data two           /2.0d0/
      DATA DIRECT        /'DIRECT  '/
      data lab_c1       /4hA   /  

      data nft11 /11/
      data nft12 /12/  

      data diavecw /' $DIAVEC'/
      data dfrmvecw /' $DFMVEC'/
      data dpsvecw /' $DPSVEC'/                               !HN 2/1/02
      data frdcw  /' $LCLDC'/                                 !HN 2/5/02
      data frvlw  /' $LCLVL'/                                 !HN 2/5/02
      data endwrd  /' $END'/
 
      data label2 /'s','px','py','pz','dxy','dyz','dzx','dxx',
     &             'dyy','dzz','f','g'/                       !HN 2/4/02

      namelist /diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,
     &                 alpha_t,slctth,moslab_charact,th_doc,
     &                 th_vir,conv_th,max_iter,ngrstate       !RV 9/21/06

      namelist /prototype/n_ref,nmlap,ndlap,norient           !HN 2/4/02

c
c avocc_act(i): averaged occupation # of i-th active MOs 
c
c list_csf(i): dominant CSF index 
c list_group(i): Group index  where list_csf(i)-th CSF belongs to
c dcicoef_ref(i,j): CI coefficents for diabatic states of list_csf(i)-th
c                 CSF for j-th diabatic states
c                    
c These three data are read from $CSFDAT 
c 
c dcicoef(i,j): CI coefficients of list_csf(i)-th CSF
c                 for j-th adiabatic states 
c
c

c==============  Start ====================

c
c ----- Set some parameters for CI --
c

      final=.false.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.
      notrf=.false.
      fullci=.true.

c
c ----- Open -DIIS- work file ---
c


      ncall=0
      ndiis=0
      idaf30=30
      nav30=0
      ndar30=4*mdiis+5
      if (focas.ne.0) then
        call raopen(idaf30,ioda30,nav30,ndar30)
      end if

c
c ---- Set parameters for integral transformation ----
c

!     nco:   # of frozen core
!     nmcco: # of core orbitals
!     nocc:  # of active orbitals

      mptrf=0
      nfrzc=0
      nmcorb=nco+nmcco
      ncorb=nco+nmcco
      norb=nqmt-ncorb
      nocc=nci
      nbf=num

c
c ---- Set parameters for -NTN- algorithm ----
c

!     ncorbs: # of (frozen+core) orbitals
!     norbs:  # of (frozen+core+active) orbitals
!     norb:   # total of MOs
!     num:    # total of AOs (i.e. basis size)

      nfzc=imofzc
      ncorbs=ncorb
      norbs=ncorb+nci
      norb1=nqmt
      nact=nci
      ntot=num

c
c ---- Set parameters for other modules
c

      na=norbs

c
c ---- Set parameters for this modules
c

      n_active=norbs-ncorbs ! (=nci=# of active MOs)

c
c ---- Set default values of parameters for diabatization
c

      call diabset                                           !HN 2/11/02

c
c Check the process 
c : nwfnflg=0--> CAS-based diabatization only
c : nwfnflg=1--> Preconditioner for  MC-QDPT diabatization
c : nwfnflg=2--> MC-QDPT diabatization with CAS-DMOs
c

      if (nwfnflg.eq.0) then 
        write(iw,*) '           '
        write(iw,'(''#*******************************************''/
     &       ''     DETERMINATION OF DIABATIC STATES BASED ON  ''/
     &       ''     CASSCF WAVE FUNCTIONS.                     ''/
     &       ''     PROGRAM WRITTEN BY H.NAKAMURA, D.G.TRUHLAR''/ 
     &       ''      x*****************************************#''/
     &       ''                                                '')')

      else if(nwfnflg.eq.1) then
        write(iw,'(''#*******************************************''/
     &       ''     DETERMINATION OF DIABATIC STATES BASED ON  ''/
     &       ''     MC-QDPT WAVE FUNCTIONS.                     ''/
     &       ''     PROGRAM WRITTEN BY H.NAKAMURA, D.G.TRUHLAR''/ 
     &       ''      x*****************************************#''/
     &       ''                                                '')')
        write(iw,'(''------------------------------------------- ''/
     &       ''     PRELIMINARY PROCEDURE, DMOs of REFERENCED  ''/
     &       ''     CASSCF WAVE FUNCTIONS, STARTS              ''/
     &       ''    --------------------------------------------''/
     &       ''                                                '')')

      else if(nwfnflg.eq.2) then
        write(iw,'(''#*******************************************''/
     &       ''     DETERMINATION OF DIABATIC STATES BASED ON  ''/
     &       ''     MC-QDPT WAVE FUNCTIONS WITH CAS-DMOs       ''/
     &       ''     PROGRAM MODIFIED BY KE R. Yang, D.G.TRUHLAR''/
     &       ''      x*****************************************#''/
     &       ''                                                '')')
    
      end if 
        

c
c Check obtained -MC- orbital is CASSCF type or not 
c

      if (nocas.ne.0) then  
        write(iw,*) 'This diabatization program assumes CASSCF type. '
        write(iw,*) 'Stop'
        stop
      end if 

c
c Check -MC- orbital is averaged Fock orbital or not.
c (In the MC-QDPT calculation, inactive and external orbital is
c (fixed this orbital. )
c

      if ((nwfnflg.ne.0).and.(nocan.ne.0)) then
        write(iw,'(''To estimate MC-QDPT adiabatic energies,''/ 
     &     ''MOs should be fixed as Fock orbital at the first stage.''/
     &     '' Set namelist $MC NOCAN as 0. Stop.'')')
        stop
      end if 



c
c Read namelist -DIABAT-
c

      rewind(ir)
      read(ir,DIABAT,end=5,err=5)
5     continue

      if ((nstry.lt.nstate).or.(ndmosym.eq.0)) nstry=nstate

c
c Read namelist -PROTOTYPE-
c

      rewind(ir)
      read(ir,PROTOTYPE,end=10,err=10)  
10    continue



c
c Write all input parameters for diabtization 
c

      write(iw,'('' #======= Input Values of $DIABAT ======='')')
      write(iw,*) 'parameters correlating to  symmetry of DMOs'
      write(iw,*) 'ndmosym=',ndmosym,' nstry=',nstry
      write(iw,*) 'parameters correlating to dominant CSF selection'
      write(iw,*) 'nrcsf=',nrcsf
      write(iw,*) 'parameters for D3 functions'
      write(iw,'('' alpha_n,alpha_n,alpha_t='',f6.2,f6.2,f6.2)')
     &              alpha_n,alpha_r,alpha_t
      write(iw,*) 'parameters for deformation check'
      write(iw,*) 'ndfrm=',ndfrm
      write(iw,*) 'max iteration number of Jacobi rotation'
      write(iw,*) 'max_iter=',max_iter
      write(iw,*) 'threshold values to determine doc vir block '
      write(iw,'('' th_doc,th_vir='',2f5.3)') th_doc,th_vir
      write(iw,*) 'other threshold parameters'
      write(iw,'('' conv_th='',e9.2,'' slctth='',f5.3)') conv_th,slctth
      write(iw,*) 'number of reference DMOs for the MORMO criterion' 
      write(iw,*) 'nmlap=',nmlap(1)
      write(iw,*) 'Block labels for the fourfold way: moslab_charact'
      write(iw,*) (moslab_charact(i),i=1,norbs-ncorbs)
      write(iw,*) 'number of --pseudo-- reference MOs'       !HN 2/2/02 
      write(iw,*) 'ndlap=',ndlap                             !HN 2/2/02
      write(iw,*) 'norient=',norient                         !HN 2/4/02
      write(iw,*) 'diabatization option for ground state'    !RV 9/21/06
      write(iw,*) 'ngrstate=',ngrstate                       !RV 9/21/06
      write(iw,'('' #============================================'')')
      write(iw,*) '                                          '

c
c Check parameters
c

      if (1.lt.n_ref) then
        write(iw,*) 'This progam version is not complete for n_ref >1'
        write(iw,*) 'Stop'
        stop
      end if 

      n_sum=nmlap(1)
!      if (ndm_size.lt.n_sum) then
      if (n_active.lt.n_sum) then
        write(iw,*) '# of reference DMOs is too large. Stop.'
        stop
      end if 

      if (ndm_size.lt.n_active) then
        write(iw,*) 'Too Large Active Space. Stop'
        stop
      end if


c
c     ----- Get CI Wave function -MC- orbital  -----
c

      ift12=1
      ift13=1
      ift14=1

      call gugdrt
    

      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
     
      call srtgug
      call gugaem
      call gugadg

c
c Back up CAS MO (av Fock orbitals) to DA10 #125
c

cc- Get core memory 
      ix1=1
      last=num*num

      call daread(idaf,ioda,x(ix1),num*num,15)
      call dawrit(idaf,ioda,x(ix1),num*num,125,nav)  

cc- Reset core memory


c
c The branch to MC-QDPT prepation procedure 
c

      if (nwfnflg.ne.0) then 
        write(iw,'(''------------------------------------------------''/
     &       ''      MC-QDPT adiabatic energies are estimated by  ''/
     &       ''      averaged Fock orbitals (inactive,external) and ''/
     &       ''      CAS-based DMOs (active) then diabatic states are''/
     &       ''      evaluated.                          ''/
     &       ''     -----------------------------------------------''/
     &       ''                                                   '')')
      end if 


c
c ======== Construction DMOs based on CASSCF wave functions ==========
c


c
c Get MO/basis DM1 and write to FT21 temporary 
c (each matrix is taken as size=ndm_size*ndm_size)
c

      call gugdmt(n_active) ! n_active= # of active orbitals

       
cc --- Get core memory ---

      n0=n_active*num
      n1=num*num
      n2=num*(num+1)/2
      n3=num*n_active
      n4=n_active*n_active*((nstate*(nstate+1)/2+1)) ! +1--> for SA-DM1
     
      ix1=1      ! ix1--ix2-1: store reference DMOs for MORMO ($DIAVEC)
      ix2=ix1+n0 ! ix2--ix3-1: store AO overlap
      ix3=ix2+n2 ! ix3--ix4-1: temp region  
      ix4=ix3+n1 ! ix4--ix5-1: store DMO/AO 
      ix5=ix4+n3 ! ix5--ix6-1: store DM1
      ix6=ix5+n4 ! ix6--last-1: store pseudeo ref MO for doc block
                                                             !HN 2/1/02
      last=ix6+ndlap*num 

      call cmem(loadcm)
      need=last
      call setc(need)


cc--- Assign irreps of $DFMVEC (if necessary) and check irreps ordering
cc-   of $DFMVEC is same with original CAS MOs to use -NTN- algorithm
cc-   in DMO-CI.
cc-   If the ordering of irreps is different, stop.
cc-   Finally $DFMVEC is symmetric orthogonalized and written to FT22

cc  Read AO overlap

      call daread(idaf,ioda,x(ix2),n2,12)

cc  Write MOs for deformation to scratch file (FT22) 
c-  When ndfrm=0, these MOs are taken as original av Fock orbitals

      if (ndfrm.ne.0) then
cc  Read $DFMVEC, check consistency for DRT, and sym. orthogonalization
        call chcksymord(num,n_active,mcmoslab(ncorbs+1),x(ix1),
     &                  x(ix3),x(ix2),x(ix4)) 
        call symdfmvec(nft22,num,n_active,x(ix1),x(ix2),x(ix3))
      else
        call daread(idaf,ioda,x(ix3),n1,125)
        call wtfuldia(nft22,n3,x(ix3+num*ncorbs))
      end if 


cc--- Transform $DIAVEC based on the orientation,
cc-   Assign irreps of $DIAVEC (if necessary) and 
cc-   make irreps label of $DIAVEC

cc Read $DIAVEC (reference DMOs), Symmetry assign. (moslab_refdmo)

      if (n_sum.ne.0) then 
        call refdmosym(n_sum,num,ndm_size,mxprim,x(ix1),x(ix3),x(ix2),
     &          x(ix4),moslab_refdmo,diavecw,endwrd,frvlw,nat,
     &          anam,num_ao,label_type,label2,norient)       !HN 2/5/02 
      end if  

cc--- Transform $DPSVEC based on the orientation,
cc-   Assign irreps of $DPSVEC (if necessary) and make irreps label
cc    Symmetry assign (moslab_refps)

      if (ndlap.ne.0) then                                  !HN 2/4/02
        call refdmosym(ndlap,num,ndm_size,mxprim,x(ix6),x(ix3),x(ix2), 
     &          x(ix4),moslab_refps,dpsvecw,endwrd,frdcw,nat,
     &          anam,num_ao,label_type,label2,norient)      !HN 2/5/02
      end if                                                !HN 2/4/02

c---- Debug print out transformed $DPSVEC---

      if (n_sum.ne.0) then 
        write(iw,*) '----Transformed (standard orienation) $DIVEC---'
        write(iw,*) n_sum, '---debug print----' 
        call prevls(x(ix1),avocc_act,moslab_refdmo,n_sum,num,num)
        write(iw,*) '------------------------------------------------'
      end if 

      if (ndlap.ne.0) then 
        write(iw,*) '----Transformed (standard orienation) $DPSVEC---'
        write(iw,*) ndlap, '---debug print----' 
        call prevls(x(ix6),avocc_act,moslab_refps,ndlap,num,num)
        write(iw,*) '------------------------------------------------'
      end if 

c-------------------------------------------


cc Pick up the active MO coefficients 

      call daread(idaf,ioda,x(ix3),n1,15)
      do i=1,n_active
        ishift1=ix3-1+num*(ncorbs+i-1)
        ishift2=ix4-1+num*(i-1)
        do j=1,num
          x(j+ishift2)=x(j+ishift1)
        end do
      end do

!-- Now x(ix1)--> reference DMOs (i.e. $DIAVEC)
!       x(ix2)--> AO overlap
!       x(ix4)--> active MO/AO 
!       x(ix6)---> pseudo reference MO (i.e. $DPSVEC)
!---       


c
c -- Optimize orbitals to Diabatic MOs ---
c


      L1=nstate*(nstate+1)/2
      L2=ndm_size*ndm_size
      L3=n_active*n_active


cc  Read all DM1 and also add SA DM1 to data
c-  On the memory size, each matrix size  should be change 
c-  from ndm_size**2 to n_active**2

      
      call arrange_dm(x(ix5),x(ix3),nstate,n_active,ndm_size,nft21)

!   
! Now x(ix5) is all DM1 + SA DM1 (each data size=n_active*n_active)
!

c
cc Make irreps label of the active MOs. If DMOs are not required
c- for symmetry conservation, all label are replaced by A 
c- Also, irreps label of reference DMOs for MORMO are replaced
c

      do i=1,n_active
        moslab_active(i)=mcmoslab(ncorbs+i)
      end do

      if (ndmosym.ne.0) then 
        do i=1,n_active
          moslab_active(i)=lab_c1
        end do
        do i=1,n_sum
          moslab_refdmo(i)=lab_c1
        end do
        do i=1,ndlap                                         !HN 2/1/02
          moslab_refps(i)=lab_c1                             !HN 2/1/02
        end do                                               !HN 2/1/02
      end if

c
c DMO optimization statrs. The procedure consists of ....
c (1) The Threefold density criterion
c (2) Apply the fourfold way to DOc-candidate MOs
c (3) Separate active space to 3 (or 2 or 1) block as 'doc,val,vir'
c (3) The fourfold way with (block) MORMO criterion 
c (4) deformation check 
c

c
c   Step (1) Carry out the threefold density criterion 
c            and transform MO coefficients. DMs (x(ix5)) and MO coefs
c            (x(ix4)) are updated. 
c             
c

      call threefold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,nstate,
     &               num,n_active,moslab_active,x(ix5),x(ix4),
     &               avocc_act,x(ix3))

c
c   Step (2) Pick up  Doc block candidate (if necessary)      !HN 2/2/02
c

      ndoc=0
      do i=1,n_active
        if (moslab_charact(i).eq.nsymbol_doc) ndoc=ndoc+1
      end do

      ndoc_cand=0                                             !HN 2/2/02

      if ((ndfrm.eq.0).or.(ndlap.eq.0).or.(ndoc.eq.0))  goto 100             !HN 2/2/02

      write(iw,*) '-------------------------------------------'
      write(iw,*) 'Catch and Release (C-R) algorithm is appled for Doc'
      write(iw,*) 'block by the fourfold way.'
      write(iw,*) '-------------------------------------------'
      
      do i=1,n_active                                         !HN 2/2/02
        if (th_doc.lt.avocc_act(i)) then                      !HN 2/2/02
          ndoc_cand=ndoc_cand+1
! Doc block candidates are treated as target of CR algorithm
! Thus such MOs are identified "nsymbol_val" because they are
! applied to the fourfold way
          moslab_pseudo(i)=nsymbol_val                        !HN 2/2/02
        else                                                  !HN 2/2/02
          moslab_pseudo(i)=nsymbol_vir                        !HN 2/2/02
        end if                                                !HN 2/2/02
      end do                                                  !HN 2/2/02
! check 
      if (ndoc_cand.lt.ndoc) then
        write(iw,*) '# of candidates MO for Dc block is too small'
        write(iw,*)'Catch & Release algorithm is NOT applied. Continue'
        goto 100
      else 
        write(iw,*) ndoc_cand,' MOs are caught '
        write(iw,*) 'then ', ndoc_cand-ndoc,' are released.'
      end if 

! Sort of MO in the caught part to accelate convergence
      icount=0                                              !HN 2/22/02
      do i=1,n_active                                       !HN 2/22/02
        if (th_doc.lt.avocc_act(i)) then                    !HN 2/22/02
          icount=icount+1                                   !HN 2/22/02
          itemp(icount)=i                                   !HN 2/22/02
        end if                                              !HN 2/22/02
      end do                                                !HN 2/22/02
      call presort(nstate,num,n_active,ndoc_cand,ndlap,itemp,x(ix4),
     &   x(ix6),x(ix2),x(ix3),x(ix5),avocc_act,moslab_active)!HN 2/22/02



! Apply fourfold to Doc-candidtates
      call fourfold(alpha_n,alpha_r,alpha_t,conv_th,max_iter, !HN 2/2/02
     &     nstate,num,n_active,ndlap,moslab_active,           !HN 2/2/02
     &     moslab_pseudo,moslab_refps,x(ix5),x(ix4),x(ix6),   !HN 2/5/02
     &     x(ix2),avocc_act,x(ix3))                           !HN 2/2/02

100   continue                                                !HN 2/2/02


c
c Step (3)  Define the block (doc/val/vir) by using $DFMVEC, the 
c           updated MO, and occupation number etc.
c

      call detblock(nft22,ndfrm,num,n_active,nstate,moslab_active,
     &          moslab_charact,avocc_act,th_doc,th_vir,x(ix4),
     &          x(ix3),x(ix2),x(ix5),ndoc_cand,moslab_pseudo) !HN 2/2/02


c
c Step (4) The fourfold way for Val block as the final determination  
c

      if (n_sum.ne.0) then 
        write(iw,*) '-------------------------------------------'
        write(iw,*) 'The used DMOs block for the MORMO (may be changed'
        write(iw,*) '(may be changed with the unput moslab_charact'
        write(iw,*) 'moslab_charact array'
        write(iw,*)  (moslab_charact(i),i=1,n_active)
        write(iw,*) '-------------------------------------------'
        write(iw,*) '                                           '

! Sort of MO in the caught part to accelate convergence
        icount=0                                             !HN 2/22/02
        do i=1,n_active                                      !HN 2/22/02
          if (moslab_charact(i).eq.nsymbol_val) then         !HN 2/22/02
            icount=icount+1                                  !HN 2/22/02
            itemp(icount)=i                                !HN 2/22/02
          end if                                             !HN 2/22/02
        end do                                               !HN 2/22/02
        call presort(nstate,num,n_active,icount,n_sum,itemp,x(ix4),
     &   x(ix1),x(ix2),x(ix3),x(ix5),avocc_act,moslab_active)!HN 2/22/02

! Apply the fourfold 
        call fourfold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &                nstate,num,n_active,n_sum,moslab_active,
     &                moslab_charact,moslab_refdmo,x(ix5),x(ix4),
     &                x(ix1),x(ix2),avocc_act,x(ix3))
      end if


c
c Step (4)  Order DMOs finally 
c

      call orddmo(nft22,num,n_active,nstate,moslab_charact,
     &            moslab_active,x(ix5),avocc_act,x(ix4),x(ix3),x(ix2))

      if (((nwfnflg.eq.0).or.(nwfnflg.eq.2)).and.(ndfrm.eq.1)) then       !KY 01/15/13
        write(iw,'(''---------------------------------------------''/
     &        ''   Overlap-like quantity matrix of the DMOs of the ''/
     &        ''   fourfold way by CASSCF wave functions ''/
     &        ''   ------------------------------------------------''/
     &        ''                                                 '')')
        call prlapdmo(nft22,num,n_active,moslab_charact,x(ix4),x(ix3),
     &                x(ix2))
      end if 

c
c  Punch out DMO/AO  as $DFMVEC
c
      if (nwfnflg.eq.0) then
        write(ip,*) '--- Diabatic Orbitals Based on CASSCF (active) ---'
        write(ip,'(a8)') dfrmvecw
        call pusql(x(ix4),n_active,num,num)
        write(ip,'(a8)') endwrd
      else if(nwfnflg.eq.2) then    ! KY 09/18/12
        write(ip,*) '--- Diabatic Orbitals Based on CASSCF (active) ---'
        write(ip,'(a8)') dfrmvecw
        call pusql(x(ix4),n_active,num,num)
        write(ip,'(a8)') endwrd
      end if 

c
c Print out DMO
c
      if (nwfnflg.eq.0) then
        write(iw,'(''(--------------------------------------------''/
     &       '' - Diabatic Orbitals (DMOs) expanded by AOs - ''/
     &       ''(DMOs may not satisfy irreps)''/
     &       ''                                  '')')
        write(iw,*) '---- DMOs based on CASSCF (active) ----'
        write(iw,*) nmlap(1), 'Active MOs are determined by MORMO'
        call prevls(x(ix4),avocc_act,moslab_active,n_active,num,num)
      else if(nwfnflg.eq.2) then    ! KY 01/15/13
        write(iw,'(''(--------------------------------------------''/
     &       '' - Diabatic Orbitals (DMOs) expanded by AOs - ''/
     &       ''(DMOs may not satisfy irreps)''/
     &       ''                                  '')')
        write(iw,*) '---- DMOs based on CASSCF (active) for MC-QDPT----'
        write(iw,*) nmlap(1), 'Active MOs are determined by MORMO'
        call prevls(x(ix4),avocc_act,moslab_active,n_active,num,num)
      end if 

c
c  Prepare inactive MO, active DMO and external MO
c

      call daread(idaf,ioda,x(ix3),n1,125)  ! read original CAS MO


cc Convert active part to DMOs

      do i=1,n3
        ishift1=ix3-1+num*ncorbs
        x(i+ishift1)=x(i+ix4-1)
      end do

!
! Now x(ix3) (length n1) is DMO
! (inactive and external are same with original CAS MO)
!

cc Write to #15

      call dawrit(idaf,ioda,x(ix3),n1,15,nav) 


!
! Now CASSCF-based DMOs are write down to DA10 #15
! to use CI in DMO base
!

cc -- Reset core memory --

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)


c-----   DMOs (based on CASSCF) determination is done ---------------


c
c======== Carry out one CI calculation by using new MOs ============
c
c (nwfnflg=0,1....DA10 # 15 is DMOs(inactive and external is av Fock)
c


cc  Save original # of states and obtained CI eneries

      nstcas=nstate
      do i=1,nstcas
        eci_sav(i)=eci(i)
      end do


cc Symmetry turned OFF  If needed

      offsym=.false.
      if ((nstate.gt.1).and.(ndmosym.ne.0)) then 
        call symoff
        typsav=hndtyp
        hndtyp=direct
        offsym=.true.
      end if

cc     ----- Get CI vectors in new MO basis ----

cc Carry out CI 

      final=.true.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.

      ift12=1
      ift13=1
      ift14=1

      call gugdrt
     
      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
 
      call srtgug
      call gugaem
      call gugadgmod(offsym,nstry)

c
c =========== Final part of the procedure   =================
c
c branch (1) Final procedure of CASSCF diabatization 
c           
c        (2) Final procedure of prepation of MC-QDPT
c
c

      if (nwfnflg.eq.0) then  ! branch (1)

c
c (i) Make some lists for dominat CSF to make group G_k 
c     If the list is specified in the card, use it.  
c (ii) Search CI coefficients of dominant CSFs and store to the array
c     dcicoef
c Note: FT21 may be used as scratch file in subroutine refdia
c

cc-- Read data of dominant CSFs if necessary
        if (nrcsf.ne.0) then
          rewind(ir)
          call rdcsfdia(nstcas,nlist,dcicoef_ref,list_csf,list_group,
     &                 maxlist,maxci)
        end if 

        call refdia(list_csf,list_group,dcicoef,nlist,
     &                nstcas,eci_sav)

cc  Make adiabatic/diabatic transformation matrix 

cc -- Get core memory
        ix1=1
        ix2=ix1+nstcas*nstcas
        ix3=ix2+nstcas*nstcas
        need=ix3

        call gethdia(nrcsf,eci_sav,nstcas,nlist,list_group,
     &           list_csf,dcicoef,dcicoef_ref,x(ix1),x(ix2),
     &           ngrstate)                                        !RV 9/21/06


cc Punch out CSF list, CI coefficients of 'truncated' Diabatic states
c- to $CSFDAT (See subroutine tmset)

        call pucsfdia(nstcas,nlist,dcicoef,list_csf,list_group,
     &               maxlist,maxci)

cc--  Reset core memory

        call cmem(ngotcm)
        if (ngotcm.ne.loadcm) call setc(loadcm)

      else ! branch (2)

c
c  Target CAS-CI vectors in FT12 is copied to FT21
c  This data will be used in MC-QDPT diabatization.
c
c

        rewind(nft12) 
        rewind(nft21)

        read(nft12)  nstry,nwks ! header
        write(nft21) nstcas,nwks
c---get core memory
        ix=1
        last=ix1+nwks
        call diabmqwr1(nft12,nft21,ndmosym,nstry,
     &                 eci_sav,eci,nwks,x(ix1))
c---reset core memory

      end if 
!================ End the final procedure =====================


c
c  To take consistency, carry out one more CI as clearing up of 
c  CASSCF diabatization 
c

      if (nwfnflg.eq.0) then 

cc Switch back DA10 #125 to #15

cc Get core memory

        n1=num*num   
        ix1=1
        last=ix1+n1
      
        call cmem(loadcm)
        need=last
        call setc(need)

        call daread(idaf,ioda,x(ix1),n1,125)
        call dawrit(idaf,ioda,x(ix1),n1,15,nav)

cc Reset core memory
        call cmem(ngotcm)
        if (ngotcm.ne.loadcm) call setc(loadcm)
    
cc Symmetry turned ON 

        if (offsym) then 
          call symon 
          hndtyp=typsav
          offsym=.false.
          nstate=nstcas
        end if


cc One more CI calculation with -MC- orbitals for consistency

        final=.false.
        mcanci=.false.
        mcnoci=.false.
        mlmoci=.false.
      
        ift12=1
        ift13=1
        ift14=1

        call gugdrt
    
        if (focas.ne.0) call castrf
        if (focas.eq.0.and.mctrf.eq.1) call trfint
        if (focas.eq.0.and.mctrf.ne.1) call trfmcx
     
        call srtgug
        call gugaem
        call gugadg

        call gug2dm

      end if 

!==============End of taking consistency for CASSCF ==========


c
c Close -DIIS- work file if it was used
c

      if (focas.ne.0) then
        call raclos(idaf30,nav30,iw)
        call rartrn(idaf30)
      end if


      return

      
      end

      subroutine wfndia_mcqdpt

      implicit real*8 (a-h,o-z)

      parameter (mxatom=128)
      parameter (mxioda=255)
      parameter (maxci=50)
      parameter (mdiis=19)
c--

      parameter (mxprim=2048)  
      parameter (ndm_size=16) 
      parameter (maxlist=64)
      parameter (maxtot_orb=1024)

c--
      parameter (mxn_ref=10) !  Max number of the reference geomrtry
                             ! (arrangement)  UNDER CONSTRUCTION 
c--
                                   
      LOGICAL FINAL,MCNOCI,MLMOCI,MCANCI
      LOGICAL NOTRF
      LOGICAL FULLCI
      LOGICAL OFFSYM
      INTEGER FOCAS
C#g77 ------------------------------------------
      CHARACTER*8 DIRECT,diavecw,dfrmvecw,dpsvecw,
     1            frdcw,frvlw,endwrd,hndtyp,typsav
      CHARACTER*4 lab_c1,moslab_active,moslab_refdmo,
     1            moslab_refps,mcmoslab
      CHARACTER*4 label2
C#g77 ------------------------------------------
      COMMON/IOFILE/IR,IW,ip 
      COMMON/DAFILE/IDAF,NAV,IODA(MXIODA)
      COMMON/INFOA/NAT,ICH,MUL,NUM,NX,NE,NA,NB,ZAN(MXATOM),
     1                                         C(3,MXATOM)
      COMMON/ENRGCI/EN,EC,ECI(MAXCI),NSTATE
      COMMON/CASOPT/CASHFT,CASDII,NRMCAS,FOCAS
      COMMON/NTNOPT/ENERGY,ENERG0,DEMAX,SQCDF,ITER,MICRO,
     1              FINAL,MCNOCI,MLMOCI,MCANCI,
     2              NOCAN,NOCAS,NOCI,LMOCI,NOTRF
      COMMON/QMTOPT/QMTTOL,NQMT
      COMMON/TRFOPT/MCTRF,MPTRF
      COMMON/DRTOPT/FULLCI,IFFULL
      COMMON/DRTPAR/NCO,NMCCO,NCI,MSYM
      COMMON/TRFPAR/NFRZC,NMCORB,NCORB,NOCC,NORB,NBF
      COMMON/ MCPAR/NFZC,NCORBS,NACT,NORBS,NORB1,NTOT,NWKS
      COMMON/DIICAS/NCALL,NDIIS
      COMMON/DIIFIL/IDAF30,NAV30,IODA30(4*MDIIS+5)
      COMMON/HNDOPT/HNDTYP,DSKTYP,FILTYP,PCKTYP
      COMMON/HNDFIL/IFT12,IFT13,IFT14
      COMMON/SCM   /X(1)

      common/mcmossym/mcmoslab(maxtot_orb)
      common/diabat/ndmosym,nstry,ndfrm,nrcsf,alpha_n,alpha_r,alpha_t,
     &              slctth,moslab_charact(ndm_size),th_doc,th_vir,
     &              conv_th,max_iter,ngrstate                           !RV 9/21/06
      common/prototype/n_ref,nmlap(mxn_ref),ndlap,norient
      common/diabmqpar1/nstci,nstcas,nstmq,kstci_cas(50),kstci_mq(50),
     &                  kstcas_mq(50)
      common/diabmqpar2/coef_max1(20),coef_max2(20),
     &                  eigvec(400),e_mcqdpt(20)
      common/symboldmo/nsymbol_doc,nsymbol_val,nsymbol_vir
      common/diatmpfil/nft21,nft22,nft23,nft24  
      common/xyzlabel_ao/label_type(mxprim,mxatom), 
     &                   label_shell(mxprim,mxatom),
     &                   num_ao(mxatom),ntot_basis 
      common/mollab/anam(mxatom),bnam(mxatom)

c--- labels 
      dimension moslab_active(ndm_size)
      dimension moslab_refdmo(ndm_size)
      dimension moslab_refps(ndm_size)
      dimension moslab_pseudo(ndm_size)
      dimension label2(12)
c---

      dimension avocc_act(ndm_size) 
      dimension list_csf(maxlist)
      dimension list_group(maxlist)
      dimension dcicoef(maxlist,maxci)
      dimension dcicoef_ref(maxlist,maxci)
      dimension title(10),title1(10)
      dimension itemp(ndm_size)                              !HN 2/22/02


      data zero          /0.0D+00/
      data one           /1.0d+00/ 
      data two           /2.0d0/
      DATA DIRECT        /'DIRECT  '/
      data lab_c1       /4hA   /   

      data nft11 /11/
      data nft12 /12/  

      data diavecw /' $DIAVEC'/
      data dfrmvecw /' $DFMVEC'/
      data dpsvecw /' $DPSVEC'/
      data frdcw /' $LCLDC'/
      data frvlw /' $LCLVL'/
      data endwrd  /' $END   '/
 
      data label2 /'s','px','py','pz','dxy','dyz','dzx','dxx',
     &             'dyy','dzz','f','g'/

c
c
c avocc_act(i): averaged occupation # of i-th active MOs 
c list_csf(i): dominant CSF index 
c list_group(i): Group index  where list_csf(i)-th CSF belongs to
c dcicoef_ref(i,j): CI coefficents for diabatic states of list_csf(i)-th
c                 CSF for j-th diabatic states
c                    
c These three data are read from $CSFDAT 
c 
c dcicoef(i,j): CI coefficients of list_csf(i)-th CSF
c               for j-th adiabatic states 
c
c


c
c ----- Set some parameters for CI --
c

      final=.false.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.
      notrf=.false.
      fullci=.true.

c
c ----- Open -DIIS- work file ---
c


      ncall=0
      ndiis=0
      idaf30=30
      nav30=0
      ndar30=4*mdiis+5
      if (focas.ne.0) then
        call raopen(idaf30,ioda30,nav30,ndar30)
      end if

c
c ---- Set parameters for integral transformation ----
c

!     nco:   # of frozen core
!     nmcco: # of core orbitals
!     nocc:  # of active orbitals

      mptrf=0
      nfrzc=0
      nmcorb=nco+nmcco
      ncorb=nco+nmcco
      norb=nqmt-ncorb
      nocc=nci
      nbf=num

c
c ---- Set parameters for -NTN- algorithm ----
c

!     ncorbs: # of (frozen+core) orbitals
!     norbs:  # of (frozen+core+active) orbitals
!     norb:   # total of MOs
!     num:    # total of AOs (i.e. basis size)

      nfzc=imofzc
      ncorbs=ncorb
      norbs=ncorb+nci
      norb1=nqmt
      nact=nci
      ntot=num

c
c ---- Set parameters for other modules
c

      na=norbs

c
c ---- Set parameters for this module
c

      n_active=norbs-ncorbs


      write(iw,'(''---------------------------------------------''/
     &       ''     Diabatization of MC-QDPT wave functions starts''/
     &       ''     -------------------------------------------''/
     &       ''                                                  '')')


c=========== Prepation of 4fold way for MC-QDPT wave function =========

c
c Notice: (1) Input $DIABAT and $DFMVEC is already done in wfndia_cas
c             thus skip
c         (2) By the subroutine wfndia_cas, the CI vectors of MC-QDPT
c         are in FT21, and sym. orthogonalized $DFMVEC is stored in FT22
c         already.
c         (3) Check of symmetry of was already done
c         in the wfndia_cas; thus skip.
c



c
c Read prepaerd data from scrath files to make projected MC-QDPT
c wave function 
c

!  Read QDPT target CAS-CI vectors 

      rewind(nft21)
      read(nft21) idum,nwks
      
cc-- Get core memory ----

      ix1=1                  ! ix1--ix2-1: CI vectors 
      ix2=ix1+nwks*nstmq     ! ix2--ix3-1: temp
      ix3=ix2+nwks           
      last=ix3
      call cmem(loadcm)
      need=last
      call setc(need)


      icount=0
      do i=1,nstcas
        if (kstcas_mq(i).eq.0) then
          call rdfuldia(nft21,nwks,x(ix3))  ! skip
        else 
          icount=icount+1
          ihead=ix1+nwks*(icount-1)
          call rdfuldia(nft21,nwks,x(ihead))
        end if
      end do


c
c Calculate new CI vectors of projected MC-QDPT wave function
c and write them to FT12 
c

!-- Change header part of FT12 
      rewind(nft12)
      read(nft12) idum,idum,title,title1
      rewind(nft12)
      write(nft12) nstmq,nwks,title,title1
!--

      call mkqdptvec(nft12,x(ix1),eigvec,x(ix2),nwks,nstmq,
     &               coef_max1,coef_max2)


c
c Set parameter to calculate DM1 by using gugdmt
c

      nstate=nstmq


cc--- Reset core memory 

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)
c
c ---- Now scratch FT21 is released ----
c



cc======== Diabatization procedure start ===========


c
c Get MO-basis DM1 and write to FT21 temporary 
c

      call gugdmt(n_active) ! n_active= # of active orbitals
      if (ndm_size.lt.n_active) then
        write(iw,*) 'Too Large Active Space. Stop'
        stop
      end if

c
c Determination DMOs based on MC-QDPT wave functions
c


      n_sum=nmlap(1)
       

cc --- Get core memory ---

      n0=num*n_active
      n1=num*num
      n2=num*(num+1)/2
      n3=num*n_active
      n4=n_active*n_active*(nstate*(nstate+1)/2+1) ! +1--> for SA DM1
     
      ix1=1        ! ix1--ix2-1: store reference DMOs
      ix2=ix1+n0   ! ix2--ix3-1: store AO overlap
      ix3=ix2+n2   ! ix3--ix4-1: temp region  
      ix4=ix3+n1   ! ix4--ix5-1: store DMO/AO 
      ix5=ix4+n3   ! ix5--ix6-1: store DM1
      ix6=ix5+n4   ! ix6--last-1: store pseudo reference DMOs
      last=ix6+ndlap*num

      call cmem(loadcm)
      need=last
      call setc(need)

cc Read AO overlap

      call daread(idaf,ioda,x(ix2),n2,12)

cc--- Transform $DIAVEC based on the specified orientation
cc--- Assign irreps of $DIAVEC (if necessary) and make irreps label 
cc    of $DIAVEC

cc Read $DIAVEC, Symmetry assign (moslab_refdmo)

      if (n_sum.ne.0) then
        call refdmosym(n_sum,num,ndm_size,mxprim,x(ix1),x(ix3),x(ix2),
     &                 x(ix4),moslab_refdmo,diavecw,endwrd,frvlw,nat,
     &                 anam,num_ao,label_type,label2,norient)
      end if

cc--- Transform $DPSVEC based on the specified orientation.
cc--- Assign irreps of $DPSVEC (if necessary) and make irreps label 
cc    Symmetry assign (moslab_refps)

      if (ndlap.ne.0) then
        call refdmosym(ndlap,num,ndm_size,mxprim,x(ix6),x(ix3),x(ix2),
     &                 x(ix4),moslab_refps,dpsvecw,endwrd,frdcw,nat,
     &                 anam,num_ao,label_type,label2,norient)
      end if


cc Pick up the active MO coefficients  (only active part)
c- (CASSCF-based DMOs)

      call daread(idaf,ioda,x(ix3),n1,15)
      do i=1,n_active
        ishift1=ix3-1+num*(ncorbs+i-1)
        ishift2=ix4-1+num*(i-1)
        do j=1,num
          x(j+ishift2)=x(j+ishift1)   
        end do
      end do

!-- Now x(ix1)--> reference DMOs (i.e. $DIAVEC)
!       x(ix2)--> AO overlap
!       x(ix4)--> active MO (DA10 #15)
!       x(ix6)---> pseudo reference MO (i.e. $DPSVEC)

c
c -- Optimize orbitals to MC-QDPT based Diabatic MOs ---
c

      L1=nstate*(nstate+1)/2
      L2=ndm_size*ndm_size
      L3=n_active*n_active


cc  Read all DM1 and also add SA DM1 to data

      call arrange_dm(x(ix5),x(ix3),nstate,n_active,ndm_size,nft21)

 
! Now x(ix5) is all DM1 + SA DM1

c
c Make irreps label of active MOs. This procedure is completely same
c with the prepation procedure in the subroutine wfndia_cas 
c

      do i=1,n_active
        moslab_active(i)=mcmoslab(ncorbs+i)
      end do
    
      if (ndmosym.ne.0) then
        do i=1,n_active
          moslab_active(i)=lab_c1
        end do
        do i=1,n_sum
          moslab_refdmo(i)=lab_c1
        end do
        do i=1,ndlap
C          moslab_refps(i)=lab_ci                                         !CK1105
          moslab_refps(i)=lab_c1                                          !CK1105
        end do
      end if 

c
c DMO optimization starts. The procedure consists of 
c (1) Threefold density criterion or use ifo of CAS based DMOs 
c (2) Apply the fourfold way to Doc-candidate MOs
c (2) Separate active space to 3 (or 2 or 1) block
c (3) The fourfold way with (block) MORMO criterion
c (4) Deformation check
c

!------ See this process ------
c
c Step (1)
c
      if (n_sum.eq.0) then  
        call threefold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,nstate,
     &                 num,n_active,moslab_active,x(ix5),x(ix4),
     &                 avocc_act,x(ix3))

      else ! use data of wfn_cas 
        idiag=ix5+n_active*n_active*(nstate*(nstate+1)/2)
        do i=1,n_active
          avocc_act(i)=x(idiag)
          idiag=idiag+n_active+1
        end do
      end if  

c
c Step (2)
c

       ndoc=0
       do i=1,n_active
         if (moslab_charact(i).eq.nsymbol_doc) ndoc=ndoc+1
       end do
 
       ndoc_cand=0

       if ((ndfrm.eq.0).or.(ndlap.eq.0).or.(ndoc.eq.0)) goto 100

       write(iw,*) '-----------------------------------------------'
       write(iw,*) ' C-R algorithm is applied to Doc block, '
       write(iw,*) ' for MC-QDPT wave functions                  '
       write(iw,*) '------------------------------------------------'

       do i=1,n_active
         if (th_doc.lt.avocc_act(i)) then 
           ndoc_cand=ndoc_cand+1
           moslab_pseudo(i)=nsymbol_val
         else
           moslab_pseudo(i)=nsymbol_vir
         end if
       end do

       if (ndoc_cand.lt.ndoc) then 
         write(iw,*) '# of candidate MO for DC block is too small'
         write(iw,*) 'C-R algorithm is NOT applied. Contine'
         goto 100
       else 
         write(iw,*) ndoc_cand,' MOs are caught based on MC-QDPT wf'
         write(iw,*) 'then ', ndoc_cand-ndoc,' are released.'
       end if

! Sort of MO in the caught part to accelate convergence
       icount=0                                              !HN 2/22/02
       do i=1,n_active                                       !HN 2/22/02
         if (th_doc.lt.avocc_act(i)) then                    !HN 2/22/02
           icount=icount+1                                   !HN 2/22/02
           itemp(icount)=i                                 !HN 2/22/02
         end if                                              !HN 2/22/02
       end do                                                !HN 2/22/02
       call presort(nstate,num,n_active,ndoc_cand,ndlap,itemp,x(ix4),
     &  x(ix6),x(ix2),x(ix3),x(ix5),avocc_act,moslab_active) !HN 2/22/02


! Apply fourfold 

       call fourfold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &     nstate,num,n_active,ndlap,moslab_active,
     &     moslab_pseudo,moslab_refps,x(ix5),x(ix4),x(ix6),x(ix2),
     &     avocc_act,x(ix3))

100    continue

c
c Step (3)
c


       call detblock(nft22,ndfrm,num,n_active,nstate,moslab_active,
     &              moslab_charact,avocc_act,th_doc,th_vir,x(ix4),
     &              x(ix3),x(ix2),x(ix5),ndoc_cand,moslab_pseudo)


c!      if (ndfrm.eq.1) then
c!        call prlapdmo(nft22,num,n_active,moslab_charact,x(ix4),x(ix3),
c!     &              x(ix2))
c!      end if 


c
c Step (4)
c

      if (n_sum.ne.0) then
        write(iw,*) '----------------------------------------------'
        write(iw,*) 'The used DMOs block for the MORMO '
        write(iw,*) '(may be changed with the input moslab_charact'
        write(iw,*)  (moslab_charact(i),i=1,n_active)
        write(iw,*) '----------------------------------------------'
        write(iw,*) '                                              '

! Sort of MO in the caught part to accelate convergence
        icount=0                                             !HN 2/22/02
        do i=1,n_active                                      !HN 2/22/02
          if (moslab_charact(i).eq.nsymbol_val) then         !HN 2/22/02
            icount=icount+1                                  !HN 2/22/02
            itemp(icount)=i                                !HN 2/22/02
          end if                                             !HN 2/22/02
        end do                                               !HN 2/22/02
        call presort(nstate,num,n_active,icount,n_sum,itemp,x(ix4),
     &  x(ix1),x(ix2),x(ix3),x(ix5),avocc_act,moslab_active) !HN 2/22/02

! Apply the fourfold 
        call fourfold(alpha_n,alpha_r,alpha_t,conv_th,max_iter,
     &                nstate,num,n_active,n_sum,moslab_active,
     &                moslab_charact,moslab_refdmo,x(ix5),x(ix4),
     &                x(ix1),x(ix2),avocc_act,x(ix3) )
      end if


c
c Step (5) Order DMOs finally
c

      call orddmo(nft22,num,n_active,nstate,moslab_charact,
     &            moslab_active,x(ix5),avocc_act,x(ix4),x(ix3),x(ix2))

      if (ndfrm.eq.1) then
        write(iw,'(''----------------------------------------------''/
     &        ''   Overlap-like quantity matrix of the DMOs of the ''/
     &        ''   fourfold way by MC-QDPT wave functions ''/
     &        ''   ------------------------------------------------''/
     &        ''                                                  '')')
        call prlapdmo(nft22,num,n_active,moslab_charact,x(ix4),x(ix3),
     &               x(ix2))
      end if

        
c
c Punch out DMO/AO  ($DFMVEC)
c
      write(ip,*) '-- Diabatic Orbitals Based on MC-QDPT (active) -- '
      write(ip,'(a8)') dfrmvecw
      call pusql(x(ix4),n_active,num,num)
      write(ip,'(a8)') endwrd

cc Print out DMO/AO
      write(iw,'(''(--------------------------------------------''/
     &       '' - Diabatic Orbitals (DMOs) expanded by AOs - ''/
     &       ''(DMOs may not satisfy irreps)'')')
      write(iw,*) '---- DMOs based on MC-QDPT (active) ----'

      write(iw,*) nmlap(1), 'Active MOs are determined by MORMO'
      call prevls(x(ix4),avocc_act,moslab_active,n_active,num,num)

c
c Set MC-QDPT based DMOs for carring out CI 
c

      call daread(idaf,ioda,x(ix3),n1,15) ! read MOs used by MC-QDPT

cc Convert active part  to MC-QDPT based DMOs

      do i=1,n3
        ishift1=ix3-1+num*ncorbs
        x(i+ishift1)=x(i+ix4-1)
      end do

cc Write MC-QDPT base DMO to #15
      call dawrit(idaf,ioda,x(ix3),n1,15,nav)

cc -- Reset core memory --

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)

     
c========= Calculation of projected MC-QDPT wave fution (CI coeffcient)
c          in MC-QDPT based DMOs starts =============================

c
c Symmetry turned OFF  If needed
c

      offsym=.false.
      if ((nstate.gt.1).and.(ndmosym.ne.0)) then ! mod 6/22
        call symoff
        typsav=hndtyp
        hndtyp=direct
        offsym=.true.
      end if


c
c Carry out CI in MC-QDPT based DMOs
c


cc Set parameter

      nstate=nstci
       
cc Do CI

      final=.true.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.

      ift12=1
      ift13=1
      ift14=1

      call gugdrt
      
      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
     
      call srtgug
      call gugaem
      call gugadgmod(offsym,nstci) 


c
c     ----- Get CI coefficient of dominant CSFs in DMO base
c

c
c Read data of dominant CSFs if necessary
c
 
      if (nrcsf.ne.0) then
        rewind(ir)
        call rdcsfdia(nstmq,nlist,dcicoef_ref,list_csf,list_group,
     &                maxlist,maxci)
      end if 


c 
c (1) Make CI vectors for projectef MC-QDPT states (QDPT target states
c     only) in MC-QDPT based DMOs
c (2) Make some lists for dominat CSF to make group G_k 
c     If the list is specified in the card, use it.  
c (3) Search CI coefficients of dominant CSFs and store to the array
c     dcicoef
c Note: FT21 may be used as scratch file in subroutine refdiamq
c

      call refdiamq(list_csf,list_group,dcicoef,nlist,nstmq,nstci,
     &              kstci_mq,coef_max1,e_mcqdpt,eigvec)



c
c  Make adiabatic/diabatic transformation matrix 
c
  
cc -- Get core memory
      ix1=1
      ix2=ix1+nstmq*nstmq
      ix3=ix2+nstmq*nstmq
      need=ix3


      call gethdia(nrcsf,e_mcqdpt,nstmq,nlist,list_group,
     &             list_csf,dcicoef,dcicoef_ref,x(ix1),x(ix2),
     &             ngrstate)                                      !RV 9/21/06


c
c Punch out CSF list, CI coefficients of 'truncated' Diabatic states
c to $CSFDAT (See subroutine tmset)
c

      call pucsfdia(nstmq,nlist,dcicoef,list_csf,list_group,
     &               maxlist,maxci)


cc Reset core memory

      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)

c
c ============= diabatization procedure was done =============
c


c
c ======= Clearning up of the MC-QDPT diabatization procedure ======
c


c
c Switch back DA10 #125 to #15
c

cc Get core memory

      n1=num*num   
      ix1=1
      last=ix1+n1
      
      call cmem(loadcm)
      need=last
      call setc(need)

      call daread(idaf,ioda,x,n1,125)
      call dawrit(idaf,ioda,x,n1,15,nav)

cc Reset core memory
      call cmem(ngotcm)
      if (ngotcm.ne.loadcm) call setc(loadcm)
  
!
!---- Now av Fock orbitals for original CASSCF calculation
!     returns to DA10 #15  -----------
!

c
c Symmetry turned ON 
c 
      if (offsym) then 
        call symon 
        hndtyp=typsav
        offsym=.false.
        nstate=nstcas
      end if

c
c One more CI calculation with -MC- orbitals for consistency
c
      final=.false.
      mcanci=.false.
      mcnoci=.false.
      mlmoci=.false.
      
      ift12=1
      ift13=1
      ift14=1

      call gugdrt
  
      if (focas.ne.0) call castrf
      if (focas.eq.0.and.mctrf.eq.1) call trfint
      if (focas.eq.0.and.mctrf.ne.1) call trfmcx
   
      call srtgug
      call gugaem
      call gugadg
      call gug2dm

c
c Close -DIIS- work file if it was used
c

      if (focas.ne.0) then
        call raclos(idaf30,nav30,iw)
        call rartrn(idaf30)
      end if



      return


      end


      subroutine wtfuldia(nft,L1,w)
      implicit real*8 (a-h,o-z)
      dimension w(L1)
      write(nft) w
      return 

      end 

